/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/



 /*+----------------------------------------------------------------+
  | Name:     SASMacros/adhoc.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  Performs an adhoc data extract for the specified date
  |           range and resource category/resource type.  Will also, 
  |           optionally, upload extracted data to CAS.  Can also 
  |           update the inventory table tracking the presence of TSV
  |           files for a given resource type.
  |
  |
  |
  | History:  Date    action
  |           31AUG17 Initial Coding                           sasgzs
  |
  | Future:
  | End
  +----------------------------------------------------------------+*/

%macro adhoc_load(start_datetime=,              /* start of period of interest (DATETIME value) */
                  end_datetime=,                /* end of period of interest (DATETIME value) */
                  load_mname=,                  /* name of "load" macro to call */
                  data_dir=,                    /* data directory */
                  resource_category=,           /* resource category */
                  resourcetype=,                /* resource type */                  
                  namematchrx=,                 /* regex for file names to load */
                  update_inventory=N,           /* Y|N flag; update inventory with last rec read info? */
                  add_new_tsv_to_inventory=N,   /* Y|N flag; if new TSV files are found, add to inventory? */                  
                  update_cas=,                  /* Y|N flag; update CAS table */
                  caslib=,                      /* CASLIB of target CAS table or search index */
                  casmemname=,                  /* name of target CAS table or search index */
                  promote=Y,                    /* Y|N flag; promote target CAS table */
                  dropfirst=Y                   /* Y|N flag; always drop target CAS table (i.e. even if no matching data is found) */
                  );                 



 %if "&update_cas" eq "Y" and "&dropfirst" eq "Y" %then
 %do; /* drop the output table */
   
     %get_cas_session();
   
     proc cas;
      %droptable(caslib=&caslib,
                 table=&casmemname);
     quit;
   
 %end;/* drop the output table */
   
  
 %if %upcase("&add_new_tsv_to_inventory") eq "Y" %then %let action_list=%str('ADD','UPDATE'); 
 %else %let action_list="UPDATE"; 
 
 %let start_date=%sysfunc(datepart(&start_datetime));
 %let end_date=%sysfunc(datepart(&end_datetime));

 %if %sysfunc(fileexist(&data_dir)) eq 0 %then 
 %do; /* confirm specified data directory exists */

    %put %str(NOTE: The data directory [&data_dir] for this resource type does not exist; nothing to load.); 
    %put;
    %put;
    %return; 

 %end;/* confirm specified data directory exists */

 %local _ANYDATA;

 %let _ANYDATA=0;
 
 %get_tsv_file_list(data_dir=&data_dir,
                    out=tsvfiles,
                    start_date=&start_date,
                    end_date=&end_date,
                    namematchrx=&namematchrx);   

 %getnobs(data=work.tsvfiles,mvar=_ANYDATA,stat=ANY);


 %if &_ANYDATA gt 0 %then 
 %do; /* matching data found */
 
    filename adldcode temp;

    data _null_;
     length line $2000;
     set tsvfiles;
    
     file adldcode;


     put '%let EVDMERR=N;';
     put '%let atobs=0;';

     line=catt('%',"&load_mname",'(inputtsv=',filename,
               ',firstobs=1',
               ",resource_category=&resource_category",
               ",resourcetype=&resourcetype",
               ",update_cas=&update_cas",
               ',caslib=&caslib',
               ',casmemname=&casmemname',   
               ');'); 
     put line;

     %if %upcase("&UPDATE_INVENTORY") eq "Y" %then
     %do; /* need to update inventory with last record read info */
  
        if update_action in (&action_list) then
        do; /* need to update inventory */
   
           line=catt('%update_rtinventory(action=',update_action,
                     ', inventory=rt.inventory, tsv_file_name=',tsv_file_name,
                     ', file_date=', put(file_date,best.),
                     ', file_ext=', file_ext,
                     ', last_rec=&atobs, last_moddate=',modfdt,
                     ', finalized=',finalized,');');
                
           put line;
       
        end;/* need to update inventory */
     
     %end;/* need to update inventory with last record read info */
  
    run;
 
   
    /* call generated code */
    /* to load data.       */
    %include adldcode /source2;
 
 
    %if "&update_cas" eq "Y"  and "&promote" eq "Y" %then
    %do; /* promote the output table */
    
         %get_cas_session();
    
         proc cas;
          %caspromote(caslib=&caslib,
                      table=&casmemname);
         quit;
    
    %end;/* promote the output table */

 %end;  /* matching data found */    
 %else
 %do; /* no matching TSVs found */
 
     %put NOTE: No data for the specified resource [&resource_category.:&resourceType.] and time period [%sysfunc(putn(&start_datetime,datetime.)) - %sysfunc(putn(&end_datetime,datetime.))] was found%str(;) nothing to load.;     
 
  %end;/* no matching TSVs found */
   

 /* clean-up after ourselves*/
 proc datasets nolist lib=work;
  delete tsvfiles;
  run;
 quit;

%mend adhoc_load;

