/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/



 /*+----------------------------------------------------------------+
  | Name:     adhoc_load2.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  Loads latest x days of data from PROCESSED files into CAS 
  |
  |
  | Inputs    PROCESSED TSVs
  |
  |
  | History:  Date    action
  |           05Apr17 Initial Coding                              snornb
  |           
  | Future:
  | End
  +----------------------------------------------------------------+*/

%macro adhoc_load2(start_datetime=,
                   end_datetime=,
                   load_mname=,
                   data_dir=,
                   namematchrx=,
                   resource_category=,     /* Resource category */
                   resourcetype=,          /* Resource type */
                   update_cas=,            /* Y|N flag; update CAS table */
                   promote=Y,              /* Y|N flag; promote target CAS table */
                   dropfirst=Y,            /* Y|N flag; always drop target CAS table (i.e. even if no matching data is found) */
                   caslib=,                /* CASLIB of target CAS table or search index */
                   casmemname=);           /* name of target CAS table or search index */


 %if %sysfunc(fileexist(&data_dir)) eq 0 %then
 %do; /* confirm specified data directory exists */

    %put %str(NOTE: The specified data directory [&data_dir] does not exist; skipping this directory);
    %let _nodata=Y;
    %return;

 %end;/* confirm specified data directory exists */


 %let start_date=%sysfunc(datepart(&start_datetime));
 %let end_date=%sysfunc(datepart(&end_datetime));
 
 libname rt "&data_dir";

 /* need to check whether path exists */
 %findfiles(dir=&data_dir,
           outds=work._foundfiles,
           namematchrx=&namematchrx);
           
 filename adldcode temp;

 %let _ANYDATA=N;

 data _null_;
  set work._foundfiles(rename=(logfile=tsv_file_name));
  
  length line $4000;
  
  re=prxparse("/(((\d{4})(\d{2})(\d{2}))?(?:(\w+)?)-(\S+)(?:\.(tsv)))(?:\.(\w+))?/");
  match=prxmatch(re,tsv_file_name);

  file_name=prxposn(re,1,tsv_file_name);
  file_date=input(prxposn(re,2,tsv_file_name),yymmdd8.);
          /*YEAR: 3 */
          /*MONTH: 4 */
          /*DAY: 5 */
  file_type=prxposn(re,6,tsv_file_name);
  vstring  =prxposn(re,7,tsv_file_name);
  file_ext =prxposn(re,8,tsv_file_name);
  file_zip =prxposn(re,9,tsv_file_name);

  if file_date ge &start_date and file_date le &end_date;

  put line;

  call symputx("_ANYDATA","Y",'F');   /* FLAG INDICATING THERE WAS DATA */
    
  filename=catt("&data_dir",'/',tsv_file_name);

  file adldcode;

  put '%let EVDMERR=N;';
  put '%let atobs=0;';
  
  line=catt('%',"&load_mname",'(inputtsv=',filename,
            ',firstobs=1',
            ",resource_category=&resource_category",
            ",resourcetype=&resourcetype",
            ",update_cas=&update_cas",
            ',caslib=&caslib',
            ',casmemname=&casmemname',   
            ');'); 
  put line;
  putlog line;
   
 run; 

 %if "&dropfirst" eq "Y" %then
 %do; /* drop the output table */

    %get_cas_session();

    proc cas;
     %droptable(caslib=&caslib,
                table=&casmemname);
    quit;

 %end;/* drop the output table */


 /* call generated code */
 /* to load data.       */
 %include adldcode /source2;


 %if "&promote" eq "Y" and "&_ANYDATA" eq "Y" %then
 %do; /* promote the output table */

    %get_cas_session();

    proc cas;
     %caspromote(caslib=&caslib,
                 table=&casmemname);
    quit;

 %end;/* promote the output table */


%mend adhoc_load2;

