/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

 /*+----------------------------------------------------------------+
   | Name:     burst_kvpmap.sas                                     |
   | Product:  SAS Environment Manager                              |
   |                                                                |
   | Purpose:  Extracts all key-value pairs from the specified map. |
   |           For each KVP found within the map, the key name and  |
   |           value will be assigned to the specified string vars. |
   |                                                                |
   | Notes:   *Lengths for the key name and value variables should  |
   |           set in the calling program.                          |
   |          *This macro removes an outer set of double quotes, if |
   |           one exists, for the property values.                 |
   |                                                                |
   | History:  Date    action                                       |
   |           31JUL17 Initial Coding                        sasgzs |
   |                                                                |
   +----------------------------------------------------------------+*/
  
%macro burst_kvpmap(map=,                 /* SAS char variable containing the "map" */
                    namevar=,             /* SAS char variable to which to assign extracted key's name */
                    valuevar=,            /* SAS char variable to which to assign extracted key's value */
                    kv_delim=%str(:),     /* delimiter character used between key and value */
  					item_delim=%str(,));  /* delimiter character used between map items */

  length _bk_kvp $32000;
  retain _bk_re _bk_dequote_re;
  
  if _n_ eq 1 then
  do; /* initialize regular expression patterns */

     _bk_pattern=catt("/",                              /* regex pattern - start */                    
                      '(',                              /* KEY NAME capture group - start */
                      '[A-Za-z0-9-_]*',                 /* KEY NAME */
                      ')',                              /* KEY NAME capture group - end */
                      "&kv_delim",                      /* key/value delimiter */
                      "(",                              /* KEY VALUE capture group - start */
                      "(?:[A-Za-z0-9+\/])*(?:={0,2})",  /* KEY VALUE */
                      ')',                              /* KEY VALUE capture group - end */
                      "&item_delim*",                   /* item delimiter */
                      "/oi");                           /* regex pattern - end + flags */
                                          
      *_bk_pattern="/([A-Za-z0-9-_]*)&kv_delim((?:[A-Za-z0-9+\/])*(?:={0,2}))&item_delim*/oi";
      _bk_re=prxparse(_bk_pattern);	
      
      _bk_dequote_re=prxparse('/^"(.*)"\s*/o');
      
  end;/* initialize regular expression patterns */

  _bk_start = 1;
  _bk_stop = length(&map);
  
  call prxnext(_bk_re, _bk_start, _bk_stop, &map, _bk_pos, _bk_len);
  
  do while (_bk_pos > 0);
  
     _bk_kvp = substr(&map, _bk_pos, _bk_len);
     
     if prxmatch(_bk_re, _bk_kvp) then
     do; /* parse KVP */
    
        &namevar=prxposn(_bk_re,1,_bk_kvp);
        
        &valuevar=input(prxposn(_bk_re,2,_bk_kvp),$base64x3200.);       
        if prxmatch(_bk_dequote_re,&valuevar) then &valuevar=prxposn(_bk_dequote_re,1,&valuevar);
        
        output;    
     end;/* parse KVP */
     
     call prxnext(_bk_re, _bk_start, _bk_stop, &map, _bk_pos, _bk_len);
     
  end; /* do-while */
 
  drop _bk_re  _bk_start _bk_stop _bk_pos _bk_len _bk_kvp _bk_pattern _bk_dequote_re;

%mend burst_kvpmap;