%macro calc_rates(indata=,file=);
/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

 
/*+----------------------------------------------------------------+
  | Name:     calc_rates.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  1. Calculates rates for counter type variables
  |
  |           
  |
  | Inputs    evdm.metricproperties
  |           evdm.whitelist
  |           
  |
  | History:  Date    action
  |           30Mar17 Initial Coding                              frf
  |           02MAY17 Change aux.whitelist to evdm.whitelist   sasgzs
  |           03May17 This macro now creates work.has_counters
  |                   instead of in GET_TSV_FILE_VERSION.
  |           04May17 Changed aux.lastvals to lastvals.lastvals   frf
  |           14Jun17 Changed rate missing values to . (normal missing value)   frf
  |           16Nov17 Keep only the last hour of lastvals tables                frf
  |
  | Future:
  | End
  +----------------------------------------------------------------+*/
 /*
   Get the metrictablename, based on resourceType. Note that the metrictablename must be a valid SAS
   table name. For example:
         resourceType           metrictablename
         ------------           ---------------
         cpu                    cpu
         system                 system
         System:usage           system_usage
         System:process         system_process

  */

 %LET metrictablename= ;
 %LET resource_type= ;
 data _null_;set evdm.whitelist;
                 if lowcase(resourcetype)="%lowcase(&indata)" then do;
                                 call symput('metrictablename',metrictablename);
                                 call symput('resource_type',resourcetype);
                                 stop;
                 end;
 run;

  %let metrictablename=%cmpres(&metrictablename);
 /*                                             */
 /* Each resourcetype will have a unique hash   */
 /* table name. The name will be the macro      */
 /* variable METRICTABLENAME                    */
 /*                                             */
 /* evdm.whitelist has variables                 */
 /* resourcetype and metrictablename. Create a  */
 /* macro var, METRICTABLENAME from the         */
 /* resourcetype to metrictablename mapping     */
 /*                                             */

 %put In macro calc_rates, metrictablename=&metrictablename;


 /*                                                          */
 /* Find variables of type=counter. These will have rates    */
 /* calculated.                                              */
 /*                                                          */

%local version_full;
%parse_file_version(file=&file);

%put In calc_rates version_full=&version_full;

proc sort 
	data=evdm.metricproperties(where=(lowcase(metrictype)='counter' 
							and lowcase(resourcetype)=%cmpres(%lowcase("&resource_type"))
							and lowcase(version_full)=%lowcase("&version_full")))
    out=Has_counters nodupkeys;
	by resourcetype metricname;
run;

%let has_counters=n;

%put In calc_rates resource_type=&resource_type;

 data _null_;set has_counters;
  length divisor 8.;
   
  if lowcase(resourcetype)="%lowcase(&resource_type)" then do;
  /*                                                */
  /* Find divisor when calculating a rate. Rates    */
  /* are rates per second                           */
  /*  s=second      divisor=1                       */
  /*  ms=milisecond divisor=1000                    */
  /*  ns=nanosecond divisor=1000000000              */
  /*  min=min divisor=1/60                          */
  /*  hour=hour divisor=1/3660                      */
  /*                                                */
  /* B=byte=1/1                                     */
  /* Kb=1/1024                                      */
  /* Mb=1/1048576                                   */
  /* Gb=1/1073741824                                */
  /*                                                */

  /*                                                */
  /* Default divisor=1 for counter type metrics.    */
  /*                                                */
 
  divisor=1;
   if lowcase(unit)='s' then divisor = 1;
     else if lowcase(unit)='ms' then divisor = 1000;
     else if lowcase(unit)='ns' then divisor = 1000000000;
     else if lowcase(unit)='min' then divisor = 1/60;
     else if lowcase(unit)='hour' then divisor = 1/3600;
     else if lowcase(unit)='b' then divisor = 1;
     else if lowcase(unit)='kb' then divisor = 1/1024;
     else if lowcase(unit)='mb' then divisor = 1/1048576;
     else if lowcase(unit)='gb' then divisor = 1/1073741824;

   rate=compress(metricname) || '_rate';
    counter_vars+1;
     call symput('counter_var_unit' || left(trim(counter_vars)),unit);
     call symput('counter_var' || left(trim(counter_vars)),metricname);
     call symput('counter_var_rate' || left(trim(counter_vars))  ,rate);
     call symput('counter_var_divisor' || left(trim(counter_vars))  ,divisor);
     call symput('counter_vars',left(trim(counter_vars)));
     call symput('has_counters','y');
  end;  /* if resourcetype=   */
run;

 /*                                                          */
 /* Stop processing if there are no counter type variables   */
 /* in this data.                                            */
 /*                                                          */

  %if &has_counters=n %then %return;

 /*                                             */
 /* In order to keep last lastvals unique,      */
 /* prepend the METRICTABLENAME to the string   */
 /* lastvals. Possible values are               */
 /* cpu_lastvals                                */
 /* system_lastvals                             */
 /* filesystem_lastvals  etc, etc               */
 /*                                             */

 /*%let lastvals=lastvals;*/
 %let lastvals=%cmpres(&metrictablename); 


 %macro initTable();
  %if %sysfunc(exist(lastvals.&lastvals)) %then %do;%end; %*return;
  %else
  %do;

  %put NOTE: Initilizing table;

  data lastvals.&lastvals;
   length resourceType $32. resourceID $ 200;
   length  lastDatetime metricValue 8 metricName $50;
   stop;
  run;
 %end;

 %mend initTable;

 %initTable;
	
%let suf=_cnt;
  
  data &indata ;
  length lastValue lastDatetime metricValue 8 metricName $50;

  set work.&indata end=lastrec;
  
 	/*                                                            */
 	/* Calculate CPU percentages                                  */
 	/*   1. total_cpu_pct - Overall CPU percent busy              */
 	/*	 2. user_cpu_pct  - Percent of the CPU used by user pgms  */
 	/*   3. idle_cpu_pct  - Percent of the time the CPU is idle   */
 	/*   4. system_cpu_pct- Percent of the CPU used by the system */
 	/*                                                            */
 	
 %if &indata = system %then %do;
 	length last_totalcpu last_systemcpu last_usercpu last_idlecpu 8.;
	drop last_totalcpu last_systemcpu last_usercpu last_idlecpu;
 %end;
 
 
   rename
		%do m=1 %to &counter_vars;
		 %let cmp=%cmpres(&&counter_var&M)&suf;		 
			&&counter_var&m = &cmp
			&&counter_var_rate&m = &&counter_var&m
		%end;
		;
    
  array _divisor {&counter_vars}
      _temporary_
       (
          %do m=1 %to &counter_vars;
              &&counter_var_divisor&m
          %end;
       )

          ;  
  
  array _rateVars {*}
          %do m=1 %to &counter_vars;
              &&counter_var&m
          %end;
          ;

  array _newRateVars {*}
          %do m=1 %to &counter_vars;
              &&counter_var_rate&m
          %end;
          ;
  array _rateVarName {&counter_vars} $16. _temporary_
         (
          %do m=1 %to &counter_vars;
             %cmpres("&&counter_var&m")
          %end;
          );
  if _n_ eq 1 then
  do; /* initialize hash */
    declare hash &metrictablename.(dataset:"lastvals.&lastvals");
    rc = &metrictablename..defineKey('resourceType','resourceId','metricName');
    rc = &metrictablename..defineData('resourceType','resourceId','metricName','lastDatetime','metricValue');
    rc = &metrictablename..defineDone();
  *  call missing(resourcetype,resourceid,metricname,lastdatetime,metricvalue);
  end;/* initialize hash */


/*
 * get last totalcpu from hash => last_total_cpu
 * get last idlecpu => last_idle_cpu
*/
  %if &indata = system %then %do;
    metricName = 'TotalCpu';
    
    rc = &metrictablename..find();
    if (rc eq 0) then
    do; 
       last_totalcpu=metricValue;      
    end;
    
    metricName = 'IdleCpu';
    
    rc = &metrictablename..find();
    if (rc eq 0) then
    do; 
       last_idlecpu=metricValue;       
    end;
    
    metricName = 'UserCpu';
    
    rc = &metrictablename..find();
    if (rc eq 0) then
    do; 
       last_usercpu=metricValue;       
    end;
    
    metricName = 'SystemCpu';
    
    rc = &metrictablename..find();
    if (rc eq 0) then
    do; 
       last_systemcpu=metricValue;      
    end;
  %end;
    



  do i = 1 to dim(_rateVars);
    metricName = _ratevarName(i);
    rc = &metrictablename..find();
    if (rc eq 0) then
    do; /* key found */
	
       lastValue=metricValue;

           if (datetime le lastDatetime) then _newRateVars(i) = .;
           if (datetime - lastDatetime) lt 3600 and (datetime gt lastDatetime) then
              _newRateVars(i) = ((_ratevars(i) - lastValue)/ (datetime - lastDatetime))/_divisor(i);
           else _newRateVars(i) = .;
           	  _newRateVars(i)=round(_newRateVars(i),.000000001);

/*
 if metricName eq  in usercpu, idle_cpu, etc then
 calculate the pct
 */

 %if &indata = system %then %do;
 select(metricName);
  when("UserCpu") 	do;
  						user_cpu_pct=100*(usercpu-last_usercpu)/(totalcpu-last_totalcpu);
  						user_cpu_pct=round(user_cpu_pct,.000000001);  						
  					end;
  					
  when("SystemCpu") do;
  						system_cpu_pct=100*(systemcpu-last_systemcpu)/(totalcpu-last_totalcpu);
  						system_cpu_pct=round(system_cpu_pct,.000000001);  						
  					end;
  					
  when("IdleCpu") 	do;
  						idle_cpu_pct=100*(idlecpu-last_idlecpu)/(totalcpu-last_totalcpu);
  						idle_cpu_pct=round(idle_cpu_pct,.000000001);
    				end;
  					
  when("TotalCpu") 	do;
  						total_cpu_pct=100*((totalcpu-last_totalcpu)-(idlecpu-last_idlecpu))/(totalcpu-last_totalcpu);
  						total_cpu_pct=round(total_cpu_pct,.000000001);  						
  					end;
  					
  otherwise;
 end;
 %end;

 

           lastDatetime = datetime;
           metricValue = _ratevars(i);

           mod_rc = &metrictablename..replace();
    end;/* key found */
    else
    do; /* key not found */

       _newRateVars(i)=.;

       lastDatetime = datetime;
           metricValue = _ratevars(i);
       add_rc = &metrictablename..add();

       if (add_rc ne 0) then put 'Add failed.';

    end;/* key not found */

  end; /* i-loop */

  if lastrec then
  do; /* write out last values table */

     rc = &metrictablename..output(dataset:"lastvals.&lastvals");

  end;/* write out last values table */

  drop lastValue lastDatetime metricname metricvalue i add_rc mod_rc rc;

 run;	

/*                                                      */
/*  Keep only the last hour of data. If this interval is*/
/*  changed (e.g. to 2 hours) also change the interval  */
/*  in the calculation step above.                      */
/*  if (datetime - lastDatetime) lt 3600 would become   */
/*  if (datetime - lastDatetime) lt 7200                */
/*                                                      */
proc sort data=lastvals.&lastvals. out=lastvals.&lastvals.(compress=yes);
 where lastDatetime ge intnx('dthour', TZONES2U(datetime()), -1, 'same');   /* @ 17:12; cutoff is 16:12 */
by resourceid metricName lastDatetime;
run;
%mend calc_rates;

