
/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/


 /*+----------------------------------------------------------------+
  | Name:     check_output_tsv_version.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  Appends to evdm.evdm_versions output TSV variables and
  |			  lengths if info for this "output" version format does 
  |			  not exist in evdm.evdm_versions
  |           
  | Inputs    evdm.evdm_versions
  | 
  |
  | History:  Date    action
  |           14May17 Initial Coding                              frf
  |           30MAY17 Revised to more closely align with       sasgzs
  |                   %CHECK_VERSION macro
  |           19JUN17 Added VARLIST parameter and logic        sasgzs
  | Future:
  | End
  +----------------------------------------------------------------+*/
%macro check_output_tsv_version(indata=,
                                varlist=,
				resource_category=,
				resourcetype=,
				version=,
				major=,
				minor=,
				patch=);

 %let evdmerr=N;

 %local where;
 
 %if "&version" ne "" %then %let where=%str(evdm_version="&version" and);
 %else %let where= %str(version_major eq &major and version_minor eq &minor and version_patch eq &patch and);

 %let known_version=0;

 %if %sysfunc(exist(EVDM.EVDM_VERSIONS)) %then
 %do;
    proc sql noprint;
     select count(*)
      into :known_version
       from evdm.evdm_versions
        where &where
              type eq "output" and
              resource_category eq "&resource_category" and
              resource eq "&resourceType";
    quit;
 %end;

 %if &known_version gt 0 %then
 %do; /* known file version */
    %put %str(INFO: Information on this output file format already exists; no further processing needed.);
    %return;
 %end;/* known file version */
 %else
 %do; /* unknown version */

    %if "&varlist" ne "" %then %let varnum_line=indexw(upcase("&varlist"), upcase(name)) as varnum;
    %else %let varnum_line= varnum as varnum;
    
    proc sql noprint;
     create table new_output_version as 
      select memname,
             name,
             length,
             type,
             &varnum_line
        from dictionary.columns
         where libname="WORK" and
               memname eq upcase("&indata")
           order by varnum;
    quit; 
 
   %local version_full version_major version_minor version_patch;
   %parse_semver(vstring=&version);
    
   data new_output_version;
    set new_output_version(rename=(type=c_type));
    
    length resource resource_category $32 evdm_version $16. lgth $8. type $8.
	   version_major version_minor version_patch 8. var var_type $200.;
		   
    keep resource resource_category evdm_version lgth  type 
         version_major version_minor version_patch var var_type ;
		   
    evdm_version="&version";
    resource="&resourcetype";
    resource_category="&resource_category";
    type='output';
    version_major=&version_major;
    version_minor=&version_minor;
    version_patch=&version_patch;
    var=name;
    var_type='';
    if c_type="num" then lgth='8.';
    else if c_type="char" then lgth=compress('$' || length);
   run;
   
   proc append base=evdm.evdm_versions data=new_output_version;
   run;
   
 %end;/* unknown version */   
%mend check_output_tsv_version;
