/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

 /*+----------------------------------------------------------------+
   | Name:     confirm_cas_table_init.sas                           |
   | Product:  SAS Environment Manager                              |
   |                                                                |
   | Purpose:  Checks that the specified CAS table exists.  If the  |
   |           CAS table does NOT exist, initialize it by calling   |
   |           the specified job.                                   |
   |                                                                |
   |   Value | Meaning (related action)                             |
   |  -------+----------------------------------------------------- |
   |       0 | Table exists                                         |
   |      93 | Specified init job does NOT exist                    |
   |      94 | Table does not exist AFTER running init job.         |
   |                                                                |
   | History:  Date    action                                       |
   |           28JUN17 Initial Coding                        sasgzs |
   |           03AUG17 Added abortonfail, return codes       sasgzs |
   |                                                                |
   +----------------------------------------------------------------+*/

%macro confirm_cas_table_init(caslib=,                    /* CASLIB that should contain the table of interest */
                              table=,                     /* Name of table to validate/initialize */
                              job_dir=&EVHOME/SASCode,    /* Directory containing the initialization job */
                              initjob=,                   /* Job that will initialize the table if it does not exist */
                              msglevel=ERROR,             /* Message level to use for "errors"; e.g. ERROR or WARNING */
                              abortonfail=N);             /* ABORT SAS session if table can't be initialized */


 %get_cas_session(abortonfail=&abortonfail);
 
 %let exists=0; /* initialize exists flag */

 proc cas;
  %tableExists(caslib=&caslib, table=&table);
  if rc then call symputx("EXISTS","1","F");
  else call symputx("EXISTS","0","F");
 quit;

 %if NOT &exists %then 
 %do; /* table does not exist */

    %put DEBUG: Attempting to initialize the table [&caslib..&table] by calling job [&job_dir/&initjob].;

    %if %sysfunc(fileexist(&job_dir/&initjob)) %then
    %do; /* specified initialization job exists */ 

       %include "&job_dir/&initjob";

       /* (re)open a CAS session; in case the */
       /* initjob closes the CAS connection.  */
       %get_cas_session(abortonfail=&abortonfail);

       /* confirm table exists after running the job */
       proc cas;
        %tableExists(caslib=&caslib, table=&table);
        if rc then call symputx("EXISTS","1","F");
        else call symputx("EXISTS","0","F");
       quit;

       %if NOT &exists %then 
       %do; /* table still does not exist */

          %put &msglevel.: Unable to initialize the table [&caslib..&table].;
          %let error_code=94;
          %goto EXIT_ERROR;

       %end;/* table does not exist */

    %end;/* specified initialization job exists */ 
    %else
    %do; /* specified initialization job does NOT exists */ 

       %put &msglevel.: The SAS job [&job_dir/&initjob] needed to initialize the table [&caslib..&table] was not found.;
       %let error_code=93;
       %goto EXIT_ERROR;

    %end;/* specified initialization job does NOT exists */ 
    
 %end;/* table does not exist */
 %else %put %str(DEBUG: CAS Table [&caslib..&table] exists.);;


%EXIT:
  %let error_code=0;
  %return;

 
%EXIT_ERROR:
  %if %upcase("&abortonfail") eq "Y" %then %goto EXIT_ABORT;     
  %return;

%EXIT_ABORT:
  %datamartlock(action=RELEASE); /* release lock on data mart if it exists */
  %put %str(ERROR: Terminating session because the CAS table [&caslib..&table] could NOT be initialized (rc=[&error_code]).);
  %abort return &error_code;
  %return;
 
%mend confirm_cas_table_init;
