/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

 /*+----------------------------------------------------------------+
   | Name:     confirm_opsdm_propsfile_init.sas                     |
   | Product:  SAS Environment Manager                              |
   |                                                                |
   | Purpose:  Checks that the data mart properties are defined for |
   |           the specified data mart, and, optionally, to         |
   |           initialize them if they have not been defined.       |
   |                                                                |
   |   Value | Meaning (related action)                             |
   |  -------+----------------------------------------------------- |
   |       0 | Properties exist.                                    |
   |      92 | Unable to define data mart properties                |
   |                                                                |
   | History:  Date    action                                       |
   |           10AUG17 Initial Coding                        sasgzs |
   |                                                                |
   +----------------------------------------------------------------+*/


%macro confirm_opsdm_propfile_init(datamart=,                     /* name of Data Mart */
                                   propfilename=opsdm.properties, /* name of property file */
                                   abortonfail=N);                /* ABORT SAS session if property file can't be initialized */

 %if "&datamart" ne "" %then 
 %do; /* data mart specified */

    %let path=%str(&evconf./var/lib/evmsvrops/&datamart);

 %end;/* data mart specified */
 %else 
 %do; /* use current data mart */

    %let path=&evdmpath;
    %let datamart=&evdm;
    
 %end;/* use current data mart */

 %if %sysfunc(fileexist(&path)) %then 
 %do; /* path found */

    %if %sysfunc(fileexist(&path./&propfilename)) %then 
    %do; /* property file found */
   
       %put %str(DEBUG: Datamart property file [&path./&propfilename] exists.); 
       %goto EXIT;
       
    %end;/* property file found */
    %else
    %do; /* initialize property file */
   
      %macro x(propname=,default=);
      
       %if %symexist(&propname) %then
       %do; /* default value for property exists */
          putlog "DEBUG: Setting property [&propname] to current default value [&&&propname]";
          put "&propname=&&&propname";
       %end;/* default value for property exists */
       %else
       %do; /* NO default value for property exists */
          putlog "DEBUG: No existing value for [&propname] is set; omitting this property from property set.";  
       %end;/* NO default value for property exists */
       
      %mend x;
      
      data _null_;
       file "&path./&propfilename";
       %x(propname=datamart);
       %x(propname=cas_cutoff);
       %x(propname=zip_tsv_and_log_delete);
      run; 

      %if %sysfunc(fileexist(&path./&propfilename)) %then 
      %do; /* property file created */
   
        %put %str(DEBUG: Datamart property file [&path./&propfilename] created.); 
        %goto EXIT;
       
      %end;/* property file created */
      %else
      %do; /* property file NOT created */
     
         %let error_code=91;
         %goto EXIT_ERROR;
         
      %end;/* property file NOT created */
       
    %end;/* initialize property file */   
 %end;/* path found */
 %else
 %do; /* path doesn't exist */
    %let error_code=92;
    %goto EXIT_ERROR;
 %end;/* path doesn't exist */


%EXIT:
  %let error_code=0;
  %return;

 
%EXIT_ERROR:
  %if %upcase("&abortonfail") eq "Y" %then %goto EXIT_ABORT;     
  %return;

%EXIT_ABORT:
  %datamartlock(action=RELEASE); /* release lock on data mart if it exists */
  %put %str(ERROR: Terminating session because the data mart properties set (&path./&propfilename) could not be found or created. (rc=[&error_code]).);
  %abort return &error_code;
  %return;
 

%mend confirm_opsdm_propfile_init;
