/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

 /*+----------------------------------------------------------------+
   | Name:     confirm_sas_table_init.sas                           |
   | Product:  SAS Environment Manager                              |
   |                                                                |
   | Purpose:  Checks that the specified SAS table exists.  If the  |
   |           CAS table does NOT exist, initialize it by calling   |
   |           the specified job.                                   |
   |                                                                |
   |   Value | Meaning (related action)                             |
   |  -------+----------------------------------------------------- |
   |      95 | Specified init job does NOT exist                    |
   |      96 | Table does not exist AFTER running init job.         |
   |                                                                |
   | History:  Date    action                                       |
   |           28JUL17 Initial Coding                        sasgzs |
   |                                                                |
   +----------------------------------------------------------------+*/

%macro confirm_sas_table_init(libname=,                    /* LIBNAME that should contain the table of interest */
                              table=,                      /* Name of table to validate/initialize */
                              job_dir=&EVHOME/SASCode,     /* Directory containing the initialization job */
                              initjob=NoneSpecified,       /* Job that will initialize the table if it does not exist */
                              abortonfail=N);              /* ABORT SAS session if table can't be initialized */


 %if %sysfunc(exist(&libname..&table)) %then  
 %do;  /* table exists */
 
     %put %str(DEBUG: Table [&libname..&table] exists.);
     %goto EXIT;

 %end;  /* table exists */
 %else
 %do; /* table does NOT exist; attempt to initialize it */

    %put %str(DEBUG: Attempting to initialize the table [&libname..&table] by calling job [&job_dir/&initjob].);
    
    %if %sysfunc(fileexist(&job_dir/&initjob)) %then
    %do; /* specified initialization job exists */ 

       %include "&job_dir/&initjob";
       
       %if %sysfunc(exist(&libname..&table))=0 %then 
       %do; /* table still doesn't exist */
       
          %put %str(ERROR: Unable to unitialize table [&libname..&table].);
          %let error_code=96;
          %goto EXIT_ERROR;
       
       %end;/* table still doesn't exist */
       %else
       %do; /* table created */
       
          %put (NOTE: Table [&libname..&table] has been initialized.);
          %goto EXIT;
          
       %end;/* table created */
       
    %end;/* specified initialization job exists */ 
    %else
    %do; /* specified initialization job does NOT exists */ 

       %put %str(ERROR: The SAS job [&job_dir/&initjob] needed to initialize the table [&libname..&table] was not found.);
       %let error_code=95;
       %goto EXIT_ERROR;

    %end;/* specified initialization job does NOT exists */ 
    
 %end;/* table does NOT exist; attempt to initialize it */


%EXIT:
  %return;
 
%EXIT_ERROR:
  %if %upcase("&abortonfail") eq "Y" %then %goto EXIT_ABORT;     
  %return;

%EXIT_ABORT:
  %datamartlock(action=RELEASE); /* release lock on data mart if it exists */
  %put %str(ERROR: Terminating session because the table [&libname..&table] could NOT be initialized (rc=[&error_code]).);
  %abort return &error_code;
  %return;
  
%mend confirm_sas_table_init;
