/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

 /*+----------------------------------------------------------------+
   | Name:     datamartlock.sas                                     |
   | Product:  SAS Environment Manager                              |
   |                                                                |
   | Purpose:  Locks or unlocks the specified data mart lock file.  |
   |           The data mart lock file is intended to prevent jobs  |
   |           from interferring with each other by signalling that |
   |           the need exclusive access to the data mart.  This is |
   |           generally because they are updating data mart tables.|
   |           In addition, this macro can report on the status of  |   
   |           data mart lock file.                                 |
   |                                                                |
   | Global Macro Variables: This macro uses uses the following     |
   |           global macro variables                               |
   |                                                                |
   |                                                                |
   |           DATAMART_LOCK_HELD - "flag" indicating that this     |
   |                   currently has the data mart locked.          |
   |                                                                |
   |           DATAMART_LOCK_RC - used to pass a return code to the |
   |                   calling program.                             |
   |                                                                |
   |   Value | Meaning (related action)                             |
   |  -------+----------------------------------------------------- |
   |       0 | Action completed w/o problems                  (ALL) |
   |      -1 | Lock file exists but unable to read contents (CHECK) |
   |      -2 | Datamart not locked;request ignored        (RELEASE) |
   |      -3 | Unable to write to lockfile;but lock obtained (LOCK) |
   |      -4 | Both lock/unlock files exists; treat as locked.(ALL) |
   |      -5 | Data mart not locked; release req ignored  (RELEASE) |
   |     197 | Unable to wipe contents of lockfile        (RELEASE) |
   |     198 | Unable to release our lock                 (RELEASE) |
   |     199 | Release failed; not locked by us           (RELEASE) |
   |     298 | Lock failed; locked by another                (LOCK) |
   |     299 | Lock failed; cannot rename lock file          (LOCK) |
   |                                                                |
   | History:  Date    action                                       |
   |           24MAY17 Initial Coding                        sasgzs |
   |           02JUN17 Revised: new naming convention        sasgzs |
   +----------------------------------------------------------------+*/
  
%macro datamartlock(action=,        /* LOCK|RELEASE|CHECK; action to perform */
                    lockfile=,      /* data mart lock file to work with */
                    abortonfail=N); /* Y|N; Abort session if request fails? */


   %global datamart_lock_rc;
   %let datamart_lock_rc=0;

   %if "&lockfile" eq "" %then %let lockfile=&evdmpath/datamartlock;
   
   %let unlockedfile=%str(&lockfile..false);    
   %let unlocked_exists=%sysfunc(fileexist(&unlockedfile));
   
   %let lockedfile=%str(&lockfile..true);
   %let locked_exists=%sysfunc(fileexist(&lockedfile));
  
   %if &unlocked_exists and &locked_exists %then
   %do; /* both LOCK and UNLOCK file exists */
  
      %let lock_status = LOCKED;
      %put %str(WARNING: Both "locked" and "unlocked" version of the data mart lock file [&lockfile] exists; this should never happen. Assuming the data mart is locked.);
      %let datamart_lock_rc=-4;
      
   %end;/* both LOCK and UNLOCK file exists */
   %else %if not &unlocked_exists and not &locked_exists %then
   %do; /* neither LOCK nor UNLOCK file exists */
  
      %let lock_status = UNLOCKED;
      %put %str(WARNING: Neither "locked" nor "unlocked" version of the data mart lock file [&lockfile] exists.);
      %let datamart_lock_rc=-4;
      %put %str(NOTE: Assuming the data mart is unlocked and (re-)creating the missing "unlocked" version of the data mart lock file [&unlockedfile].);
      %writedmlock(file=&unlockedfile,clear=Y);
      
   %end;/* neither LOCK nor UNLOCK file exists */
   %else %if &unlocked_exists %then %let lock_status= UNLOCKED;
   %else %if &locked_exists %then %let lock_status= LOCKED;
   
   %if %upcase("&action")  eq "RELEASE" or  %upcase("&action")  eq "UNLOCK" %then
   %do; /* release the lock */
      
      %if %symexist(DATAMART_LOCK_HELD) %then
      %do; /* we hold the lock */

         %writedmlock(file=&lockedfile,clear=Y);               
         
         %if &datamart_lock_rc %then
         %do; /* error clearing lock file */
        
            %put %str(ERROR: Unable to clear contents of the data mart lock file; data mart remains locked (lockfile: [&lockfile] sysmsg:[&msg]).);
            %goto ERROR;

         %end;/* error clearing lock file */                
        
         %let rc=%sysfunc(rename(&lockedfile,&unlockedfile,FILE));
         %let msg=%sysfunc(sysmsg());           
         
         %if &rc %then 
         %do; /* problem renaming lock file */
        
            %put %str(ERROR: Unable to release the data mart lock; data mart remains locked (lockfile: [&lockfile] sysmsg:[&msg]).);
            %let datamart_lock_rc=198;
            %goto ERROR;
            
         %end;/* problem renaming lock file */
         %else
         %do; /* lock file renamed */
                        
            %put %str(NOTE: Data mart lock released; data mart no longer locked.);
            %symdel DATAMART_LOCK_HELD;
            
            %let datamart_lock_rc=0;                       
            %goto EXIT;
            
         %end;/* lock file renamed */
      %end;/* we hold the lock */
      %else %if &lock_status eq LOCKED %then
      %do; /* we do NOT hold the lock */
     
         %put %str(ERROR: Request to release lock on data mart failed; you do NOT hold the lock.);        

         %readdmlock(file=&lockedfile);   

         %let datamart_lock_rc=199;
         %goto ERROR;
        
      %end;/* we do NOT hold the lock */        
      %else
      %do; /* it is not locked */
     
         %put %str(NOTE: The data mart lock file [&lockfile] is NOT locked; request to release lock on data mart ignored.);        
         %let datamart_lock_rc=-5;
      
      %end;/* it is not locked */      
   %end;/* release the lock */

   %if %upcase("&action")  eq "LOCK" %then
   %do; /* obtain the lock */
  
      %if %symexist(DATAMART_LOCK_HELD) %then
      %do; /* we hold the lock */
     
         %put %str(NOTE: A lock on data mart has already been obtained; nothing to do (lockfile: [&lockedfile]).);
         
         %let datamart_lock_rc=0;                       
         %goto EXIT;
         
      %end;/* we hold the lock */
      %else
      %do; /* we do NOT hold the lock */
     
         %if &lock_status eq UNLOCKED %then 
         %do; /* "unlock" file found! */
        
            %let rc=%sysfunc(rename(&unlockedfile,&lockedfile,FILE));           
            %let msg=%sysfunc(sysmsg());           

            %if &rc ne 0 %then
            %do; /* unable to rename lock file */
        
               %put %str(ERROR: Unable to lock the data mart (lockfile: [&unlockedfile] sysmsg:[&msg]).);
            
               %let datamart_lock_rc=299;           
               %goto ERROR;
            
            %end;/* unable to rename lock file */
            %else
            %do; /* lock file renamed */
            
               %put %str(NOTE: The data mart has been locked by this session (lockfile:[&lockedfile]).);

               %writedmlock(file=&lockedfile);               
               
               %if &datamart_lock_rc %then
               %do; /* error writing lock file */
        
                  %put %str(WARN: Unable to write to the data mart lock file; however, the data mart was successfully locked.);
                  %let datamart_lock_rc=-3;

               %end;/* error writing lock file */                
               %else %let datamart_lock_rc=0;             
               
               %global DATAMART_LOCK_HELD;           
               %let DATAMART_LOCK_HELD=%sysfunc(putn(%sysfunc(datetime()),datetime.)); 
               
               %goto EXIT;
            
            %end;/* lock file renamed */         
         %end;/* "unlock" file found! */
         %else %if &lock_status eq LOCKED %then 
         %do; /* "lock" file not found */
        
            %put %str(ERROR: The data mart appears to be locked by another process (the lockfile: [&unlockedfile] not found).);
            
            %readdmlock(file=&lockedfile);   

            %let datamart_lock_rc=298;       
            %goto ERROR;
            
         %end;/* "lock" file not found */        
        
      %end;/* we do NOT hold the lock */     
   %end;/* obtain the lock */
  
   %if %upcase("&action")  eq "CHECK" %then
   %do; /* check if data mart is locked */
        
      %if %symexist(DATAMART_LOCK_HELD) %then
      %do; /* we hold the data mart lock */
      
         %put %str(NOTE: The data mart is locked by this session (lockfile:[&lockedfile].));
         
         %let datamart_lock_rc=0;         
         %goto EXIT;
         
      %end; /* we hold the data mart lock */
         
      %if &lock_status eq LOCKED %then
      %do; /* data mart is locked by someone else */
     
         %evcomment(/H,/C,WARNING!,
                    /C,DATA MART LOCKED!,,
                    /L, It appears that this data mart has been locked by another process.,,                   
                    /D,/C, PROCEED WITH CAUTION,/D,,
                    /L,Some tables may be unavailable or in the process of being updated.,
                    Updating or modifying tables in this session may interfere with the other process.,,
                    You may want to exit and wait for the other process to complete.,(lockfile:[&unlockedfile]),
                    /H
                    );
                             
         %readdmlock(file=&lockedfile);   

         %let datamart_lock_rc=1;                            
         %goto ERROR;
         
      %end;/* data mart is locked by someone else */ 
      %else
      %do; /* data mart is unlocked */
      
         %put %str(NOTE: The data mart does not appear to be locked at this time (lockfile:[&unlockedfile].));
         
         %let datamart_lock_rc=0;        
         %goto EXIT;
         
      %end; /* data mart is unlocked */
         
   %end;/* check if data mart is locked */
  
 %return;

 
 %EXIT:
 %return;

 
 %ERROR:
  %if %upcase("&abortonfail") eq "Y" %then %goto EXIT_ABORT;     
 %return;   

  
 %EXIT_ABORT:
  %put %str(ERROR: Session aborting due to data mart lock failure (rc=[&datamart_lock_rc]).);
  %let datamart_lock_rc=%sysfunc(max(1,&datamart_lock_rc));
  %abort return &datamart_lock_rc;
 %return;
  
%mend datamartlock;


%macro readdmlock(file=);

 %let rc=%sysfunc(filename(DMLOCK, &file));
 %let fid=%sysfunc(fopen(&DMLOCK));

 %if &fid > 0 %then
 %do; /* opened file */

    %put %str(INFO: The current contents of the file [&file] are as follows:);
    
    %do %while(%sysfunc(fread(&fid))=0);
       %let rc=%sysfunc(fget(&fid, c, 200));
       %put %str(INFO:   ) &c;  
    %end; /*do-while*/
   
 %end;/* opened file */
 %else
 %do; /*open failed */

   %let msg="ERROR: Unable to open datamart lock file [file: &file sysmsg: %sysfunc(sysmsg())].";
   %let datamart_lock_rc=-99;

 %end; /* open failed */

 %let rc=%sysfunc(fclose(&fid));
 %let rc=%sysfunc(filename(DMLOCK));
%mend readdmlock;



%macro writedmlock(file=, clear=N);

 %let rc=%sysfunc(filename(DMLOCK, &file));
 %let fid=%sysfunc(fopen(&DMLOCK,O));

 %if &fid > 0 %then
 %do; /* opened file */

    %if "&clear" eq "Y" %then %let rc=%sysfunc(fput(&fid,%str()));  
    %else
    %do; /*write out lock info */

       %let rc=%sysfunc(fput(&fid,dmlockpid=&sysjobid));  
       %if (%sysfunc(fwrite(&fid)) ne 0) %then %let datamart_lock_rc=197;
       %else
       %do;
          %let rc=%sysfunc(fput(&fid,dmlockuser=&sysuserid));    
          %if (%sysfunc(fwrite(&fid)) ne 0) %then %let datamart_lock_rc=197;
          %else
          %do;
             %let rc=%sysfunc(fput(&fid,dmlockprocess=&sysprocessname));   
             %if (%sysfunc(fwrite(&fid)) ne 0) %then %let datamart_lock_rc=197;
             %else %let datamart_lock_rc=0;
          %end;
      %end;

      %if &datamart_lock_rc %then
      %do;

         %let msg="ERROR: Problem encountered while attempting to write to datamart lock file [file: &file sysmsg: %sysfunc(sysmsg())].";
      %end;

    %end;/*write out lock info */   
   
 %end;/* opened file */
 %else
 %do; /*open failed */

   %let msg="ERROR: Unable to open datamart lock file [file: &file sysmsg: %sysfunc(sysmsg())].";
   %let datamart_lock_rc=197;

 %end; /* open failed */

 %let rc=%sysfunc(fclose(&fid));
 %let rc=%sysfunc(filename(DMLOCK));
%mend writedmlock;









