/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

 /*+----------------------------------------------------------------+
   | Name:     dataRollOff.sas                                      |
   | Product:  SAS Environment Manager                              |
   |                                                                |
   | Purpose:  Rolls off (i.e. deletes; with optional export first) |
   |           data older than the specified age from the CAS Search|
   |           Index.  Age is determined based on a date-derived    |
   |           "batch identifier".                                  |
   |                                                                |
   | History:  Date    action                                       |
   |           17MAR17 Initial Coding                        sasgzs |
   |                                                                |
   +----------------------------------------------------------------+*/
   
 

%macro dataRollOff(caslib=,               /* CASLIB of target CAS Search Index */
                   indexname=,            /* name of target CAS Search Index */
                   maxage=9999,           /* Threshold (i.e. # of days) before data is considered "old" */
                   basedate="&sysdate9"d, /* Date from which the threshold is calculated.  Defaults to today */
                   export=N,              /* Flag (Y|N) indicating whether "old" data is exported before being deleted */
                   out_caslib=&caslib,    /* CASLIB of target CAS table */                  
                   basename=BATCH);       /* Prefix to use when creating exported data files */

 %emimacroinit();
   
 proc cas;
  action search.searchIndex status = s result=results;
  param
      json='{"query":{"matchall":true}}'
      index={caslib="&caslib",name="&indexname"}
      casOut={caslib="&caslib",name="__MATCHALL",replace=true}
      jsonOut=false;
  run;

  %check_results(status=s, message='Executing MATCHALL query');
  
  action simple.freq status = s result=results;
  param casOut={caslib="&caslib",name="__FREQ",replace=true}
        table={caslib="&caslib", name="__MATCHALL",
               vars={{name="batchid",format="$",formattedLength=20.0}}
               };
  
  %check_results(status=s, message='Generating counts of BATCHID');
  
  run;
 quit;
 
 proc sql noprint;
  create table batches as
   select distinct batchid, 
          input(batchid,yymmdd6.) as date,
          &basedate - CALCULATED date as age       
    from &caslib..__MATCHALL;
  select batchid
   into :batchid1-:batchid9999
    from work.batches
      where age gt &maxage;
 quit;

 %let batchcount=&sqlobs;
 %put BATCHCOUNT: &batchcount;
 
 proc cas;
 %do i = 1 %to &batchcount; 

    %let batch2remove=&&batchid&i;
 
    %put DEBUG: &BATCH2remove;
    %if "&export" eq "Y" %then
    %do; /* export old data before deleting */
   
       %exportData(caslib=&caslib, indexname=&indexname, batch_id=&batch2remove, out_caslib=&out_caslib, out_table=&basename.&batch2remove);
       
    %end;/* export old data before deleting */    
   
    %deleteData(caslib=&caslib, indexname=&indexname, batch_id=&batch2remove);
    
 %end;
 quit;
  
%mend dataRolloff;
