/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

 /*+----------------------------------------------------------------+
   | Name:     etl_driver.sas                                       |
   | Product:  SAS Environment Manager                              |
   |                                                                |
   | Purpose:  Validates the data mart environment before calling   |
   |           the the appropriate ETL program to load data into    |
   |           CAS table(s).  If a 'canary' table is identified,    |
   |           this macro will confirm it exists, and, if not, call |
   |           the specified job to initialize it.                  |
   |                                                                |
   | History:  Date    action                                       |
   |           03AUG17 Logic moved from etl_driver job       sasgzs |
   |           29SEP17 Added CAS_UPDATE parm & assoc logic   sasgzs |
   |                                                                |
   +----------------------------------------------------------------+*/

%macro etl_driver(resourcecategory=,     /* resource category of data to be loaded */
                  cas_update=Y,          /* Y|N; should CAS be updated if appropriate for resourceType? */
                  canary_lib=,           /* name of library in which canary table is stored */
                  canary_table=,         /* name of the canary table */
                  initjob=,              /* name of SAS Job to call to initialize canary table */
                  exitonfail=);          /* Y|N; Continue with ETL if canary job does not exist? */




 %if "&canary_table" ne "" %then
 %do;  /* Check for 'canary' table   */

    %local error_code;  /* used for return code from initjob */
    %let error_code=0;  /* initialize return code */

    %confirm_cas_table_init(caslib=&canary_lib,table=&canary_table,initjob=&initjob,msglevel=WARNING); 

    /* RE-LOCK THE DATA MART; IF NEEDED */
    /* IN CASE INITJOB UNLOCKED         */
    /* ABORT IF LOCK CANNOT BE OBTAINED */
    %datamartlock(action=LOCK,abortonfail=Y);


    /* RE-CONNECT TO CAS; IF NEEDED     */
    /* IN CASE INITJOB DISCONNECTED     */
    /* ABORT IF CONNECTION FAILS.       */
    %get_cas_session(abortonfail=Y);

    %if &error_code gt 0 and %upcase("&exitonfail") eq "Y" %then
    %do; /* error encountered; exiting */

       %put NOTE: Exiting ETL for resource category [&resourcecategory] without further processing.[rc=&error_code];
       %return;

    %end;/* error encountered; exiting */

 %end; /* Check for 'canary' table   */

 filename etldcode temp;

 data _null_;
  length line $4000;
  set evdm.whitelist;
  where resource_category eq "&resourcecategory";

  retain cutoff_date allow_cas_update;

  if _n_ eq 1 then
  do; /* initialize some vars */

     cutoff_date = datepart(tzones2u(datetime())) - &cas_cutoff;

     allow_cas_update="&cas_update";  /* "global" flag indicating CAS updates are allowed */

  end;/* initialize some vars */
 
  file etldcode;
 
  data_dir=catx('/', "&evdmpath", resource_category, resourcetype);


  if upcase(allow_cas_update) eq "Y" and upcase(update_cas) eq "Y" then update_cas_flag="Y";
  else update_cas_flag="N";

  line=catt('%load_tsv_from_dir(data_dir=',data_dir,
            ', etl_mname=', etl_mname,
            ', cutoff_date=', cutoff_date,
            ', resource_category=',resource_category,
            ', resourcetype=',resourcetype,
            ', update_cas=',update_cas_flag,
            ');');
  put line;
 
 run;

 %include etldcode/source2;


%mend etl_driver;
