/***************************************************/
/* Copyright (c) 2014 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

/*---------------------------------------------------------------------------
 * NAME: %evinit_gmtdt
 *                             
 * DESC: This macro creates two macro variables. One variable contains the
 *       current system date and time in Greenwich Mean Time (GMT). The
 *       other variable returns the Coordinated Universal Time (UTC) offset
 *       for your time zone. Use this macro if you want to convert local
 *       times to GMT times or vice versa. For example, you can use the
 *       macro to convert the servers local date and time to GMT, and then
 *       use this value to filter input data that contains GMT fields (for
 *       example, the MetadataCreated and MetadataUpdated fields in the
 *       metadata repository).      
 *
 * PARAMETERS:
 *     None.
 *
 * OUTPUT:
 *    Generates the following macro variables that can be used during
 *    processing.
 *
 *    # ev_gmtdt  a SAS datetime value that represents the current
 *                 GMT (UTC) date and time.
 *
 *    # ev_tz    - the time zone offset based on the zero meridian in 
 *                 Greenwich, England.
 *
 *
 * USAGE:   
 * 
 *   In this example, the SAS system is running in Eastern Standard Time. 
 *
 *    %put System date time is %sysfunc(datetime(),datetime21.2);
 *    %evinit_gmtdt;
 *    %put Raw GMT Datetime is &ev_gmtdt;
 *    %put GMT Datetime is %sysfunc(putn(&ev_gmtdt.,datetime21.2));
 *    %put Raw time zone is &ev_tz;
 *    %put Time zone is %sysfunc(putn(&ev_tz.,time6.));
 *
 *    System date time is 15NOV2011:16:17:53.76
 *    Raw GMT Datetime is 1637011073.8
 *    GMT Datetime is 15NOV2011:21:17:53.80
 *    Raw time zone is -18000
 *    Time zone is  -5:00
 *      
 * UPDATED: 05NOV011                              
 *-------------------------------------------------------------------------*/
%macro evinit_gmtdt;
%global ev_gmtdt
        ev_tz;
data _null_;
  tltz = put(time(),E8601lZ14.);
  offset=translate(substr(tltz,indexc(tltz,'-+')),' ','+');
  offtime = input(offset,time6.);
  gmtdt = datetime() - input(offset,time6.);
  call symputx('ev_gmtdt',gmtdt,'F');
  call symputx('ev_tz',offtime,'F');
run;
%mend evinit_gmtdt;