/***************************************************/
/* Copyright (c) 2014 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

/*+----------------------------------------------------------------+
  | Name:     evsgplot.sas
  | Product:  SAS Environment Manager
  | Usage:
  |    dataset       - single dataset or view
  |    var1,...      - list of up to fifteen analysis (response) variable names
  |    period        - transform datetime variable by clumping
  |                    or folding. The following period keywords
  |                    are supported.
  |       type         Keyword   Example         Def.Format
  |       ------       -------   -------------   ----------
  |       Cluster      15MIN     03Jan14:01:15   DATETIME14.
  |                    HOUR      03Jan14:01      DATETIME10.
  |                    DATE      03Jan14         DATETIME7.
  |                    WEEK      01Jan14         DATETIME7.
  |                    MONTH     Jan14           MONYY.
  |                    QTR       Jan14           MONYY.
  |       Folded       24HOUR    hour-of-day (0-23)
  |                    WEEKDAY   day-of-week (1-7) or (Sun, Mon, etc).
  |                    MONTHDAY  day-of-month (1-31)
  |                    YEARDAY   day-of-year (1-366)
  |                    YEARMONTH month-of-year (1-12)
  |    todbins       - Integer (1 - 1440), used to create a number of discrete bins by dividing the
  |                    number of seconds in a day by bin number. Default equals 24 or hourly bins 
  |    collapse      - NONE|TOD|PERIOD After period and tod recoding, collapse summary statistic by 
  |                    by time (TOD) or datetime (PERIOD) 
  |    by            - by-variable (separate graph for each by variable)
  |    begin         - Beginning time span.
  |    end           - Ending time span.
  |                        BEGIN=TODAY-5:00:00, END=TODAY-2:23:59
  |                           or
  |                        BEGIN=04Jul53:00:00, END=12Apr76:23:59
  |    tzoneoff      - time zone offset (DATAMART, DATA, UTC or a +/- Integer 
  |                    equal to number of tz offset seconds (+/-)
  |    where         - data subsetting criteria
  |    stat          - analysis statistic (MEAN|SUM|FREQ|PERCENT|MEDIAN)
  |    labels        - paired list of variables and their labels
  |    formats       - paired list of variables and their formats
  |                    Title and Footnote parameters control the content, appearance,
  |                    and placement of titles and footnotes according to the SAS
  |                    statement definitions.
  |    plottype      - PROC SGPLOT supported plot types selection (HBAR|VBAR|HLINE|VLINE|DOT|VBARPARM|HBARPARM
  |                    SERIES|SCATTER|VECTOR|BUBBLE|STEP|REG|VBOX|HBOX|DENSITY|HISTOGRAM)  
  |    panelbyvars   -
  |    uniscale      - 
  |    group         - specifies a variable that is used to group the data
  |    xvar          - x axis or categorization variable. Datetime is the default for 
  |                    plot selects with a TIME axis
  |    groupdisplay  - Specifies how to display groups, STACK groups are overlaid and stacked,
  |                    CLUSTER values are displayed as separate adjacent bars. OVERLAY, if supported will
  |                    overlay the displays. STACK|CLUSTER|OVERLAY|_blank_  
  |    yvar          - bubble plot vertical axis variable
  |    y2var         - second, secondary right axis response variable. Uses vline or hline plot 
  |    xaxislab      - Text string for the x axis label
  |    yaxislab      - Text string for the y axis label
  |    title1        - title 1 
  |    title2        - title 2
  |    fnote1        - footnote 1
  |    fnote2        - footnote 2
  |    fnote3        - footnote 3 ( setting will overide and replace the default
  |                    report begin end datetime and the report execution datetime)
  |    xaxisopt      - options for the x axis
  |    yaxisopt      - options for the y axis  
  |    procopt       - any options for the PROC statement
  |    stmtopt       - any options for the PLOT statement
  |
  | Purpose:  This macro program generates a chart or plot using SAS proc sgplot
  |
  | History:  Date    action
  |           19Oct13 Coded for EV                           rnb
  |           17Jan15 Updated to match evtmpplate macro      rnb
  | End
  +----------------------------------------------------------------+*/

%macro evsgplot(dataset,var1,var2,var3,var4,var5,var6,var7,var8,var9,var10,
    var11,var12,var13,var14,var15,caller=,groupdisplay=,yaxislab=,xaxislab=,plottype=,
    panelbyvars=,uniscale=,group=,xvar=,yvar=,y2var=,gonly=NO,yaxisopt=,xaxisopt=,
    by=,begin=,end=,where=,stat=MEAN,debug=,period=ASIS,tzoneoff=DATAMART,xvartype=,
    title1=,title2=,fnote1=,fnote2=,fnote3=,collapse=,showparm=,todbins=,
    imagename=,procopt=,stmtopt=,panelopt=,labels=,formats=)
    / des='SGPlot Scatter Plot Graphs and Analysis';

   %local names i whglobal whlocal bystmt everror mname inds numvars 
          evallvars evrptvars qsubstr1 parmlist grpdisp invertdata axis
          cpleg1 tbyvarnum grpstmt response category pdelim y2plotstmt
          schkset obsset;
   
   %let grpstmt=;
   %let everror=0;
   /* %let schkset = %sysfunc(getoption($SYNTAXCHECK));*/
   %let obsset = %sysfunc(getoption(OBS));
   options no$syntaxcheck replace obs=max;

   %let mname=EVSGPLOT;
   %let inds=&dataset;
   %evsetglb;
   %evcomment((&MNAME) ...initializing);
   %evTIMLOG( START );
   %if "&debug" EQ "" %then %let debug=&evdebug;   
   %evsetdebug(SET,_debug=&debug);
   
   %let evrptvars =;
   %let numvars=0;
   %do i = 1 %to 15;
      %if &&var&i ne   %then %do;
         %let evrptvars = %qcmpres(&evrptvars &&var&i);
         %let numvars = &i;
      %end;
   %end;
   
   %if &debug GT 1 %then %put DEBUG: numrptvars [&numvars];
   %let evallvars = %str(datetime shift tzoneoff &group. &by. &xvar. &yvar. &y2var. &panelbyvars. &evrptvars.);

   %let parmlist = %str(dataset var1 var2 var3 var4 var5 var6 var7 var8 var9 var10
   var11 var12 var13 var14 var15
   plottype panelbyvars uniscale group xvar yvar y2var groupdisplay
   by begin end where stat debug period tzoneoff panelopt todbins xvartype
   labels formats title1 title2 fnote1 fnote2 fnote3 procopt stmtopt);
   
   %if "&showparm" = "ONLY" %then %goto EXIT;
   
   %let cpleg1=Y;
   %let collapse = %upcase(&collapse);
   %let period = %upcase(&period);
   %let groupdisplay = %upcase(&groupdisplay);
   %let tzoneoff=%upcase(&tzoneoff);
   %let plottype = %upcase(&plottype.);
   %let stat = %upcase(&stat);
   %let xvartype = %upcase(&xvartype);
   %let gonly = %upcase(&gonly);
   %if "&plottype" EQ "" %then %let plottype=SERIES;   
   
   %if "&by" NE "" %then    %let bystmt=%str(by &by;);
   %else                    %let bystmt=;

   %*********************************************************;
   %* Check paramenters;
   %*********************************************************;
   %if "&dataset" EQ "" and ("&var1" EQ "" or "&xvar" EQ "") %then %do;
      %everrmsg(&MNAME,msg=must specify dataset and one measure or categorization input.);
      %goto EXIT;
   %end;
   
   %if %index(%dqlist(HBAR VBAR HLINE VLINE DOT VBARPARM HBARPARM 
      SERIES SCATTER VECTOR BUBBLE STEP REG VBOX HBOX DENSITY HISTOGRAM),"&plottype.") EQ 0
   %then %do;
      %everrmsg(&MNAME,msg=SGPlot Type [&plottype] not supported.);
      %goto EXIT;
   %end;

   %*********************************************************;
   %* Check variable inputs match plot type choosen;
   %*********************************************************;
   %if "&yvar" NE "" and "&plottype" NE "BUBBLE" %then %do;
      %everrmsg(&MNAME,msg=Plottype [&plottype] does not support yvar inputs.);
      %goto EXIT;    
   %end;
   
   %if %index(%dqlist(SERIES SCATTER VECTOR BUBBLE STEP REG),"&plottype.") 
      %then %do;
      %if "&var1." EQ "" %then %do;
         %everrmsg(&MNAME,msg=Plottype [&plottype] requires analysis variable);
         %goto EXIT;
      %end;
   %end;

   %let axis=1;
   %if "&xvar" EQ "" %then %let xvar = %str(datetime);
   %*******************************************;
   %* Non computed plottype;
   %*******************************************;
   %if %index(%dqlist(VBARPARM HBARPARM SERIES SCATTER VECTOR BUBBLE STEP),"&plottype.") NE 0 %then %do;
      %if "&collapse" EQ "" %then %do;
         %let collapse=PERIOD;
      %end;
      %else %if "&collapse" EQ "NONE" %then %do;
         %evcomment(WARNING: (&MNAME) Non Computed plottype [&plottype] and collapse [&collapse] is expected
to be pre-summarized computed values (sum, mean, and so on) and may cause plotting problems due to sort order.
Consider using [PERIOD|TOD]);
      %end;
      %if "&var1." EQ "" or 
          "&xvar." EQ "" %then %do;
         %everrmsg(&MNAME,msg=[&plottype] requires a measure [&var1] and a catagorization variable [&xvar].);
         %goto EXIT;
       %end;
      %if "&period" EQ "ASIS" and "&stat" NE "" %then %do;
         %evcomment(WARNING: (&MNAME) Noncomputed plottype [&plottype] cannot aggregate);              
      %end;
      %if "&plottype" EQ "BUBBLE" and "&yvar." EQ "" %then %do;
         %everrmsg(&MNAME,msg=Bubble plot requires a yvar variable. Bubble size set equal to input measure [&var1]);
         %goto EXIT;
      %end;
      %if "&stat" EQ "PERCENT" %then %do;
         %everrmsg(&MNAME,msg=Statistic [&stat] not supported with [&plottype]. Review Means Statistic keywords);
         %goto EXIT;      
      %end;
   %end;
   
   %*******************************************;
   %* Computed plottype;
   %*******************************************;   
   %else %if %index(%dqlist(HBOX VBOX DENSITY HISTOGRAM HBAR VBAR VLINE HLINE DOT REG),"&plottype.") NE 0 %then %do;
      %let collapse=NONE;
   
      %if "&plottype." EQ "HBOX" or 
          "&plottype." EQ "VBOX" %then %do;
          %if "&var1." EQ "" %then %do;
             %everrmsg(&MNAME,msg=Type [&plottype.] requires an input measure [&var1.]);
            %goto EXIT;
          %end;
          %if "&xvar." NE "" %then %let category = %str(category=&xvar.);     
      %end;
   %end;
   
   %else %do;
      %everrmsg(&MNAME,msg=SGPlot Type [&plottype] not supported.);
      %goto EXIT;
   %end;

   %*********************************************************;
   %* Check stat;
   %*********************************************************;
   %if "&stat" NE "" %then %do;
      %if "&stat" EQ "MEAN" or 
          "&stat" EQ "SUM" or 
          "&stat" EQ "MEDIAN" or
          "&stat" EQ "PERCENT" or 
          "&stat" EQ "FREQ" %then %let statstmt = %str(STAT = &stat.);
      %else %do;
         %everrmsg(&MNAME,msg= Valid stat values FREQ|MEAN|SUM|MEDIUM|PERCENT [&stat].);
         %goto EXIT;
      %end;    
   %end;

   %*******************************************;
   %* y2var  without group  add it later;
   %*******************************************;
   %if "&y2var" NE "" %then %do;
      %if "&plottype" EQ "HBAR" or "&plottype" EQ "HLINE" %then %do;
         %let y2plotstmt = %str(HLINE &xvar. / response=&y2var. y2axis name="y2leg");          
      %end;
      %else %if "&plottype" EQ "VBAR" or "&plottype" EQ "VLINE" %then %do;
         %let y2plotstmt = %str(VLINE &xvar. / response=&y2var. y2axis name="y2leg");          
      %end;
      %else %if "&plottype" EQ "VBARPARM" or "&plottype" EQ "HBARPARM" %then %do;
         %let y2plotstmt = %str(SERIES x=&xvar. y=&y2var. / y2axis name="y2leg");
      %end;
      %else %do;
         %let y2plotstmt = %str(SERIES x=&xvar y=&y2var. / y2axis name="y2leg"); 
      %end;
   %end;
  
   %*******************************************;
   %* collapse;
   %*******************************************;  
   %if "&collapse" EQ "" %then %let collapse=PERIOD;
   %if "&collapse." EQ "PERIOD" or
       "&collapse." EQ "TOD" or
       "&collapse." EQ "NONE" %then %do;
      %if "&collapse." EQ "NONE" and "&period." NE "ASIS" %then %do;
           %evcomment((&MNAME) Period eq [&period.] and collapse eq [&collapse.], 
Proc selections must handle multiple values per datetime);
      %end; 
   %end;
   %else %do;
       %everrmsg(&MNAME,msg=Invalid Collapse option = [&collapse.].);
       %goto EXIT;
   %end;

   %*******************************************;
   %* groupdisplay;
   %*******************************************;
   %if "&groupdisplay." NE "" and "&group" ne "" %then %do; 
      %if %index(%dqlist(HBAR VBAR VBARPARM HBARPARM SERIES SCATTER VECTOR BUBBLE STEP VBOX HBOX
DENSITY HISTOGRAM HLINE VLINE),"&plottype.") EQ 0
      %then %do;
         %everrmsg(&MNAME,msg=Groupdisplay Invalid for plottype [&plottype]);
         %goto EXIT;
      %end;      
      %else %do;
         %if "&groupdisplay" EQ "STACK" or
             "&groupdisplay" EQ "CLUSTER" or
             "&groupdisplay" EQ "OVERLAY" %then %do;
           %let grpdisp=%str(groupdisplay=&groupdisplay. clusterwidth=0.7 /* display=(FILL)*/ );
         %end;
         %else %do;
            %everrmsg(&MNAME,msg=Invalid Groupdisplay [&groupdisplay.] [STACK|CLUSTER|OVERLAY]);
            %goto EXIT;
         %end;
      %end;
      %if "&groupdisplay" EQ "STACK" and ("&plottype" EQ "VLINE" or "&plottype" EQ "HLINE" ) %then %do;
         %everrmsg(&MNAME,msg=Invalid Groupdisplay [&groupdisplay.] for plottype [&plottype]);
         %goto EXIT;
      %end;
   %end;
 
   %*******************************************;
   %* xvar type = discrete;
   %* otherwise xvar type = measure;
   %*******************************************;
   %if "&xvartype" EQ "" %then %do;
      %let xvartype=DISCRETE;
   %end;
   
   %*********************************************************;
   %* Check panelby inputs;
   %* need to decide to invert data, panel yes, overlay no;
   %*********************************************************;
   %let sgxxx=SGPLOT;
   %let pdelim=;
   %if "&panelbyvars" NE "" %then %do;
      %let sgxxx=SGPANEL;
      
      %let uniscale = %upcase(&uniscale);
      %if "&uniscale." EQ "COLUMN" or
          "&uniscale." EQ "ROW" or
          "&uniscale." EQ "ALL" %then %do;
         %let uniscale = %str(uniscale = &uniscale);
         %let pdelim=%str(/);
      %end;
      %if "&panelopt" NE "" %then %let pdelim=%str(/);
      %if "&xaxisopt" NE "" or 
          "&yaxisopt" NE "" or
          "&xaxislab" NE "" or
          "&yaxislab" NE "" %then %do;
             %everrmsg(&MNAME,msg=Axis labels or axis options not valid with PROC SGPANEL.);
             %goto EXIT;
      %end;
   %end;
   %else %do;
      %if "&uniscale." NE "" or
          "&panelopt." NE "" %then %do;
             %everrmsg(&MNAME,msg= Panelopt and Uniscale only valid with panelby option.);
             %goto EXIT; 
      %end;
   %end;

   %*******************************************;
   %* xaxis option overrides.;
   %*******************************************;
   %if "&xaxisopt." EQ "" %then %do;
      %let xaxisopt = %str(labelposition=CENTER);
      %evcomment((&MNAME) No x axis option overides, defaults [&xaxisopt.]);
   %end;
   %else %evcomment((&MNAME) x axis option overrides, [&xaxisopt.]);    
   
   %*******************************************;
   %* yaxis option overrides.;
   %*******************************************;
   %if "&yaxisopt." EQ "" %then %do;
      %let yaxisopt = %str(labelposition=CENTER);
      %evcomment((&MNAME) No yaxis option overides, defaults [&yaxisopt.]);
   %end;
   %else %evcomment((&MNAME) y axis option overrides, [&yaxisopt.]); 
   
   %*******************************************;
   %* proc and proc statement option overrides.;
   %*******************************************;
   %if "&procopt." EQ "" %then %do;
      %let procopt = %str(CYCLEATTRS);
      %evcomment((&MNAME) No procopt overides, defaults [&procopt.]);
   %end;
   %else %evcomment((&MNAME) procopt option overrides, [&procopt.]);

   %if "&stmtopt." EQ "" %then %do;
      %let stmtopt = %str();
      %evcomment((&MNAME) No stmtopt overides, defaults [&stmtopt.]);
   %end;
   %else %evcomment((&MNAME) stmtopt option overrides, [&stmtopt.]);  

   %*********************************************************;
   %* prep plot options;
   %*********************************************************;
   %if "&yvar." NE "" and "&plottype." NE "BUBBLE" %then %do;
      %everrmsg(&MNAME,msg= yvar [&yvar.] only supported on bubble plot.);
      %goto EXIT; 
   %end;  
   
   %*********************************************************;
   %* Check to see if this is a graph only, skip transforms.
   %*********************************************************;
   %if "&gonly" EQ "YES" %then %do;
      %if "&period" NE "ASIS" %then %do;
         %everrmsg(&MNAME,msg= Graph only option does not support period collapsing. [&period] invalid.);
         %goto EXIT;
      %end;
      %if "&group" NE "" %then %let grpstmt=%str(group=&group.);
      data work._EVSCRATCH;
         set &dataset;
         %evmkfmt(&formats);
         %evmklab(&labels);
      run;
      %if &syserr GT 4 %then %goto ERROR;
         %let dataset = work._EVSCRATCH;
         %let _dsid = %sysfunc(open(&dataset.));
         %let cpnobs = %sysfunc(attrn(&_dsid,NOBS));           
            %if "&cpnobs" EQ "0" %then %do;
               %let rc = %sysfunc(close(&_dsid));
               %goto NOOBS;
            %end;
      %let rc = %sysfunc(close(&_dsid));
      %goto GONLY;
   %end;

   %*********************************************************;
   %* Check valid dataset exists with Datetime and other requested variables;
   %*********************************************************;
   %if "&caller" NE "EVSTP" %then %do;
   %if %sysfunc(exist(&dataset)) or
       %sysfunc(exist(&dataset,view)) 
       %then %do;
           %let _dsid = %sysfunc(open(&dataset.));
           %do i=1 %to %num_tokens(&evallvars.);
                %if &debug GT 1 %then %put DEBUG: (&MNAME) %scan(&evallvars.,&i);
                %if not %sysfunc(varnum(&_dsid,%scan(&evallvars.,&i.))) %then %do;
                   %if "%upcase(%scan(&evallvars.,&i))" NE "TIME" %then %do;
                      %everrmsg(&MNAME,msg=Variable <%scan(&evallvars.,&i.)> missing from Dataset [&dataset].);
                      %let rc = %sysfunc(close(&_dsid));
                      %goto EXIT;
                   %end;
                %end; 
           %end; 
      %end;
      %else %do;
         %everrmsg(&MNAME,msg=Dataset [&dataset] missing.);
         %goto EXIT;
      %end;
   %let rc = %sysfunc(close(&_dsid));
   %end;
   
   %*********************************************************;
   %* inverting data allows the combining of group and multiple input measures;
   %*********************************************************;   
   %if &numvars EQ 0 %then %let axis=0;   
   %let invertdata=NO;   
   %if "&group" NE "" %then %do;
      %let grpstmt = %str(group=&group.);  
      %if &numvars GT 1 %then %do;
         %let invertdata=YES;      
         %let sgxxx=SGPANEL;
         %let panelbyvars=_cptype;      
         %evcomment(NOTE: (&MNAME) Input measures GT 1 and group selection [&group]. Using panel for group);       
      %end;
   %end;
   %else %if &numvars gt 1 %then %do;
   %end;
   
   %*********************************************************;
   %* Validate and/or calculate begin and end parms/timestamps;
   %*********************************************************;
   %if &debug GT 1 %then %put DEBUG: (&MNAME) begin [&begin] end [&end];  
   %if "&begin" EQ "" or "&end" EQ "" %then %do;
      %everrmsg(&MNAME,msg=Begin [&begin] or End [&end] time window missing);
      %goto EXIT;
   %end;
   %if "&todbins" NE "" %then %do;
      %let todbins = %sysfunc(ceil(&todbins));
      %if NOT (&todbins GT 0 and &todbins LT 1440) %then %do;
         %everrmsg(&MNAME,msg=todbins [&todbins] Positive integer (1 - 1440));
         %goto EXIT;
      %end;
   %end;  
   
   %********************************************************;
   %* Adjusting for requested time zone offset             *;
   %* LOCAL = use local timezone of host/platform          *;
   %* DATA = use timezone associated with data (per record)*;
   %* UTC or GMT = Use 0 seconds, Data Mart in UTC          *;
   %* or valid integer value = number of offset seconds    *;
   %********************************************************;
   %if "&tzoneoff." EQ "" %then %do;
      %evcomment((&MNAME) tzoneoff value eq [&tzoneoff] setting to DATAMART);
      %let tzoneoff=%str(DATAMART);
   %end;
   %evsettz;
    
   %********************************************************;
   %* Where clause handling;
   %********************************************************;
   %evwheval(where=&where,begin=&begin,end=&end,tzoneoff=&tzoneoff);
   
   %******************************************************************;
   %* Establish temporary view with subsetting (and index support)   *;
   %******************************************************************;
   %* clean out any previous temporary view;
   proc datasets nolist nowarn dd=work;
      delete _EVSCRATCH / memtype=data;
      run;
    data work._EVSCRATCH;
       set &dataset;
       &whglobal;
       &whlocal;
       %evmkfmt(&formats);
       %evmklab(&labels);
       run;
       
   %if &syserr GT 4 %then %goto ERROR;
   %let dataset = work._EVSCRATCH;
   %let _dsid = %sysfunc(open(&dataset.));
   %let cpnobs = %sysfunc(attrn(&_dsid,NOBS));
   %let rc = %sysfunc(close(&_dsid));
   %if "&cpnobs" EQ "0" %then %do;
       %goto NOOBS;
   %end;
   
   %evcomment((&MNAME) Time window conversion then running sgplot);
   %******************************************************************;
   %* Transpose the data and subset if necessary;
   %******************************************************************;
   %let cl_nam = &xvar. &group. &yvar. &panelbyvars.;
   %if &debug >1 %then %do;
      %evcomment((&MNAME) evxform inputs, period=[&period], collapse=[&collapse], stat=[&stat], 
      cl_nam=[&cl_nam], cumulate=VAR, axis=[&axis], invert=[&invertdata]);
   %end; 
   %evxform(&dataset,period=&period,by=&by,collapse=&collapse,axis=&axis,xvar=&xvar,xvartype=&xvartype,/*cumulate=VAR,*/
      _rc=everror,stat=&stat,cl_nam=&cl_nam,invert=&invertdata,outds=tmpdata,keep=&yvar. &y2var.);
   %let dataset=tmpdata;
   %if "&everror" NE "0" %then %do;
      %if "&everror" = "2" %then %goto NOOBS;
      %else %goto ERROR;
   %end;
   
%GONLY:   
   %********************************************************;
   %* titles and footnodes;
   %*********************************************************;
   %evtitlesfoot;
   
   %if "imagename" NE "" %then %str(ods graphics on / imagename="&imagename";);

   ods proclabel = "&title1.";
   proc &sgxxx. data=&dataset &procopt description="";

   %if "&sgxxx." EQ "SGPLOT" %then %do;
      xaxis &xaxisopt.
            %if "&xaxislab" NE "" %then %do;
               %if "&xaxislab" EQ "_VARNAME" %then %str(label = "&xvar.");
               %else %if "&xaxislab" EQ "_VARLABEL" %then;
               %else %str(label = "&xaxislab.");
            %end;
            ;
      yaxis &yaxisopt.
            %if "&yaxislab" NE "" %then %do;
                %if "&yaxislab" EQ "_VARNAME" %then %str(label = "&var1.");
                %else %if "&yaxislab" EQ "_VARLABEL" %then;
                %else %str(label = "&yaxislab.");
            %end;
            ;
   %end;

   %goto &plottype;

%SCATTER:
   %if "&stmtopt" EQ "" %then %do; 
      %let stmtopt=%str(/*markerattrs=(symbol=circlefilled size=1.25pct)*/);
      %evcomment((&MNAME) Using SCATTER defaults [&stmtopt.]);
   %end;
%SERIES:
%STEP:
%VECTOR:
%REG:
   %*********************************************************;
   %* Draw chart;
   %*********************************************************;
      %if "&panelbyvars." NE "" %then 
         %str(panelby &panelbyvars. &pdelim. &uniscale &panelopt;);

      %if "&y2var" NE "" %then %do;
         &y2plotstmt. &grpstmt.;
      %end;      
      %if "&invertdata" EQ "YES" %then %do;
         &plottype. x=&xvar. y=&var1. / &grpstmt. &grpdisp &stmtopt. name="&var1.";
         keylegend %str("&var1.");
      %end;
      %else %do;
         %do i = 1 %to &numvars.;
            &plottype. x=&xvar. y=&&var&i. / &grpstmt. &grpdisp &stmtopt. name="&&var&i.";
         %end;
         keylegend  
            %do i = 1 %to &numvars.; 
               %str("&&var&i.")
            %end;
            %if "&y2var" NE "" %then %str("y2leg");
          ;
      %end;
   run;
   quit;       
   %if &syserr GT 4 %then %goto ERROR;
   %goto EXIT;

%HBAR:
%VBAR:
%DOT:
%HLINE:
%VLINE:
   %*********************************************************;
   %* Draw chart;
   %*********************************************************;
      %if "&panelbyvars." NE "" %then 
          %str(panelby &panelbyvars. &pdelim. &uniscale &panelopt;);

      %if "&invertdata" EQ "YES" or &numvars EQ 0 %then %do;
         %if "&var1." NE "" %then %let response = %str(response=&var1.);      
         &plottype. &xvar. / &grpstmt. &grpdisp &response. &statstmt. &stmtopt. name="&xvar.";      
         keylegend %str("&xvar.");
      %end;
      %else %do;
         %do i = 1 %to &numvars.;
            %if "&var1." NE "" %then %let response = %str(response=&&var&i.);
            &plottype. &xvar. / &grpstmt. &grpdisp &response. &statstmt. &stmtopt. name="&&var&i.";
         %end;
      %end;
      %if "&y2var" NE "" %then %do;
         &y2plotstmt. &grpstmt.;
      %end;
   run;
   quit;       
   %if &syserr GT 4 %then %goto ERROR;
   %goto EXIT;    

%VBARPARM:
%HBARPARM:
      %if "&panelbyvars." NE "" %then 
          %str(panelby &panelbyvars. &pdelim. &uniscale &panelopt;);
      %let category = %str(category=&xvar.);
      
      %do i = 1 %to &numvars.;
         %if "&var1." NE "" %then %let response = %str(response=&&var&i.);
         &plottype. &category. &response. / &grpstmt. &grpdisp &stmtopt. name="&&var&i.";
      %end;
      %if "&y2var" NE "" %then %do;
         &y2plotstmt. &grpstmt.;
      %end;
   run;
   quit;       
   %if &syserr GT 4 %then %goto ERROR;
   %goto EXIT;  

%DENSITY:
%HISTOGRAM:
      %if "&panelbyvars." NE "" %then 
          %str(panelby &panelbyvars. &pdelim. &uniscale &panelopt;);
          
      %if "&var1." EQ "" %then %do;
            &plottype. &xvar. / &grpstmt. &stmtopt.;         
      %end;
      %else %do;
         %do i = 1 %to &numvars.;
            &plottype. &&var&i. / &grpstmt. &stmtopt.; /* bug in density leave off name="&&var&i." */
         %end;
      %end;
      %if "&y2var" NE "" %then %do;
         &y2plotstmt. &grpstmt.;
      %end;
   run;
   quit;  
   %if &syserr GT 4 %then %goto ERROR;
   %goto EXIT;

%HBOX:
%VBOX: 
      %if "&panelbyvars." NE "" %then %str(panelby &panelbyvars. &pdelim. &uniscale &panelopt;);            
          
      %do i = 1 %to &numvars.;         
         &plottype. &&var&i. / &category. &grpstmt. &grpdisp &stmtopt. name="&&var&i.";
      %end;
          
      %if "&sgxxx." NE "SGPANEL" and &numvars GT 1 %then %do;
         keylegend  
         %do i = 1 %to &numvars.;
            %str("&&var&i.")
         %end;
         %if "&y2var" NE "" %then %str("y2leg");  
         ;
      %end;
   run;
   quit;       
   %if &syserr GT 4 %then %goto ERROR;
   %goto EXIT;

%BUBBLE:
      %if "&panelbyvars." NE "" %then 
         %str(panelby &panelbyvars. &pdelim. &uniscale &panelopt;);

      %if "&invertdata" EQ "YES" %then %do;
         &plottype. x=&xvar. y=&yvar. size=&var1. / &grpstmt. &grpdisp &stmtopt. name="&var1.";
         keylegend %str("&var1.");
      %end;
      %else %do;
         %do i = 1 %to &numvars.;
            &plottype. x=&xvar. y=&yvar. size=&&var&i. / &grpstmt. &grpdisp &stmtopt. name="&&var&i.";
         %end;
         keylegend  
            %do i = 1 %to &numvars.; 
               %str("&&var&i.")
            %end;
          ;
      %end;

   run;
   quit;       
   %if &syserr GT 4 %then %goto ERROR;
   %goto EXIT;

%NOOBS:

   %noobsrpt(inds=&inds,begin=&begin=,end=&end,where=&where);
   %goto EXIT;

%ERROR:
   /*----------------------------------------------------------------
    | Syserr Description;
    |
    | 0 Execution completed successfully. 
    | 1 Execution was canceled by a user with RUN or CANCEL. 
    | 2 Execution was canceled by a user with ATTN or BREAK command. 
    | 3 An error in a program run in batch or non-interactive mode caused 
    |   SAS to enter syntax-check mode. 
    | 4 Execution completed successfully with warning messages. 
    | >4 An error occurred, value returned is procedure dependent.
    +----------------------------------------------------------------+*/
    %everrmsg(&MNAME,msg= %superq(syserrortext));
    %let everror=0;

 %EXIT:

   %*********************************************************;
   %* Show invocation parameters;
   %*********************************************************;
   %let showparm = %upcase(&showparm);
   %if "&showparm." EQ "YES" or "&showparm." EQ "ONLY" %then %do;
      %let dataset=&inds;
      data parms;
         format parameter $32. value $80.;
         label parameter = "&sysmacroname parameter" value = 'value';
      
         %do i=1 %to %num_tokens(&parmlist);
            %let pvalue=&&&%scan(%nrquote(&parmlist.),&i).;
             %if "&pvalue" NE "" %then %do;
                parameter = "%scan(&parmlist.,&i)";
                value = " %bquote(&pvalue) ";
                output;
             %end;
         %end;
      run;
/*
      Proc report data=parms nowd headline headskip
         style(header) = [fontweight=bold];
         %if "&title1." NE "" 
         %then column ("&title1." parameter value);
         %else column (parameter value);
         ;
      run;
rnb replace with proc sql      */
   %end;

   %*********************************************************;
   %* Cleanup scratch datasets;
   %*********************************************************;
   %if &debug LT 3 %then %do;
   proc datasets library=work memtype=(data view) nolist nowarn;
      delete parms noobs _evscratch tmpdata / memtype=data;
      run; quit;
   %end;

   %evsetdebug(RESET);
   %evTIMLOG( STOP, &MNAME );

   options &schkset OBS=&obsset;

%mend evsgplot;