/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

/*+----------------------------------------------------------------+
  | Name:     evtmpplate.sas
  | Product:  SAS Environment Manager
  | Usage:
  |    dataset       - single dataset or view
  |    var1,...      - list of up to 15 analysis (response) variable names
  |    period        - transform datetime variable by clumping
  |                    or folding. The following period keywords
  |                    are supported.
  |       type         Keyword   Example         Def.Format
  |       ------       -------   -------------   ----------
  |       Cluster      15MIN     03Jan14:01:15   DATETIME14.
  |                    HOUR      03Jan14:01      DATETIME10.
  |                    DATE      03Jan14         DATETIME7.
  |                    WEEK      01Jan14         DATETIME7.
  |                    MONTH     Jan14           MONYY.
  |                    QTR       Jan14           MONYY.
  |       Folded       24HOUR    hour-of-day (0-23)
  |                    WEEKDAY   day-of-week (1-7) or (Sun, Mon, etc).
  |                    MONTHDAY  day-of-month (1-31)
  |                    YEARDAY   day-of-year (1-366)
  |                    YEARMONTH month-of-year (1-12)
  |    todbins       - Integer (1 - 1440), used to create a number of discrete bins by dividing the
  |                    number of seconds in a day by bin number. Default equals 24 or hourly bins
  |    collapse      - NONE|TOD|PERIOD After period and tod recoding, collapse summary statistic by 
  |                    by time (TOD) or datetime (PERIOD)   
  |    by            - by-variable (separate graph for each by variable)
  |    begin         - Beginning time span.
  |    end           - Ending time span.
  |                        BEGIN=TODAY-5:00:00, END=TODAY-2:23:59
  |                           or
  |                        BEGIN=04Jul53:00:00, END=12Apr76:23:59
  |    tzoneoff      - time zone offset (DATAMART, DATA, UTC or a +/- Integer 
  |                    equal to number of tz offset seconds (+/-) 
  |    where         - data subsetting criteria
  |    stat          - analysis statistic (mean, sum, freq, pct)
  |    layout        - PROC Template supported layout selection (GRIDDED|OVERLAY|DATAPANEL)
  |    plottype      - PROC Template supported plottype selection (BARCHART|BARCHARTPARM|
  |                    BOXPLOT|HEATMAPPARM|LINEPLOT|SERIESPLOT|SCATTERPLOT|DENSITYPLOT|
  |                    FRINGEPLOT|NEEDLEPLOT|STEPPLOT|HISTOGRAM|PIECHART
  |    group         - specifies a variable that is used to group the data
  |    yvar          - yvar or vertical axis used for HEATMAPPARM plotting, use var1, var2... 
  |                    as the color response variable
  |    xvar          - x axis or categorization variable. Datetimye is the default for 
  |                    plot selects with a TIME axis type  
  |    y2var         - second, secondary right axis response variable. Uses seriesplot plot
  |    columns       - number of columns
  |    rows          - number of rows
  |    groupdisplay  - Specifies how to display groups, STACK groups are overlaid and stacked,
  |                    CLUSTER values are displayed as separate adjacent bars. OVERLAY, if supported will
  |                    overlay the displays. STACK|CLUSTER|OVERLAY|_blank_
  |    orient        - Plot orientation (VERTICAL|HORIZONTAL|_blank_)
  |    xaxislab      - Text string for the x axis label
  |    yaxislab      - Text string for the y axis label
  |    legendlab     - Text string for the legend label
  |    legendtitle   - Text string for the legend title
  |    labels        - paired list of variables and their labels
  |    formats       - paired list of variables and their formats
  |                    Title and Footnote parameters control the content, appearance,
  |                    and placement of titles and footnotes according to the SAS
  |                    statement definitions.
  |    title1        - title 1 
  |    title2        - title 2
  |    fnote1        - footnote 1
  |    fnote2        - footnote 2
  |    fnote3        - footnote 3 ( setting will overide and replace the default
  |                    report begin end datetime and the report execution datetime)
  |    hmrange       - True|False include heatmaprange construction using style colorrampthree
  |    hmrangemin    - Min value for the heatmaprange
  |    hmrangemax    - Max value for the heatmaprange
  |    hmrangekeys   - range keywords including UNDER|OVER|Missing
  |    xaxisopt      - options for the x axis
  |    yaxisopt      - options for the y axis
  |    layoutopt     - options for the layout statement selected via layout parameter
  |    plotopt       - options for the plot statement selected via plottype parameter
  |
  | Purpose:  This macro program generates plots via PROC Template
  |
  | History:  Date    action
  |           17Nov14 Initial Coding                           rnb
  | Notes:
  +----------------------------------------------------------------+*/

%macro evtmpplate(dataset,var1,var2,var3,var4,var5,var6,var7,var8,var9,var10,
   var11,var12,var13,var14,var15,xvar=datetime,yvar=,y2var=,
   by=,begin=,end=,where=,stat=,todbins=,collapse=,xaxislab=,xorigin=,yorigin=,
   group=,legendtitle=,legendlab=,statlab=,yaxislab=,debug=,showparm=,orient=,
   layout=,layoutopt=,plottype=,plotopt=,columns=,rows=,groupdisplay=,
   columnaxisopts=,rowaxisopts=,imagename=,xaxisopt=,yaxisopt=,
   hmrange=,hmrangekeys=,hmrangemin=,hmrangemax=,bsize=,xvartype=,
   period=ASIS,tzoneoff=DATAMART,labels=,formats=,caller=,
   title1=,title2=,fnote1=,fnote2=,fnote3=) 
   / des='proc Template';

   %local names i whglobal whlocal bystmt everror mname inds xvartype dislegend grptypestmt
      response lastby qsubstr1 parmlist n2leg fnid evrptvars evallvars efnote axis
      schkset obsset grpdisp orientstmt rowstmt colstmt statstmt numvars bvar;

   %let everror=0;

   %let schkset = %sysfunc(getoption($SYNTAXCHECK));
   %let obsset = %sysfunc(getoption(OBS));
   options no$syntaxcheck obs=max;
   
   %*******************************************************************;
   %* Init section;
   %*******************************************************************;  
   %let mname=evtmpplate;
   %let inds=&dataset;
   %evsetglb;
   %evcomment((&MNAME) ... initializing);
   %evTIMLOG( START );
   %if "&debug" EQ "" %then %let debug=&evdebug;   
   %evsetdebug(SET,_debug=&debug);
   
   %let parmlist = %str(dataset var1 var2 var3 var4 var5 var6 var7 var8 var9 var10
   var11 var12 var13 var14 var15 xvar yvar columns rows layout layoutopt plottype plotopt
   by begin end where stat group legendtitle statlab yaxislab xaxislab xaxisopt yaxisopt
   period tzoneoff todbins debug groupdisplay collapse orient hmrangekeys hmrangemin
   hmrange hmrangemax bsize labels formats title1 title2 fnote1 fnote2 fnote3);

   %if "&showparm" = "ONLY" %then %goto EXIT;
   
   %let evrptvars =;
   %let numvars=0;
   %do i = 1 %to 15;
      %if &&var&i ne   %then %do;
         %let evrptvars = %qcmpres(&evrptvars &&var&i);
         %let numvars = &i;
      %end;
   %end;
   %if &debug GT 1 %then %put DEBUG: (&MNAME) numrptvars [&numvars];
   %let evallvars = %str(datetime shift tzoneoff &by. &group. &evrptvars.);  
  
   %if "&by" NE "" %then %let bystmt=%str(by &by;);
   %else                 %let bystmt=%str();
   %if "&xvar" EQ "" %then %let xvar=DATETIME;
   %let tzoneoff = %upcase(&tzoneoff);   
   %let layout = %upcase(&layout);
   %let plottype = %upcase(&plottype);
   %let xvartype = %upcase(&xvartype);
   %let stat = %upcase(&stat);   
   %if "&plottype" EQ "" %then %let plottype=SERIESPLOT;
     
   %*********************************************************;
   %* Check paramenters;
   %*********************************************************;
   %if "&dataset" EQ "" and ("&var1" EQ "" or "&xvar" EQ "") %then %do;
      %everrmsg(&MNAME,msg=Invocation requires dataset and one measure or categorization input.);
      %goto EXIT;
   %end;
   
   %*********************************************************;
   %* Check layout selection;
   %*********************************************************;
   %if "&layout" EQ "" %then %let layout=%str(OVERLAY);
   %if %index(%dqlist(OVERLAY GRIDDED DATALATTICE DATAPANEL),"&layout.") EQ 0
   %then %do;
      %everrmsg(&MNAME,msg=Invalid layout selection [&layout]);
      %goto EXIT;
   %end;
   %if %num_tokens(&group) GT 1 %then %do;
       %if "&layout" NE "DATAPANEL" %then %do;
          %everrmsg(&MNAME,msg=Multiple groups [&group] only supported with DATAPANEL layout);
          %goto EXIT;        
       %end;
   %end;
   %if "&y2var" NE "" %then %do;
      %if "&layout" EQ "GRIDDED" %then %do;
         %everrmsg(&MNAME,msg=Layout GRIDDED does not support two axis);
         %goto EXIT;
      %end;
      %if "&plottype" EQ "PIECHART" %then %do;
         %put WARNING: (&MNAME) y2var [&y2var] ignored for plottype of PIECHART;
      %end;
   %end;
   
   %*********************************************************;
   %* Check period;
   %*********************************************************;
   %let period=%upcase(&period);
   %if %index(%dqlist(ASIS 15MIN HOUR DATE WHOLEDAY WEEK MONTH QTR 24HOUR
   WEEKDAY MONTHDAY YEARDAY YEARMONTH),"&period.") EQ 0
   %then %do;
     %everrmsg(&MNAME,msg=Period [&period] not supported.);
     %goto EXIT;
   %end;
   
   %*********************************************************;
   %* Check plottype layout selection;
   %*********************************************************;

   /*
   %if "&yvar" NE "" and "&plottype" NE "HEATMAPPARM" %then %do;
      %everrmsg(&MNAME,msg=yvar input [&yvar] only supported with plottype of HEATMAPPARM);
      %goto EXIT;
   %end;
   */
   
   %*********************************************************;
   %* DISCRETE Computed Plot Types;
   %* first, test for a measure, if missing assume request for PCT|FREQ;
   %*********************************************************;
   %let axis=1;
   %if "&var1" EQ "" %then %do;
      %let axis=0;
      %if "&stat" EQ "" %then %let stat=FREQ;
      %else %if "&stat" EQ "MEAN" or "&stat" EQ "SUM" %then %do;
          %everrmsg(&MNAME,msg= Statistic [&stat.] requires one or more measures [var1 var2 ...].);
          %goto EXIT;
      %end;
      %if %index(%dqlist(BARCHART LINECHART PIECHART HISTOGRAM DENSITYPLOT BARCHARTPARM),"&plottype.") EQ 0
       %then %do;
         %everrmsg(&MNAME,msg=Plottype [&plottype] requires one or more measure input.);
         %goto EXIT;         
      %end;
   %end;
   
   %*********************************************************;
   %* Computed Plot Types;
   %*********************************************************;
   %if %index(%dqlist(BOXPLOT BARCHART LINECHART PIECHART HISTOGRAM DENSITYPLOT),"&plottype.") NE 0
   %then %do;
      %if "&collapse" EQ "" %then %do;
         %let collapse=NONE;
      %end;
      %else %if "&collapse" NE "NONE" %then %do;
         %evcomment(WARNING: (&MNAME) Computed plottype [&plottype] and collapse [&collapse] may cause 
invalid statistical aggregation. Consider using [NONE]);
      %end;
      
      %if "&layout" EQ "DATAPANEL" %then %do;
         %everrmsg(&MNAME,msg=Computed plots cannot be used with layout DATAPANEL.);
         %goto EXIT;         
      %end;
      %if "&plottype" EQ "PIECHART" %then %do;
         %if "&layout" EQ "OVERLAY" %then %let layout=REGION;
      %end;
      %*********************************************************;
      %* STAT and axis value only works with Computed plot types;
      %*********************************************************;
      %let statstmt=;
      %if "&stat" NE "" %then %do;
         %if "&plottype" EQ "BARCHART" or 
             "&plottype" EQ "PIECHART" or 
             "&plottype" EQ "LINECHART" 
         %then %do;
            %if "&stat" NE "MEAN" and 
                "&stat" NE "SUM" and
                "&stat" NE "PCT" and
                "&stat" NE "FREQ" and
                "&stat" NE "PROPORTION"
            %then %do;
               %everrmsg(&MNAME,msg=Invalid &plottype statistic [&stat] [MEAN|PCT|SUM|FREQ|PROPORTION]);
               %goto EXIT;      
            %end;
            %let statstmt = %str(stat=&stat);
         %end;
      %end;
      %if "&y2var" NE "" %then %do;
         %evwarnmsg(&MNAME,msg=Mixing computed and non computed plot types. May product inconsistent results);
         /* %goto EXIT; */         
      %end;
   %end; /* computed plot types */
   
   %*********************************************************;
   %* Non Computed Plot Types;
   %*********************************************************;    
   %else %do;
      %if %index(%dqlist(SERIESPLOT SCATTERPLOT SCATTERPLOTMATRIX NEEDLEPLOT STEPPLOT FRINGEPLOT
BARCHARTPARM HEATMAPPARM VECTORPLOT),"&plottype.") EQ 0
      %then %do;
         %everrmsg(&MNAME,msg=Invalid plottype selection [&plottype]);
         %goto EXIT;
      %end;
      %if "&collapse" EQ "" %then %let collapse=PERIOD;
      
      %*********************************************************;
      %* SCATTERPLOTMATRIX requires GRIDDED layout;
      %*********************************************************;
      %if "&plottype" EQ "SCATTERPLOTMATRIX" and 
          "&layout" NE "GRIDDED" %then %do;
         %everrmsg(&MNAME,msg=Plot type SCATTERPLOTMATRIX incompatible with layout [&layout]);
         %goto EXIT;       
      %end;
      
      %*********************************************************;
      %* HEATMAPPARM mixtures;
      %*********************************************************;     
      %if "&plottype" EQ "HEATMAPPARM" %then %do;
         %if "&yvar" EQ "" and &numvars LT 2 and "&group" EQ "" %then %do;
            %everrmsg(&MNAME,msg=HEATMAPPARM Required yvar input missing);
            %goto EXIT;       
         %end;
         %if "&layout" EQ "DATAPANEL" and "&group" EQ "" and &numvars EQ 1 %then %do;
             %let layout=OVERLAY;
             %everrmsg(&MNAME,msg=Heatmap Datapanel requires group [&group] or multiple measures. Switching to OVERLAY);
             /* %goto EXIT; */
         %end;         
         %if "&layout" EQ "OVERLAY" %then %do;
            %if &numvars GT 1 and "&group" NE "" and "&yvar" NE "" %then %do;
               %everrmsg(&MNAME,msg=HEATMAPPARM dimension error. Cannot input yvar group and multiple response variables);
               %goto EXIT;
            %end;
         %end;
      %end;
      
      %if "&plottype" EQ "VECTORPLOT" %then %do;
         %if "&xorigin" EQ "" %then %let xorigin=0;
         %if "&yorigin" EQ "" %then %let yorigin=0;
      %end;
      %if "&plottype" EQ "BUBBLEPLOT" %then %do;
         %let bsizestmt=;
         %let bvar=;
         %if "&bsize" EQ "" %then %let bsize=10;
         /* testing if bsize is a integer */
         %if "%verify(&bsize,0123456789)" EQ "0" %then %let bsize=%str(eval(&bsize));
         %else %let bvar=&bsize;
         %let bsizestmt=%str(size=&bsize);
      %end;
   %end;
   
   %if "&group" NE "" and "&plottype" EQ "OVERLAY" AND
          (&numvars GT 1 or "&y2var" NE "") %then %do;
         %everrmsg(&MNAME,msg=Invocation does not support group [&group] selection and multiple analysis variables);
         %goto EXIT;       
   %end;
   
   /*
   %if "&group" NE "" and "&plottype" NE "SCATTERPLOTMATRIX" and 
       (&numvars GT 1 or "&y2var" NE "") %then %do;
      %everrmsg(&MNAME,msg=Invocation does not support group [&group] selection and multiple analysis variables);
      %goto EXIT;       
   %end;
   */
   
   %if "&rows" NE "" or "&columns" NE "" %then %do;
      %if "&layout" NE "DATAPANEL" AND "&layout" NE "GRIDDED" %then %do;
         %everrmsg(&MNAME,msg=rows or column inputs only support with layouts DATAPANEL or GRIDDED.);
         %goto EXIT;           
      %end;
      %if "&rows" NE "" %then %let rowstmt=%str(rows=&rows); 
      %if "&columns" NE "" %then %let colstmt=%str(columns=&columns);
   %end;

   %*********************************************************;
   %* BARCHART BARCHARTPARM LINECHART orientation HORIZONTAL or VERTICAL ;
   %*********************************************************;   
   %if "&orient" NE "" %then %do;
      %let orientstmt=;
      %if "&plottype" EQ "BARCHART" OR 
          "&plottype" EQ "BARCHARTPARM" OR
          "&plottype" EQ "LINECHART" %then %do;
         %let orient=%upcase(&orient);
         %if "&orient" EQ "HORIZONTAL" or "&orient" EQ "VERTICAL" %then %do;
            %let orientstmt=%str(orient=&orient);
         %end;
         %else %do;
            %everrmsg(&MNAME,msg=Orientation invalid selection [&orient] [VERTICAL|HORIZONTAL|_blank_]);
            %goto EXIT;     
         %end;
      %end;
      %else %do;
         %everrmsg(&MNAME,msg=Invalid orientation [&orient] for plottype [&plottype]);
         %goto EXIT;           
      %end;
   %end;
   
   %*******************************************;
   %* groupdisplay supported with plottype barchart, barchartparm, or linechart;
   %*******************************************;
   %if "&groupdisplay." NE "" %then %do;
      %let groupdisplay=%upcase(&groupdisplay);
      %if "&groupdisplay" EQ "STACK" %then %do;
         %if "&plottype" EQ "BARCHART" or
             "&plottype" EQ "BARCHARTPARM" or
             "&plottype" EQ "LINECHART" %then %do;
             %let grpdisp=%str(groupdisplay=&groupdisplay. display=(FILL) );
         %end;
         %else %do;
            %everrmsg(&MNAME,msg=Groupdisplay [&groupdisplay.] valid for plottype [BARCHART|BARCHARTPARM|LINECHART]);
            %goto EXIT;
         %end;
      %end;
   %end;

   %*******************************************;
   %* xvar type = discrete for barchart, barchartparm, heatmapparm or linechart;
   %* otherwise xvar type = measure;
   %*******************************************;
   %if "&xvartype" EQ "" %then %do;
      %let xvartype=DISCRETE;
      /*
      %if "&plottype" EQ "BARCHART" or
          "&plottype" EQ "BARCHARTPARM" or
          "&plottype" EQ "LINECHART" or
          "&plottype" EQ "HEATMAPPARM" %then %do;
          %let xvartype=DISCRETE;
      %end;
      */
   %end;

   %*********************************************************;
   %* Legend uses varname or varlabel, Default = _VARLABEL;
   %*********************************************************;
   %if not ("&legendlab" EQ "" or
            "&legendlab" EQ "_VARNAME" or
            "&legendlab" EQ "_VARLABEL") %then %do;
      %everrmsg(&MNAME,msg=Legendlab invalid selection [&legendlab] legendlab [_VARNAME|_VARLABEL|_blank_]);
      %goto EXIT;                  
   %end;
   
   %*********************************************************;
   %* Setup transforms;
   %*********************************************************;
   %let collapse = %upcase(&collapse);
   %if not ("&collapse" EQ "PERIOD" or
            "&collapse" EQ "TOD" or
            "&collapse" EQ "NONE" or
            "&collapse" EQ "") %then %do;
      %everrmsg(&MNAME,msg=Collapse invalid selection [&collapse] collapse [PERIOD|TOD|NONE|_blank_]);
      %goto EXIT;       
   %end;
   
   %if ("&collapse" EQ "PERIOD" or 
       "&collapse" EQ "TOD" ) and "&stat" NE "" %then %do;
      %evcomment(WARNING: (&MNAME) STAT option [&stat] and collapse [PERIOD|TOD] may produce inaccurate summarization statistics);
   %end;
   %else %if "&collapse" EQ "NO" and "&stat" NE "" %then %do;
      %put WARNING: (&MNAME) Collapse EQ [NO] ignores summarization statistic [&stat]);
   %end;

   %*********************************************************;
   %* Check stat selection;
   %*********************************************************;
   %if "&stat" EQ "" %then %let stat=%str(MEAN);
   %if %index(%dqlist(MEAN MAX MIN SUM RANGE STD CV N NMISS SUMWGT CSS USS VAR DEFAULT FREQ 
CFREQ PERCENT CPERCENT PCT),"&stat.") EQ 0
   %then %do;
      %everrmsg(&MNAME,msg=Invalid statistic [&stat] See valid statistic-keywords selection);
      %goto EXIT;
   %end;   

   %*******************************************;
   %* xaxis option overrides.;
   %*******************************************;
   %if "&xaxisopt." EQ "" %then %do;
      %let xaxisopt = %str(labelposition=CENTER);
      %evcomment((&MNAME) No x axis option overides, defaults [&xaxisopt.]);
   %end;
   %else %evcomment((&MNAME) x axis option overrides, [&xaxisopt.]);    
   
   %*******************************************;
   %* yaxis option overrides.;
   %*******************************************;
   %if "&yaxisopt." EQ "" %then %do;
      %let yaxisopt = %str(labelposition=CENTER);
      %evcomment((&MNAME) No yaxis option overides, defaults [&yaxisopt.]);
   %end;
   %else %evcomment((&MNAME) y axis option overrides, [&yaxisopt.]); 
   
   %*******************************************;
   %* Layout option overrides.;
   %*******************************************;   
   %if "&layoutopt." EQ "" %then %do;
      %let layoutopt = %str();
      %evcomment((&MNAME) No layout option overides, defaults [&layoutopt.]);
   %end;
   %else %evcomment((&MNAME) layout option overrides, [&layoutopt.]);
   
   %*******************************************;
   %* Plot option overrides.;
   %*******************************************;   
   %if "&plotopt." EQ "" %then %do;
      %if "&plottype" EQ "SERIESPLOT" %then %let plotopt = %str(lineattrs=(thickness=2));
      %if "&plottype" EQ "HEATMAPPARM" %then
         %let plotopt =%str(colormodel=threecolorramp display=(fill outline) outlineattrs=(color=cxf7f7f7));
      %evcomment((&MNAME) No Plot statement option overides, defaults [&plotopt.]);
   %end;
   %else %evcomment((&MNAME) Plot option overrides, [&plotopt.]);

   %*******************************************;
   %* Heatmap input ranges.;
   %*******************************************;
   %let hmrange = %upcase(&hmrange);
   %if "&hmrange" EQ "TRUE" %then %do;
      %if "&plottype." EQ "HEATMAPPARM" %then %do;
         %if  "&hmrangemin" EQ "" or "&hmrangemax" EQ "" %then %do;
            %everrmsg(&MNAME,msg=Heatmap ranges require valid Min [&hmrangemin] and Max [&hmrangemax]);
            %goto EXIT;         
         %end;
      %end;
      %else %do;
         %everrmsg(&MNAME,msg=Heatmap ranges only valid for plottype HEATMAPPARM);
         %goto EXIT;   
      %end;
   %end;

   %*********************************************************;
   %* Check valid dataset exists with Datetime and other requested variables;
   %*********************************************************;
   %if "&caller" NE "EVSTP" %then %do;
   %if %sysfunc(exist(&dataset)) or
       %sysfunc(exist(&dataset,view)) 
       %then %do;
           %let _dsid = %sysfunc(open(&dataset.));
           %do i=1 %to %num_tokens(&evallvars.);
                %if &debug GT 1 %then %put DEBUG: (&MNAME) %scan(&evallvars.,&i);
                %if not %sysfunc(varnum(&_dsid,%scan(&evallvars.,&i.))) %then %do;
                   %let intcheck=%upcase(%scan(&evallvars.,&i));
                   %if "&intcheck" NE "TIME" AND "&intcheck" NE "_CPTYPE" %then %do;
                      %everrmsg(&MNAME,msg=Variable [%scan(&evallvars.,&i.)] [&intcheck] missing from Dataset [&dataset]);
                      %let rc = %sysfunc(close(&_dsid));
                      %goto EXIT;
                   %end;
                %end; 
           %end; 
      %end;
      %else %do;
         %everrmsg(&MNAME,msg=Dataset [&dataset] missing);
         %goto EXIT;
      %end;
   %let rc = %sysfunc(close(&_dsid));
   %end;
   
   %*********************************************************;
   %* Validate and/or calculate begin and end parms/timestamps;
   %*********************************************************;
   %if &debug GT 1 %then %put DEBUG: (&MNAME) begin [&begin] end [&end];  
   %if "&begin" EQ "" or "&end" EQ "" %then %do;
      %everrmsg(&MNAME,msg=Begin [&begin] or End [&end] time window missing);
      %goto EXIT;
   %end;
   
   %if "&todbins" NE "" %then %do;
      %let todbins = %sysfunc(ceil(&todbins));
      %if NOT (&todbins GT 0 and &todbins LT 1441) %then %do;
         %everrmsg(&MNAME,msg=todbins [&todbins] Positive integer (1 - 1440));
         %goto EXIT;
      %end;
   %end;
   
   %********************************************************;
   %* Adjusting for requested time zone offset             *;
   %* LOCAL = use local timezone of host/platform          *;
   %* DATA = use timezone associated with data (per record)*;
   %* UTC or GMT = Use 0 seconds, Data Mart in UTC          *;
   %* or valid integer value = number of offset seconds    *;
   %********************************************************;
   %evsettz;

   %********************************************************;
   %* Where clause handling;
   %*********************************************************;
   %evwheval(where=&where,begin=&begin,end=&end,tzoneoff=&tzoneoff);
  
   %******************************************************************;
   %* Establish temporary view with subsetting (and index support)   *;
   %******************************************************************;
   data work._EVSCRATCH;
      set &dataset;
      &whglobal;
      &whlocal;
      %evmkfmt(&formats);
      %evmklab(&labels);
      run;
   %if &syserr GT 4 %then %goto ERROR;
   %let dataset = work._EVSCRATCH;
   %let _dsid = %sysfunc(open(&dataset.));
      %if "%sysfunc(attrn(&_dsid,NOBS))" = "0" %then %do;
         %let rc = %sysfunc(close(&_dsid));
         %goto NOOBS;
      %end;
   %let rc = %sysfunc(close(&_dsid));
   
   %*********************************************************;
   %* checking for a consistent plot request and dimensions for DATAPANEL, REGION, and OVERLAY;
   %*  too many dimensions for the requested plot inputs;
   %*  too few dimensions for the requested plot variable inputs;
   %*  invert data setting;
   %*  must invert data if numvars analysis variables > 1 or let the group determine plot number;
   %*********************************************************;
   %let invertdata=YES;
   %if "&layout" EQ "DATAPANEL" %then %do;
      %if &numvars LT 2 %then 
         %let invertdata=NO;
        /* %if "&group" NE "" %then %let invertdata=NO; */
      %if "&group" EQ "" and &numvars LT 2 %then %do;
         %let layout=OVERLAY;
         %evcomment((&MNAME) DATAPANEL layout requested with no subgroup discrete input, switching layout to OVERLAY);
      %end;
   %end;
   %if "&layout" EQ "OVERLAY" %then %do;
      %if "&group" NE "" and &numvars GT 1 %then %do;
         %if %index(%dqlist(BOXPLOT BARCHART LINECHART PIECHART HISTOGRAM DENSITYPLOT),"&plottype.") NE 0
         %then %do;
             %everrmsg(&MNAME,msg=OVERLAY and computed plottype [&plottype] incompatible with subgrouping and multiple measures);
             %goto EXIT;
         %end;
         %else %do;
            %let layout=DATAPANEL;
            %evcomment((&MNAME) OVERLAY layout requested with both a subgrouping input and multiple measures, switching to DATAPANEL);  
         %end;
      %end;
   %end;
   %if "&layout" EQ "GRIDDED" %then %do;
       %let invertdata=NO;
       %if "&plottype" EQ "SCATTERPLOTMATRIX" %then %let invertdata=NO;
       %if "&yaxislab" NE "" or "&xaxislab" NE "" %then 
          %put WARNING: (&MNAME) layout [GRIDDED] does not support axis label overrides;
   %end;
   
   %if &numvars EQ 0 %then %let axis=0;
   %evcomment((&MNAME) Running datetime extraction and summarization);
   %******************************************************************;
   %* Transpose the data and subset if necessary;
   %******************************************************************;
   %if &debug >1 %then %do;
      %evcomment((&MNAME) evxform inputs, period=[&period], collapse=[&collapse], stat=[&stat], 
      cl_nam=[&group], axis=[&axis], invert=[&invertdata]);
   %end;
   %evxform(&dataset,period=&period,by=&by,collapse=&collapse,_rc=everror,xvar=&xvar,xvartype=&xvartype,
      stat=&stat,axis=&axis,cl_nam=&group &bvar,invert=&invertdata.,outds=tmpdata,keep=&yvar &y2var);
   %let dataset=tmpdata;
   %if "&everror" NE "0" %then %do;
      %if "&everror" = "2" %then %goto NOOBS;
      %else %goto ERROR;
   %end;
   
   %*********************************************************;
   %* Draw chart using appropriate options for appearance;
   %*********************************************************;
   %let grpstmt=;
   %if "&group" NE "" %then %let grpstmt = %str(group = &group);
   %let legtitlestmt=;
   %if "&legendtitle" NE "" %then %let legtitlestmt = %str(title = "&legendtitle");
   %let leglabstmt=;
   %if "&legendlab" NE "" %then %do;
      %if "&legendlab" EQ "_VARNAME" %then %let leglabstmt=%str(legendlabel="&var1.");
      %else;
   %end;
   %if &debug GT 1 %then %put DEBUG: grpstmt [&grpstmt];
   %let legend=NO;

   ods path (prepend) work.templat(update);
   
 %goto &layout;
 
 %*********************************************************;
 %* GRIDDED;
 %*********************************************************; 
 %GRIDDED:
    %if "&group" NE "" %then %do;
      %let grpstmt = %str(group = &group);
    %end;
    %else %if "&invertdata" EQ "YES" %then %do;
      /* should never happen */
      %let grpstmt = %str(group = _cptype);
    %end;

    proc template;
       define statgraph ev&plottype;
       begingraph / attrpriority=color;
          %evodstext;
           %if "&hmrange" EQ "TRUE" %then %do;
               %str(rangeattrmap name="heatmaprange";);
               %if "&hmrangekeys" NE "" %then %do;
                  %let nvp=;
                  %do i=1 %to %num_tokens(&hmrangekeys);
                     %let nvp=%scan(%nrquote(&hmrangekeys.),&i,' ',);
                     %if "&nvp" NE "" %then %do;
                        %let hmname = %scan(%nrquote(&nvp.),1,=);
                        %let hmvalue = %scan(%nrquote(&nvp.),2,=);
                        %str(range &hmname / rangecolormodel=&hmvalue;);
                     %end;     
                  %end;
               %end;
               %str(range &hmrangemin - &hmrangemax / rangecolormodel=threecolorramp;);
               endrangeattrmap;
               rangeattrvar attrvar=&var1. var=&var1. attrmap="heatmaprange" ;             
           %end;
          layout gridded  / &rowstmt. &colstmt. &layoutopt.
          ;
          %let dislegend=;
       %goto &plottype.G;
       
       %SCATTERPLOTG:
       %SERIESPLOTG:
       %BOXPLOTG:
       %STEPPLOTG:
       %NEEDLEPLOTG:
          %do i=1 %to &numvars.;
             &plottype x=&xvar. y=&&var&i. / &grpstmt. &grpdisp. &plotopt. name="&plottype";
          %end;
             %if "&group" NE "" %then %let dislegend=%str(discretelegend "&plottype" / &legtitlestmt. ;);
          %goto NGSTEP; 

       %BARCHARTG:
       %LINECHARTG:
       %PIECHARTG:
          %if "&var1" EQ "" %then
             &plottype category=&xvar. / &grpstmt. &plotopt. &grpdisp. &statstmt. &orientstmt. name="&plottype"; ;        
       %BARCHARTPARMG:
          %do i=1 %to &numvars.;
	        &plottype category=&xvar. response=&&var&i. / &grpstmt. &plotopt. &grpdisp. &statstmt. &orientstmt. name="&plottype";
          %end;
          %if "&group" NE "" %then %let dislegend=%str(discretelegend "&plottype" / &legtitlestmt. ;);          
       %goto NGSTEP;
          
       %SCATTERPLOTMATRIXG:
          &plottype &evrptvars. / &grpstmt. &plotopt name="&plottype";
          %if "&group" NE "" %then %do;
             %let dislegend=%str(discretelegend "&plottype" / title="&legendtitle. Period: &period";);
          %end;
          %else %do;
             %let efnote=%str(entryfootnote HALIGN=RIGHT "Period: &period");
          %end;
          %goto NGSTEP;
       
       %HEATMAPPARMG:
          %let grpstmt=;
          %let colorstmt = %str(colorresponse = &var1);    
          %if "&group" NE "" %then %do;
             %let ystmt= %str(y=&group.);
          %end;
          %else %if "&yvar" NE "" %then %do;
	        %let ystmt = %str(y=&yvar);
          %end;
          /* should never happen */
          %else %let ystmt = %str(y=_cptype);
          
          %if &numvars EQ 1 AND "&legendtitle" EQ "" %then %let legtitlestmt=%str(title = "&var1.");
          %do i=1 %to &numvars.;
             &plottype x=&xvar. &ystmt. colorresponse=&&var&i& / &plotopt. name="HEATMAPPARM" ;
          %end;
          continuouslegend "HEATMAPPARM" / &legtitlestmt. autoalign=AUTO orient=horizontal location=INSIDE;
          %let dislegend=%str(discretelegend "&plottype" / title="&legendtitle. &period";);
          %goto NGSTEP;
       
       %DENSITYPLOTG:
       %HISTOGRAMG:
       %FRINGEPLOTG:
          %if "&var1." EQ "" %then %do;
 		   &plottype &xvar. / &grpstmt. &grpdisp. &plotopt. name="&plottype";          
          %end;
          %else %do;
             %do i=1 %to &numvars.;
                /* %if &numvars GT 1 %then %let xvar=_cptype;*/
		      &plottype &&var&i. / &grpstmt. &grpdisp. &plotopt. name="&plottype";
             %end;
          %end;
          %let dislegend=%str(discretelegend "&plottype" / &legtitlestmt. ;);
          %goto NGSTEP;    
       
       %VECTORPLOTG:
          %do i=1 %to &numvars.;
             &plottype x=&xvar. y=&&var&i. xorigin=&xorigin yorigin=&yorigin / &grpstmt. &grpdisp. &plotopt. &leglabstmt. name="&plottype";
          %end;
          %if "&group" NE "" %then %let dislegend=%str(discretelegend "&plottype" / &legtitlestmt. ;);
       
       %NGSTEP:
        /*  %if (&numvars EQ 1 and "&group" NE "") %then */
          %str(&dislegend);
       endlayout;
       %str(&efnote);
       endgraph;
    end;
    %goto GREND;
  
 %*********************************************************;
 %* OVERLAY;
 %*********************************************************; 
 %OVERLAY:
    %if "&group" NE "" %then %let grpstmt = %str(group=&group.);
    %else %if &numvars > 1 %then %do;
       %let grpstmt = %str(group = _cptype);
       %if "&yaxislab" EQ "" %then %let yaxislab=;
    %end;
    
    proc template;
       define statgraph ev&plottype;
       begingraph;
          %evodstext;
          %if "&hmrange" EQ "TRUE" %then %do;
              %str(rangeattrmap name="heatmaprange";);
              %if "&hmrangekeys" NE "" %then %do;
                 %let nvp=;
                 %do i=1 %to %num_tokens(&hmrangekeys);
                    %let nvp=%scan(%nrquote(&hmrangekeys.),&i,' ',);
                    %if "&nvp" NE "" %then %do;
                       %let hmname = %scan(%nrquote(&nvp.),1,=);
                       %let hmvalue = %scan(%nrquote(&nvp.),2,=);
                       %str(range &hmname / rangecolormodel=&hmvalue;);
                    %end;     
                 %end;
              %end;
              %str(range &hmrangemin - &hmrangemax / rangecolormodel=threecolorramp;);
              endrangeattrmap;
              rangeattrvar attrvar=&var1. var=&var1. attrmap="heatmaprange" ;             
           %end;
          
          layout overlay / cycleattrs=true &layoutopt.
          xaxisopts=( &xaxisopt. 
                %if "&xaxislab" NE "" %then %do;
                   %if "&xaxislab" EQ "_VARNAME" %then %str(label = "&xvar.");
                   %else %if "&xaxislab" EQ "_VARLABEL" %then;
                   %else %str(label = "&xaxislab.");
                %end;          
          discreteopts=(tickvaluefitpolicy=thin) )
       
          %if "&plottype" EQ "DENSITYPLOT" or
               "&plottype" EQ "HISTOGRAM" %then %do;
          %end;
          %else %do;       
             yaxisopts=( &yaxisopt.
                %if "&yaxislab" NE "" %then %do;
                   %if "&yaxislab" EQ "_VARNAME" %then %str(label = "&var1.");
                   %else %if "&yaxislab" EQ "_VARLABEL" %then;
                   %else %str(label = "&yaxislab.");
                %end;
                );
          %end;
          ;
    
    %goto &plottype.O;
    
       %BARCHARTO:
       %LINECHARTO:      
       %BARCHARTPARMO:
          %if "&var1" NE "" %then  %let response = %str(response=&var1.);   
             &plottype category=&xvar. &response. / &grpstmt. &plotopt. &grpdisp. &statstmt. &orientstmt. name="&plottype"; ;
          %if "&y2var" NE "" %then %do;
             seriesplot x=&xvar. y=&y2var. / yaxis=y2 /*&grpstmt.*/ 
                %if "&legendlab" EQ "_VARNAME" %then %str(legendlabel="&y2var");
             &plotopt. name="overlay2";          
             %let n2leg=%str("overlay2");
          %end;    
          %let dislegend=%str(discretelegend "&plottype" &n2leg / DISPLAYCLIPPED=TRUE &legtitlestmt. ;);          
          %goto NEXTO;
          
       %SCATTERPLOTO:
       %SERIESPLOTO:
       %BOXPLOTO:
       %STEPPLOTO:
       %NEEDLEPLOTO:
          &plottype x=&xvar. y=&var1. / &grpstmt. &grpdisp. &plotopt. &leglabstmt. name="&plottype";
          %if "&y2var" NE "" %then %do;
             &plottype x=&xvar. y=&y2var. / yaxis=y2 /*&grpstmt.*/ 
                %if "&legendlab" EQ "_VARNAME" %then %str(legendlabel="&y2var");
             &plotopt. name="overlay2";          
             %let n2leg=%str("overlay2");
          %end;
          %let dislegend=%str(discretelegend "&plottype" &n2leg / DISPLAYCLIPPED=TRUE &legtitlestmt. ;);   
          %goto NEXTO;

       %DENSITYPLOTO:
       %HISTOGRAMO:
       %FRINGEPLOTO:
          %if "&var1." EQ "" %then %do; 
             &plottype &xvar. / &grpstmt. &grpdisp. &plotopt. name="&plottype";
          %end;
          %else %do; 
             &plottype &var1. / &grpstmt. &grpdisp. &plotopt. name="&plottype";
          %end;
          %let dislegend=%str(discretelegend "&plottype" / DISPLAYCLIPPED=TRUE &legtitlestmt. ;);
          %goto NEXTO;       
           
       %HEATMAPPARMO:
       %BUBBLEPLOTO:
          %let grpstmt=;
          
          %let ystmt = %str(y=_cptype);
          %if "&group" NE "" %then %let ystmt = %str(y=&group);
          %else %if "&yvar" NE "" %then %let ystmt = %str(y=&yvar);
          %let colorstmt = %str(colorresponse = &var1.);
       
          %if "&plottype" EQ "HEATMAPPARM" %then %do;
             &plottype x=&xvar. &ystmt. &colorstmt. / &plotopt. name="&plottype";
          %end;
          %else %do;  /* its a bubbleplot */ 
             &plottype x=&xvar. &ystmt. &bsizestmt. / &colorstmt. bubbleradiusmin=4 bubbleradiusmax=4 display=(fill) &plotopt. name="&plottype"; 
          %end;
          
          %if &numvars EQ 1 AND "&legendtitle" EQ "" %then %let legtitlestmt=%str(title = "&var1.");
          continuouslegend "&plottype" / &legtitlestmt.;
          %let dislegend=%str(discretelegend "&plottype" / DISPLAYCLIPPED=TRUE &legtitlestmt. ;);       
          %goto NEXTO;    
       
       %VECTORPLOTO:
          &plottype x=&xvar. y=&var1. xorigin=&xorigin yorigin=&yorigin / &grpstmt. &grpdisp. &plotopt. &leglabstmt. name="&plottype";
          %let dislegend=%str(discretelegend "&plottype" / DISPLAYCLIPPED=TRUE &legtitlestmt. ;);
          %goto NEXTO;
          
       %NEXTO:
         %if "&group" NE "" or 
             "&yvar" NE "" or
             &numvars GT 1 %then
            %str(&dislegend);

       endlayout;
       endgraph;
    end; 
    %goto GREND;  
 
  %*********************************************************;
  %* DATAPANEL;
  %*********************************************************; 
  %DATAPANEL:
  
     %if "&group" NE "" %then %let grpstmt = %str(classvars=(&group));
     %else %if "&var2" NE "" %then %do;
        %let grpstmt = %str(classvars=(_cptype));
        %if "&yaxislab" EQ "" %then %let yaxislab=%str( );
     %end;
     %else %do;
         %everrmsg(&MNAME,msg=Heatmap Datapanel parm mixmatch);
         %goto EXIT;           
     %end;
     
     proc template;
        define statgraph ev&plottype;
        begingraph;
           %evodstext;
           layout datapanel &grpstmt / &rowstmt. &colstmt. rowdatarange=union 
              cellwidthmin=65 cellheightmin=65  headerlabeldisplay=value 
              &layoutopt.
              rowaxisopts=( griddisplay=on &rowaxisopts.               
                %if "&yaxislab" NE "" %then %do;
                   %if "&yaxislab" EQ "_VARNAME" %then %str(label = "&var1.");
                   %else %if "&yaxislab" EQ "_VARLABEL" %then;
                   %else %str(label = "&yaxislab.");
                %end; 
                %else %str(label = " ");
                 )
              columnaxisopts= ( &columnaxisopts.
                 %if "&xaxislab" NE "" %then %do;
                    %if "&xaxislab" EQ "_VARNAME" %then %str(label = "&xvar.");
                    %else %if "&xaxislab" EQ "_VARLABEL" %then;
                    %else %str(label = "&xaxislab.");
                 %end; 
                 %else %str(label = "&xvar.");  
              
                 discreteopts=(tickvaluefitpolicy=truncatethin)
                 )
              ;
  
              %*********************************************************;
              %* layout prototype;
              %*********************************************************;              
              layout prototype / cycleattrs=true;
              
              %if "&plottype" EQ "HEATMAPPARM" %then %do;
                 %let colorstmt = %str(colorresponse = &var1);
                 %let ystmt = %str(y=_cptype);
                 %if "&yvar" NE "" %then %let ystmt = %str(y=&yvar);
                 &plottype x=&xvar. &ystmt. &colorstmt. / &plotopt. name="_cptype";   
              %end;
              /**** Not HEATMAPPARM ****/
              %else %do;
                 %if "&var1" EQ "" %then %do;
                    &plottype category=&xvar. / &orientstmt. &leglabstmt. &plotopt. name="&xvar."; ;
                 %end;
                 %else %do;
                    %if "&invertdata" EQ "YES" %then %let numvars=1;
                    %if %index(&grpstmt.,classvars) %then %let grptypestmt = %str(group = _cptype);
                    %do i=1 %to &numvars.;
                       &plottype x=&xvar. y=&&var&i. / &grptypestmt. &orientstmt. &leglabstmt. &plotopt. name="&&var&i.";
                    %end;
                 %end;
                 %if "&y2var" NE "" %then %do;
                    seriesplot x=&xvar. y=&y2var. / yaxis=y2 /*&grpstmt.*/ 
                    %if "&legendlab" EQ "_VARNAME" %then %str(legendlabel="&y2var");
                    &plotopt. name="overlay2";          
                    %let n2leg=%str("overlay2");
                 %end;        
              %end;
              endlayout;   /* end of prototype */
          
              %if "&plottype" EQ "HEATMAPPARM" %then %do;
                 sidebar / align=right;
                    continuouslegend "_cptype" / &legtitlestmt. location=INSIDE;
	            endsidebar;
		    %end;
              %else %do;
                 /* %if &numvars GT 0 %then %do;  */
                 %if "&group" NE "" %then %do; 
                    sidebar / ;
                       discretelegend 
                       %do i=1 %to &numvars.;
                          %str("&&var&i")
                       %end;
                      &n2leg / border=false displayclipped=true &legtitlestmt;
                    endsidebar;
                 %end;
              %end;  
           endlayout;       /* of datapanel */
        endgraph;
     end; 
    %goto GREND;  
 
 %*********************************************************;
 %* REGION;
 %*********************************************************; 
 %REGION:
    %if "&group" NE "" %then %let grpstmt = %str(group = &group);
    %if &numvars > 1 %then %do;
       %let xvar = _cptype;
    %end;
    
    proc template;
       define statgraph ev&plottype;
       begingraph;
          %evodstext;
          layout region / &layoutopt.
          ;
    
    %goto &plottype.R;

       %PIECHARTR:
          &plottype category=&xvar. response=&var1. / &grpstmt. &plotopt. &grpdisp. &statstmt. &orientstmt. name="&plottype"; 
             %let dislegend=%str(discretelegend "&plottype" / DISPLAYCLIPPED=TRUE &legtitlestmt. ;);
          %goto NEXTR;
          
       %NEXTR:
         %if "&group" NE "" or
             &numvars GT 1 %then
            %str(&dislegend);

       endlayout;
       endgraph;
    end; 
    %goto GREND;  
  
 
 /* Not supported at this time 
 %DATALATTICE:
 %if "&var2" NE "" %then %let grpstmt = %str(group=_cptype);
    proc template;
       define statgraph seriesplot;
       begingraph;
       entrytitle "title1";
       layout datalattice rowvar= columnvar= ;
          layout prototype;
    ;
         &plottype x=&xvar. y=&var1 / &grpstmt. name="legend";
        /* %if "&var2" NE "" %then %str(seriesplot x=datetime y=&var2 / name="&var2.";);*/
         %if "&var3" NE "" %then %str(seriesplot x=datetime y=&var3 / name="&var3.";);
         %if "&grpstmt" NE "" %then %str(discretelegend "legend";);
          endlayout;
       endlayout;
       endgraph;
    end; 
    %goto GREND;   
 */
 
%*********************************************************;
%* GRENDER;
%*********************************************************;  
%GREND:

   %if "&imagename" NE "" %then %str(ods graphics on / imagename="&imagename.";);
   ods proclabel = "&title1.";
   proc sgrender data=&dataset template=ev&plottype objectlabel="";

   &bystmt.
   run;
 
   %goto EXIT;
   
%NOOBS:

   %noobsrpt(inds=&inds,begin=&begin=,end=&end,where=&where);
   %goto EXIT;

 %ERROR:
   /*----------------------------------------------------------------
    | Syserr Description;
    |
    | 0 Execution completed successfully. 
    | 1 Execution was canceled by a user with RUN or CANCEL. 
    | 2 Execution was canceled by a user with ATTN or BREAK command. 
    | 3 An error in a program run in batch or non-interactive mode caused 
    |   SAS to enter syntax-check mode. 
    | 4 Execution completed successfully with warning messages. 
    | >4 An error occurred, value returned is procedure dependent.
    +----------------------------------------------------------------+*/
    %everrmsg(&MNAME,msg= %superq(syserrortext));
    %let everror=0;

 %GDONE:
 %EXIT:
   %*********************************************************;
   %* Show invocation parameters;
   %*********************************************************;
   %let showparm = %upcase(&showparm);
   %evcomment((&MNAME) Showparms code then exit with cleanup.);
   %if "&showparm." EQ "YES" or "&showparm." EQ "ONLY" %then %do;
      %let dataset=&inds;
      data parms;
         format parameter $32. value $80.;
          label parameter = "&sysmacroname parameter" value = 'value';
      
         %do i=1 %to %num_tokens(&parmlist);
            %let pvalue=&&&%scan(%nrquote(&parmlist.),&i).;
             %if "&pvalue" NE "" %then %do;
                parameter = "%scan(&parmlist.,&i)";
                value = " %bquote(&pvalue) ";
                output;
             %end;
         %end;
      run;
/*  
      Proc report data=parms nowd headline headskip
         style(header) = [fontweight=bold];
         %if "&title1." NE "" 
         %then column ("&title1." parameter value);
         %else column (parameter value);
         ;
      run;
rnb replace with proc sql
*/
   %end;
   %*********************************************************;
   %* Cleanup scratch datasets;
   %*********************************************************;
   %if &debug LT 3 %then %do;
   proc datasets library=work memtype=(data view) nolist nowarn;
      delete parms tmpdata _EVSCRATCH noobs / memtype=data;
      run; quit;
   %end;
      
   %evsetdebug(RESET);
   %evTIMLOG( STOP, &MNAME );

   options &schkset OBS=&obsset;

%mend evtmpplate;