/***************************************************/
/* Copyright (c) 2014 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

/*---------------------------------------------------------------------------
 *  PRODUCT: SAS Environment Manager
 *  NAME:    %EVUTILS
 *  DATE:    09Jun2013
 *  DESC:    The %EVUTILS macro must be specified at the beginning of each
 *           stored process that is used for SAS Environment Manager reports.
 *           This macro prepares the SAS environment for reporting by doing
 *           the following:
 *           
 *           # It sets some internal macro variables.
 *           # It defines several utility macros that can be used to help
 *             generate reports and charts.
 *
 *  PARAMETERS
 *
 *  UPDATED:
 *-------------------------------------------------------------------------*/

%macro evutils;

  %let nulllbl=00;
  %if "%evhsttyp" EQ "WIN" %then %let nulllbl =1E;
  %evcomment((EVUTILS) host [&SYSHOSTNAME] hosttype [%evhsttyp] null character <&nulllbl>);

/*---------------------------------------------------------------------------
  * NAME: ev_startdate
  *                             
  * DESC: 
  *-------------------------------------------------------------------------*/
%macro ev_startdate(date,minus);
	%qcmpres(%sysfunc(putn(%sysfunc(intnx(Day,&date,&minus)),date8.)))
%mend ev_startdate;

/*---------------------------------------------------------------------------
  * NAME: removew
  *                             
  * DESC: removes words from string, assumes ws delims, use inline
  *       for detail explaination, purchase regexp textbook
  *-------------------------------------------------------------------------*/
%macro removew(string,target);
   %local target_regex cleansed_str;
   /* create regex string */
   %let target_regex=\b%sysfunc(prxchange(s/ +/ ?\b|\b/oi,-1,&target))\b;
 
   /* use SAS regex function prxchange() to remove words from target */
   %let cleansed_str=%sysfunc(prxchange(s/&target_regex//oi,-1,&string));

&cleansed_str.
%mend removew;

/*---------------------------------------------------------------------------
  * NAME: %checkds
  *                             
  * DESC: Confirms ds exists and has gt zero observations        
  *-------------------------------------------------------------------------*/
%macro checkds(ds2chk=,nuobs=noobs,_rc=);
   %local MNAME _dsid any;
   %let mname=checkds;
   %if %sysfunc(exist(&ds2chk.)) or %sysfunc(exist(&ds2chk.,view)) 
   %then %do;   
     %let _dsid = %sysfunc(open(&ds2chk.));
     %let &_rc = %sysfunc(attrn(&_dsid,ANY));     /* -1 no vars*/
     %let &nuobs = %sysfunc(attrn(&_dsid,NOBS));
     %if &&&_rc EQ -1 %then %do;
        %everrmsg(&MNAME,msg= Dataset [&ds2chk] contains no variables);     
     %end;
     %let _lrc = %sysfunc(close(&_dsid));
   %end;
   %else %do;
      %everrmsg(&MNAME,msg= Missing dataset [&ds2chk]);
      %let &_rc=-1;
   %end;
%mend checkds;

/*---------------------------------------------------------------------------
  * NAME: %noobsrpt
  *                             
  * DESC: Displays noobs report        
  *-------------------------------------------------------------------------*/
%macro noobsrpt(inds=,begin=,end=,where=,caller=);

   %if "&caller" EQ "" %then %let caller=&MNAME;
   %put *******************************************************;
   %put WARNING: (&caller) Dataset &inds. 0 observations.;
   %put *******************************************************;
   
   data noobs;
      format msg $80.;
      label msg="The result of the query, including subsets, produced 0 observations for your report /
           Report=[&caller] /     Input Dataset=[&inds]";
      msg = "There could be several reasons, including:"; Output;
      msg = " 1. No valid records"; Output;
      msg = " 2. An invalid date range was specified: Begin=[&begin] End=[&end]"; Output;
      msg = " 3. Begin datetime set to today, typically ETL's run nightly"; Output;
      msg = " 4. The where statement returned 0 observations"; Output;
      msg = "    where=[&where]"; Output;
      msg = " 5. The measures and categories may have all missing values"; Output;
      
   ods proclabel ="[&caller] Result query produced 0 observations" NOPROCTITLE;
   /*
   Proc report data=noobs nowd headline headskip box split='/'
      style(header) = [fontweight=bold] contents='';
      column msg;
   run;
   rnb replace with proc sql*/
   ods proclabel = "" PROCTITLE;
   Proc datasets library=work nolist nowarn;
      delete noobs / memtype=data;
   run;
%mend;

/*---------------------------------------------------------------------------
  * NAME: %add_dt2str
  *                             
  * DESC: Adds the date and time to string in format _yyyymmdd_hhmm        
  *-------------------------------------------------------------------------*/
%macro add_dt2str(str);
   &str._%sysfunc(datetime(),B8601DN8.)_%sysfunc(compress(%sysfunc(time(),tod5.),":"))
%mend add_dt2str;

/*---------------------------------------------------------------------------
  * NAME: %num_tokens
  *                             
  * DESC: Macro to count num of tokens in a string.
  *          
  * UPDATED: 14Sept2013      rnb                              
  *-------------------------------------------------------------------------*/
%macro num_tokens(words, delim=%str( )); 
  %local count; 
  %let count = 1; 
  %do %while (%length(%scan(&words, &count, &delim)) > 0); 
    %let count = %eval(&count + 1);
  %end; 
  %let count = %eval(&count - 1);
  &count
%mend num_tokens;

/*---------------------------------------------------------------------------
  * NAME: msg2stdout
  *                             
  * DESC: Macro to send text string to stdout when running in batch.
  *          
  * UPDATED: 29Sept2013      rnb                              
  * UPDATED: 04OCT2013       gzs                             
  *-------------------------------------------------------------------------*/
%macro msg2stdout(msg,dest=STDOUT,debug=N);

  %let fid=%sysfunc(fopen(&dest,O));
  %put %sysfunc(sysmsg());
  %let rc1=%sysfunc(fput(&fid,&msg));
  %let rc2=%sysfunc(fwrite(&fid));
  %let rc3=%sysfunc(fclose(&fid));

  %if &debug gt 2 %then %put DEBUG: FID=&fid RC1=&rc1 RC2=&rc2;;
%mend msg2stdout;

/*---------------------------------------------------------------------------
  * NAME: create_dir
  *                             
  * DESC: Macro to create directory given parent path and directory name.
  *-------------------------------------------------------------------------*/
%macro create_dir(dirnam,parentpath);
   %let MNAME=create_dir;
   %let rc = %sysfunc(filename(pathFR,&parentpath.));  /* zero if successful */
   %let rc = %sysfunc(fexist(&pathFR)); /* 1 if good and exists */
   %if "&rc" = "0" %then %do;
      %evcomment(ERROR: (&MNAME) parentpath [&parentpath] does not exist.);
      %goto EXIT;
   %end;
   %let rc = %sysfunc(filename(memFR,&parentpath./&dirnam));  /* zero if successful */
   %let rc = %sysfunc(fexist(&memFR)); /* 1 if good and exists */
   %if "&rc" EQ "0" %then %do;
      /* attempt to create returns path if successful, otherwise blank */
      %let rc = %sysfunc(dcreate(&dirnam,&parentpath.));
	 %if "&rc" EQ "" %then %do;
        %evcomment(ERROR: (&MNAME) Failed to create directory [&dirnam], in path [&parentpath]);
	 %end;
	 %else %do;
        %evcomment((&MNAME) Successfully created directory [&dirnam], in path [&parentpath]);
	 %end;
   %end;
   %else %do;
      %evcomment((&MNAME) Directory [&dirnam], exists in path [&parentpath]);
   %end; 
   %EXIT:
%mend create_dir;

/*---------------------------------------------------------------------------
  * NAME: %ev_createVarList
  *                             
  * DESC: Macro to define varlist from stp prompting.
  *          
  * UPDATED: 14Aug2013      rnb                              
  *-------------------------------------------------------------------------*/
%macro ev_createVarList(varname=,maxnum=1);
   %local I VARNAME;
   
   %************************;
   %* test, if varlist was not defined;
   %************************;
   %if "%symexist(&varname)" EQ "0" %then %do;
      %global &varname &varname.0;
	  %let &&VARNAME.0=0;
   %end;
   %else %do;
      %************************;
      %* test, if only one item - array does not exist;
      %************************;
      %if ("%symexist(&VARNAME.0)" EQ "0") %then %do;
         %global &varname.0 &varname.1;
         %let &&VARNAME.0 = 1;
         %let &&VARNAME.1 = &&&VARNAME;
      %end;
   %end;

   %global &&VARNAME.0;
   %do I = 1 %to &maxnum.;
      %global &VARNAME.&I.;
   %end;

   %EXIT:
%mend ev_createVarList;

/*---------------------------------------------------------------------------
  * NAME: %ev_createVarString
  *                             
  * DESC: Macro to define varstring from stp prompting.
  *          
  * UPDATED: 14Jul2014      rnb                              
  *-------------------------------------------------------------------------*/
%macro ev_createVarString(varname=,str=,maxnum=1);
   %local I;
   %global &str;
   
   %************************;
   %* test, if varlist was not defined;
   %************************;
   %if "%symexist(&varname)" EQ "0" %then %do;
	  %let &str=;
   %end;
   %else %do;
      %************************;
      %* test, if only one item - array does not exist;
      %************************;
      %if ("%symexist(&VARNAME.0)" EQ "0") %then %do;
         %let &str = %str(&&&VARNAME);
      %end;
   %end;
   %if "&maxnum" EQ "" %then %goto EXIT;
   %do I = 1 %to &maxnum.;
      %let &str=%str(%trim(&&&str) &&&VARNAME.&I.);
   %end;

   %EXIT:
%mend ev_createVarString;

/*---------------------------------------------------------------------------
  * NAME: %ev_promptselects
  *                             
  * DESC: Macro to retrieve multi selects from a prompt.
  *          
  * UPDATED: 11Oct2013      rnb                              
  *-------------------------------------------------------------------------*/
%macro ev_promptselects(varname=);
  %if %symexist(&varname._count) %then %do;
     %if &&&varname._count. GT 1 %then %do;
        %let &varname=&&&varname.1;
        %do i = 2 %to &&&varname._count;
          %let &varname=%str(&&&varname. , &&&varname&i.);
        %end;
     %end;
  %end;
  %if &evdebug GT 1 %then %put DEBUG: &varname. <&&&varname.>;
%mend ev_promptselects;

/*---------------------------------------------------------------------------
  * NAME: %obsnvars
  *                             
  * DESC: Macro to retrieve number of observations or variables from dataset.
  *       output is value to be used in assignment or displayed
  *          
  * UPDATED: 26Feb2015      rnb                             
  *-------------------------------------------------------------------------*/
%macro obsnvars(ds,type=OBS);
   %local nobs nvars;
   %if "&ds" EQ "" %then %do;
      %put NOTE: (obsnvars) Missing dataset name [&ds], Exiting...;
      %goto EXIT;
   %end;
   %if "&type" NE "OBS" AND "&type" NE "VARS" %then %do;
      %put WARNING: (obsnvars) Unknown type [&type];
      %goto EXIT;
   %end;
   
   /* Open data set passed as the macro parameter */
   %let dsid = %sysfunc(open(&ds));

   /* If the data set exists, then grab the number of observations and */
   /* variables then close the data set                                */
   %if &dsid %then %do;
      %let nobs =%sysfunc(attrn(&dsid,nobs));
      %let nvars=%sysfunc(attrn(&dsid,nvars));
      %let rc = %sysfunc(close(&dsid));
   %end;
   %else %do;   /* Otherwise, write a message that the data set could not be opened */
      %put WARNING: (obsnvars) Missing or could not open ds [&ds];
      %put %sysfunc(sysmsg());
      %goto EXIT;
   %end;
   
   %if "&type" EQ "OBS" %then %do;
&nobs.
   %end;
   %else %do;
&nvars.
   %end;
   
   %EXIT:
   
%mend obsnvars;

/*---------------------------------------------------------------------------
  * NAME: %getmvar
  *                             
  * DESC: Macro to check and then set a default value if missing in global namespace.
  *          
  * UPDATED: 15Mar2015      rnb                              
  *-------------------------------------------------------------------------*/
%macro getmvar(mvar=,default=);
   %if %symexist(&mvar.) %then %do;
      %if "&&&mvar" EQ "" %then %let &mvar=%str(&default);
   %end;
   %else %do;
      %global &mvar.;
      %let &mvar=%str(&default.);
   %end;
%mend getmvar;

/*---------------------------------------------------------------------------
  * NAME: %evsetglb
  *                             
  * DESC: Macro to support setting globals after invocation.
  *          
  * UPDATED: 15Jul2013      rnb                              
  *-------------------------------------------------------------------------*/
 
%macro evsetglb;
    %if "&begin" EQ "" %then %let begin = &evbegin;
    %if "&end" EQ ""   %then %let end = &evend;
    %if "&debug" EQ "" %then %let debug = &evdebug;
    %if "&where" EQ "" %then %let where = &evwhere;
    %if %symexist(topn) %then %do;
       %if &topn LT 1 %then %do;
          %if &evtopn GT 1 %then %let topn = &evtopn;
          %else %let topn=10;
       %end;
    %end;
    %if %symexist(showparm) %then %if "&showparm" EQ "" %then %let showparm = &evshowparm;
    %if %symexist(period)   %then %if "&period" EQ "" %then %let period = &evperiod;
    %if &debug GT 1 %then %do;
       %put DEBUG: (evsetglb) evbegin=[&evbegin] evend=[&evend] where=[&evwhere] period=[&evperiod] _loadtm=[&_loadtm];
    %end;
%mend evsetglb;

/*---------------------------------------------------------------------------
  * NAME: %evdebug;
  *                             
  * DESC: Macro to output additional information depending on the level (value)
  *       of evdebug.
  *          
  * UPDATED: 06Jan2014      rnb                              
  *-------------------------------------------------------------------------*/
%macro evdebug(lvl=);
   %local MNAME;
   %let MNAME=evdebug;
   %if "&lvl" EQ "" %then %do;
      %if %symexist(evdebug) %then %let lvl=&evdebug;
      %else %do;
         %put DEBUG: (&MNAME) evdebug global does not exist;
         %let lvl=2;      
      %end;
   %end;
   %if &lvl GT 0 %then %do;
         %put DEBUG: (&MNAME) SASPREFIX [&SASPREFIX.];
         %put DEBUG: (&MNAME) SASDEPLOYID [&SASDEPLOYID.];
         %put DEBUG: (&MNAME) EVCONF [&EVCONF];
         %put DEBUG: (&MNAME) EVDM [&EVDM];
         %put DEBUG: (&MNAME) EVDMPATH [&EVDMPATH];
   %end;
   %if &lvl GT 1 %then %do;
      %put INFO: (&mname) proc options sasautos filename libname _all_;
      proc options group=memory;
      run;
      %put _all_;
      %put %sysfunc(getoption(sasautos));
      filename _ALL_ list;
      libname _ALL_ LIST;      
   %end;
   %if &lvl GT 2 %then %do;
      proc options; 
      run;
   %end;
%mend evdebug;

 /*---------------------------------------------------------------------------
  * NAME: %evspinit
  *                             
  * DESC: Macro to support global reporting values with sp overrides.
  *          
  * UPDATED: 15Jun2013      rnb                              
  *-------------------------------------------------------------------------*/
 %macro evspinit;
    %if %symexist(dayrange) %then %if "&dayrange" EQ "" %then %let dayrange=5;
    %else %let dayrange=5;
    %if "&evbegin" EQ "" %then %do;
       %let evbegin=%ev_startdate("&sysdate"d,-&dayrange.); %* Analysis Start Date;
       %evcomment((evspinit) Begin datetime evbegin missing, setting to <&evbegin.>.);
    %end;
    %if "&evend"   EQ "" %then %let evend=&sysdate;                         %* Analysis End Date;
    %if %symexist(_evshowparm) %then %if "&_evshowparm" EQ "" %then %let _evshowparm=&evshowparm;
    %if %symexist(_evtopn)  %then %if "&_evtopn" EQ "0" %then %let _evtopn=&evtopn;
    %if %symexist(_evbegin) %then %if "&_evbegin" EQ "" %then %let _evbegin=&evbegin;
    %if %symexist(_evend)   %then %if "&_evend"   EQ "" %then %let _evend=&evend;
    %if &evdebug GT 1 %then %put DEBUG: (evspinit) <&_evbegin> <&evbegin> end <&_evend> <&evend>;
    %if %symexist(_evwhere) %then %if "&_evwhere" EQ "" %then %let _evwhere=&evwhere;
 %mend evspinit;
 
 /*---------------------------------------------------------------------------
  * NAME: %qwords
  *                             
  * DESC: Macro to double quote word list.
  *                                     
  *-------------------------------------------------------------------------*/
  %macro dqlist(str,quote=%str(%"),delim=%str( ));
     %local i list;
     %let i=1;
     %do %while(%length(%scan(&str,&i,%str( ))) GT 0);
       %if "%length(&list)" EQ "0" %then %let list=&quote.%scan(&str,&i,%str( ))&quote;
       %else %let list=&list.&quote.%scan(&str,&i,%str( ))&quote;
       %let i=%eval(&i + 1);
       %if %length(%scan(&str,&i,%str( ))) GT 0 %then %let list=&list.&delim;
     %end;
     %unquote(&list.)
  %mend dqlist;
  
 /*---------------------------------------------------------------------------
   * NAME: %add_dlm
   *                             
   * DESC: Macro to add delimitor to a word list.
   *                                     
  *-------------------------------------------------------------------------*/
  %macro add_dlm(list, token=%nrbquote(,));
	%******************************************************;
	%* Use nrbquote on call to handle special character;
	%******************************************************;
	%let newlist=;
	%let i=1;

	%do %until(%qscan(nrbquote(&list),&i,%str( ))=);
		%let word = %qscan(%nrbquote(&list),&i,%str( ));

		%if "&i" eq "1" %then
			%let newlist=&word;
		%else
		/*%if &word NQ*/
			%let newlist = %str(&newlist.&token &word);
		%let i = %eval(&i+1);
	%end;
	&newlist
  %mend add_dlm; 
 
  /*---------------------------------------------------------------------------
     * NAME: %evsettz;
     *                             
     * DESC: 
     * Adjusting for requested time zone offset
     * DATAMART = use local timezone of host/platform
     * DATA = use timezone associated with data (per record)
     * UTC or GMT = Use 0 seconds, Data Mart in UTC
     * or valid integer value = number of offset seconds
     *          
     * UPDATED: 15Nov2013      rnb                              
  *-------------------------------------------------------------------------*/
  %macro evsettz;
     %if "&tzoneoff" NE "" %then %do; 
       %if "&tzoneoff" = "DATAMART" %then %do;
          %let tzoneoff = &evdmtzoffset.;
          %if "%sysfunc(tzoneid())" EQ "" %then
             %let tzonelab = %quote(TZ (%sysfunc(putn(&evdmtzoffset.,hhmm.))));
          %else 
             %let tzonelab = %quote(TZ (%sysfunc(tzoneid())));
       %end;
       %else %if "&tzoneoff" = "GMT" or
                 "&tzoneoff" = "UTC" %then %do;
          %let tzoneoff = %str(0);
          %let tzonelab = %str(TZ (UTC));
       %end;
       %else %if "&tzoneoff" = "DATA" %then %do;
          %let tzonelab = %str(TZ (Local Time of Data));
       %end;
       %else %do;  /* can only be +/- 14 hours */
          %let test = %eval(&tzoneoff. + 0);
          %if %sysfunc(abs(&tzoneoff.)) GT 50400
          %then %do;
             %put ***********************************************;
             %put %str(ER)ROR: (&MNAME) Timezone <&tzoneoff.> GT 14 hours;
    	     %put ***********************************************;
             %goto EXIT;
          %end;
          %let tzonelab = %quote(TZ (%sysfunc(putn(&tzoneoff.,hhmm.))));
       %end;
    %end;
    %else %do;
       %let tzoneoff = %str(0);
       %let tzonelab = %str(TZ (UTC));
    %end;
    %EXIT:
  %mend evsettz;
 
 /*---------------------------------------------------------------------------
   * NAME: %evtitlesfoot;
   *                             
   * DESC: Macro to support standardized titles and footnotes including evfootnotes.
   *          
   * UPDATED: 15Sept2013      rnb                              
  *-------------------------------------------------------------------------*/
 %macro evtitlesfoot(showtz=NO);
    %local wstr;
    %let wstr=%str();
    %if "&showtz" EQ "YES" %then %do;
       %if %symexist(tzonelab) %then %if "&tzonelab." NE "" %then %let wstr=%str(&tzonelab.);
    %end;
    %if "%nrbquote(&where)" NE "" %then %do;
       %let wstrlen=%length(%nrbquote(&where));
       %if &wstrlen GT 30 %then %let wstr=%qsubstr(&where,1,30)%str(...);
       %else %let wstr=%qsubstr(&where,1,&wstrlen.);
    %end;
    %if %quote(&title1.) NE %quote() %then %do;
       %str(title1 &title1.;);
    %end;
    %if %quote(&title2.) NE %quote() %then %do;
       %str(title2 &title2.;);
    %end;
    %if %quote(&fnote1.) EQ %quote() AND
        %quote(&fnote2.) EQ %quote() %then %do;
           %if %quote(&fnote3.) EQ %quote() %then 
              %evfootnote(&wstr.,num=1,begin=&begin,end=&end);   
           %else %do;
              %quote(footnote3 &fnote3.;);
              %evcomment((&MNAME) fnode3 value replaces footnote for begin end dt ranges and report date);
           %end;
     %end;
     %else %if %quote(&fnote1.) EQ %quote() AND %quote(&fnote2.) NE %quote() %then %do;
        %evfootnote(&wstr.,num=1,begin=&begin,end=&end);
        %quote(footnote2 &fnote2.;);
     %end;
     %else %if %quote(&fnote1.) NE %quote() AND %quote(&fnote2.) EQ %quote() %then %do;
        %quote(footnote1 &fnote1.;);
        %evfootnote(&wstr.,num=2,begin=&begin,end=&end);
     %end;
     %else %do;
        %quote(footnote1 &fnote1.;);
        %quote(footnote2 &fnote2.;);
        %if %quote(&fnote3.) EQ %quote() %then 
           %evfootnote(&wstr.,num=3,begin=&evbegin,end=&evend);
        %else %do;
           %quote(footnote3 &fnote3.;);
           %evcomment((&MNAME) fnode3 value replaces footnote for begin end dt ranges and report date);
        %end;
     %end;
 %mend evtitlesfoot;
 
 %macro evodstext(showtz=NO);
    %local wstr;
    %let wstr=%str();
    %if "&showtz" EQ "YES" %then %do;
       %if %symexist(tzonelab) %then %if "&tzonelab." NE "" %then %let wstr=%str(&tzonelab.);
    %end;
    %if "%nrbquote(&where)" NE "" %then %do;
       %let wstrlen=%length(%nrbquote(&where));
       %if &wstrlen GT 30 %then %let wstr=%qsubstr(&where,1,30)%str(...);
       %else %let wstr=%qsubstr(&where,1,&wstrlen.);
   %end;

   %if %quote(&title1.) NE %quote() %then %do;
      %str(entrytitle "&title1.";);
   %end;
   %if %quote(&title2.) NE %quote() %then %do;
      %str(entrytitle "&title2.";);
   %end;
    %if %quote(&fnote1.) EQ %quote() AND
        %quote(&fnote2.) EQ %quote() %then %do;
           %if %quote(&fnote3.) EQ %quote() %then 
              %evodsfootnote(&wstr.,num=1,begin=&begin,end=&end);   
           %else %do;
              %quote(entryfootnote "&fnote3.";);
              %evcomment((&MNAME) fnode3 value replaces footnote for begin end dt ranges and report date);
           %end;
     %end;
     %else %if %quote(&fnote1.) EQ %quote() AND %quote(&fnote2.) NE %quote() %then %do;
        %evodsfootnote(&wstr.,num=1,begin=&begin,end=&end);
        %quote(entryfootnote "&fnote2.";);
     %end;
     %else %if %quote(&fnote1.) NE %quote() AND %quote(&fnote2.) EQ %quote() %then %do;
        %quote(entryfootnote "&fnote1.";);
        %evodsfootnote(&wstr.,num=2,begin=&begin,end=&end);
     %end;
     %else %do;
        %quote(entryfootnote "&fnote1.";);
        %quote(entryfootnote "&fnote2.";);
        %if %quote(&fnote3.) EQ %quote() %then 
           %evodsfootnote(&wstr.,num=3,begin=&evbegin,end=&evend);
        %else %do;
           %quote(entryfootnote "&fnote3.";);
           %evcomment((&MNAME) fnode3 value replaces footnote for begin end dt ranges and report date);
        %end;
     %end;
%mend evodstext;
 
/*---------------------------------------------------------------------------
 * NAME: %evfootnote
 *                             
 * DESC: This macro generates a footnote to appear on the report.
 *       The footnote specifies the date and time that the report was
 *       generated.     
 *
 * PARAMETERS:
 *
 * OUTPUT: 
 *       The macro generates a footnote in the following format and 
 *       places it in the lower right hand corner of the report:
 *
 *       Generated: ddmmmyyyy:hhmm AM/PM 
 *
 * USAGE:
 *
 *    %evinit_cmnFootNote;
 *          
 * UPDATED: 08Jun2013                              
 *-------------------------------------------------------------------------*/
 %macro evfootnote(msg,num=1,begin=,end=);
    footnote&num
    %if "&begin" NE "" %then %do;
    	  %if "&end" NE "" %then j=l %quote("Start: &begin. - End: &end.");
       %else j=l %quote("Start: &begin.");
    %end;
    %else %do;
    	  %if "&end" NE "" %then %do;
    	     %quote(j=l "Ending: &end.")
    	  %end;
    %end;
    %if "&msg" NE "" %then %do;
    	  %quote(j=c "&msg")
    %end;
    %quote(j=r "&cpnobs obs on %sysfunc(date(),date7.)");
%mend evfootnote;
   
%macro evodsfootnote(msg,num=1,begin=,end=);
    entryfootnote 
    %if "&begin" NE "" %then %do;
    	  %if "&end" NE "" %then halign=left %quote("Start: &begin. - End: &end.");
       %else halign=left %quote("Start: &begin.");
    %end;
    %else %do;
    	  %if "&end" NE "" %then %do;
    	     %quote(halign=left "Ending: &end.")
    	  %end;
    %end;
    %if "&msg" NE "" %then %do;
    	  %quote(halign=center "&msg")
    %end;
    %quote(halign=right "&cpnobs obs on %sysfunc(date(),date7.)");
%mend evodsfootnote;   
   
/*+----------------------------------------------------------------+
  | Name:     evnodatafound.sas
  | Product:  SAS Environment Manager
  |
  | Language: SAS
  | Purpose:  
  | History:  Date    action
  |           15Jul13 Created for EV                            rnb
  | Future:
  | End
  +----------------------------------------------------------------+*/
%macro evnodatafound(report,title=);
   data nodata;
      format msg $80.;
      label msg="Results";
      msg="Query produced zero records for &report";
   run;
   /*
   Proc report data=nodata nowd headline headskip;
      column msg;
   run;
   rnb replace with proc sql */
%mend;

/*+----------------------------------------------------------------+
  | Name:     everrmsg.sas &evwarnmsg.sas
  | Product:  SAS Environment Manager
  |
  | Language: SAS
  | Purpose:  
  | History:  Date     action
  |           15Sept13 Created for EV                            rnb
  | Future:
  | End
  +----------------------------------------------------------------+*/
%macro everrmsg(caller,log=YES,msg=);

   %if "&LOG" EQ "YES" %then %do;
      %put ********************************************************;
      %put %str(ER)ROR: (&caller) &msg;
      %put ********************************************************;
   %end;
   
   data nodata;
      format msg $100.;
      label msg="%str(Er)ror: &caller";
      msg="&msg";
   run;
   /*
   Proc report data=nodata nowd headline headskip;
      column msg;
   run;
   rnb replace with proc sql */
%mend everrmsg;

%macro evwarnmsg(caller,log=YES,msg=);

   %if "&LOG" EQ "YES" %then %do;
      %put ********************************************************;
      %put %str(WA)RNING: (&caller) &msg;
      %put ********************************************************;
   %end;
   
   data nodata;
      format msg $80.;
      label msg="%str(Wa)rning: &caller";
      msg="&msg";
   run;
   /*
   Proc report data=nodata nowd headline headskip;
      column msg;
   run;
   rnb replace with proc sql */
%mend evwarnmsg;

/*+----------------------------------------------------------------+
  | Name:     evreportstatus.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  
  | History:  Date    action
  |           15Jul13 Created for EV                            rnb
  | Future:
  | End
  +----------------------------------------------------------------+*/

%macro evreportstatus(dataset,message,_rc=status);
   %local MNAME;
   %let MNAME=evreportstatus;
   %let &_rc=0;

   %*********************************************************;
   %* Check results dataset exists and has obs;
   %*********************************************************;
   %if %sysfunc(exist(&dataset.)) or %sysfunc(exist(&dataset.,view)) 
   %then %do;
      %let _dsid = %sysfunc(open(&dataset.));
      %if "%sysfunc(attrn(&_dsid,NOBS))" = "0" %then %do;
           %put ****************************************************;
           %put (&MNAME) WARNING: Dataset &dataset. 0 obs. Halting;
           %put ****************************************************;
           %let rc = %sysfunc(close(&_dsid));
  	   %evnodatafound(&message);
  	   %let &_rc=1;
           %goto EXIT;
      %end;
      %let rc = %sysfunc(close(&_dsid));  
   %end;
   %else %do;
       %put ********************************************************;
       %put (&MNAME) %str(ER)ROR: Dataset &dataset. missing. Halting;
       %put ********************************************************;
       %evnodatafound(&dataset.. Missing dataset);
       %let &_rc=1;
       %goto EXIT;
   %end;
    
   %EXIT:
           
%mend evreportstatus;

/*+----------------------------------------------------------------+
  | Name:     evmkfmt.sas
  | Usage:
  |    list - list of variable format pairs, parenthesized, 
  |           space delimited
  |
  | Purpose:  Utility macro for reporting
  +----------------------------------------------------------------+*/
%macro evmkfmt(list);
   %local i copy;
   %if "&list" EQ "" %then %goto EXIT;
   /* %let copy=%quote &list ; */
   %let copy=%quote(&list);
   %let eom=%eval(%length(%trim(&copy)) gt 0);
   %do %while(&eom);
       %let varname=%qscan(&copy,1,%str()( =));
       %let cutpoint=%eval(%index(&copy,&varname)+%length(&varname));
       %if &cutpoint=0 or %eval(&cutpoint gt %length(&copy)) %then %goto EXIT;
       %let copy=%qsubstr(&copy,&cutpoint);
       %let format=%qscan(&copy,1,%str()( =));
       %str(format &varname &format;)
       %let cutpoint=%eval(%index(&copy,&format)+%length(&format));
       %if &cutpoint=0 or %eval(&cutpoint gt %length(&copy)) %then %goto EXIT;
       %let copy=%qsubstr(&copy,&cutpoint);
       %let eom=%eval(%length(&copy) gt 0;
   %end;
   
   %EXIT: 
%mend;

/*+----------------------------------------------------------------+
  | Name:     evmklab.sas
  | Usage:
  |    list - list of variable = label pairs, label quoted, parenthesized
  |
  | Purpose:  Utility macro for reporting                     rnb
  +----------------------------------------------------------------+*/
%macro evmklab(lcopy,cmd=label);
   %local lcopy;
   %if %quote(&lcopy) EQ %quote() %then %goto EXIT;
   %quote(&cmd &lcopy);
   %EXIT: 
%mend;

/*+----------------------------------------------------------------+
  | Name:     evgsymbols.sas
  | Usage:
  |    Generates symbol definitions based on ev macro globals and input 
  |    parameters, evcolors, evsymbols, areas for stacking, and requested
  |    report (plot) type
  |
  | History:  Date    action
  |           20Dec13 Created for EV                            rnb
  +----------------------------------------------------------------+*/
  %macro evgsymbols;
     %let mname=evgsymbols;
         %***************************************************************;
         %* Code to use SASEV symbol and color registered properites;
         %* properties evcolors and evsymbols in properties file.;
         %***************************************************************;
         %if "&interpol" NE "" %then %do;
            %let numcolors = %num_tokens(&evcolors.);
            %let numsyms = %num_tokens(&evsymbols.);
            %if "&interpol" EQ "SCATTER" %then %do;
                %let symint=%str(i=NONE h=&evsymbolh.);
                %let j = 0;
                %do %until (&j GT &numcolors); 
                   %let j = %eval(&j+1); 
                   %if "&symbols." EQ "NO" %then %do;
                      %let symstmt = %str(symbol&j. &symint. value=DOT;);
                      %quote(&symstmt)
                      %if &evdebug >1 %then %put DEBUG: (&MNAME) symbol statement <&symstmt>);
                   %end;
                   %else %do;  /* requested symbols and colors */
                       %do k = 1 %to &numsyms;
                          %let val = %scan(&evsymbols.,&k);
                          %let symtot=%eval(&j.+&k-1);
                          %let color = %scan(&evcolors.,&symtot);
         	          %let symstmt = %quote(symbol&symtot. &symint. value=&val color=&color);
      		          %quote(&symstmt;)
      		          %if &evdebug >1 %then %put DEBUG: (&MNAME) symbol statement <&symstmt> <&symtot>);
                       %end;  /* of k loop */
                       %let j = &symtot.;
                    %end;
                 %end;    /* do until */
              %end;                
              %else %do;  /* not scatter */
                 %let symint=%str(i=&interpol h=&evsymbolh.);
                 %do j = 1 %to &numsyms.;
                    %if "&symbols." EQ "NO" 
      		       %then %let symstmt = %str(symbol&j. &symint. value=NONE;);  /* rnbbug repeats colors per symbol */
                       %else %let symstmt = %quote(symbol&j. &symint. value=%scan(&evsymbols.,&j.) color=%scan(&evcolors.,&j.));
                    %quote(&symstmt.);
                    %if &evdebug >1 %then %put DEBUG: (&MNAME) symbol statement <&symstmt>;
                  %end; /* of j do loop */
               %end;    /* of scatter test */
	  %end;      /* of interpol test */
   %EXIT:
%mend evgsymbols;

/*---------------------------------------------------------------------------
 * NAME: %getevar
 *                             
 * DESC: Returns the value of the requested environment variable. Environnment
 * variables can be case sensitive.
 *
 * USAGE:
 * 
 *    %getevar(var=USER);
 *          
 * UPDATED: 15Nov2014                              
 *-------------------------------------------------------------------------*/
%macro getevar(var=);
	%local MNAME;
	%let MNAME=getevar;
	%if "&var" NE "" %then %do;
		%if %sysfunc(envlen(&var)) GT -1 %then %do;
			%global &var;
			%let &var = %sysget(&var);
			%if &evdebug GT 1 %then %put (&MNAME) var = [&var.];
			&&&var.
		%end;
		%else %do;
			%put WARNING: (&MNAME) Environment variable [&var] missing;
		%end;
	%end;
	%else %do;
		%put ERROR: (&MNAME) Invalid invocation, missing value for prompt var input.;
	%end;
%mend getevar;

/*---------------------------------------------------------------------------
 * NAME: %evinit_cleanup
 *                             
 * DESC: Removes all global macros that were created during the use of
 *       the other evinit_* macros. This macro is optional.
 *
 * USAGE:
 * 
 *    %evinit_cleanup;
 *          
 * UPDATED: 03SEP2013                              
 *-------------------------------------------------------------------------*/
 %macro evinit_cleanup();
   %symdel  
     ev_fromdateset
     ev_tz / nowarn;

  %mend evinit_cleanup;

%mend evutils;
