/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

 /*+----------------------------------------------------------------+
  | Name:     evxform.sas
  | 
  | Purpose:  general purpose data conversion program
  | History:  Date    action
  |           21Feb92 Initial Coding                              jtr
  |           31Mar92 Added Stacked, percent                      rnb
  |           93..01  v2 IT Intelligent life                      various
  |           17May13 Recoded for EV                              rnb
  |           20Oct13 Restructured for ev                         rnb
  |
  | Parameters:
  |    dataset       - single dataset or view
  |    axis          - 0|1|2   One or two vertical axes
  |    period        - transform datetime variable by clumping
  |                    or folding. The following period keywords
  |                    are supported.
  |       type         Keyword   Example         Def.Format
  |       ------       -------   -------------   ----------
  |       Cluster      5MIN      03Jan14:01:05   DATETIME14.
  |                    15MIN     03Jan14:01:15   DATETIME14.
  |                    HOUR      03Jan14:01      DATETIME10.
  |                    DATE      03Jan14         DATETIME7.
  |                    WEEK      01Jan14         DATETIME7.
  |                    MONTH     Jan14           MONYY.
  |                    QTR       Jan14           MONYY.
  |       Folded       24HOUR    hour-of-day (0-23)
  |                    WEEKDAY   day-of-week (1-7) or (Sun, Mon, etc).
  |                    MONTHDAY  day-of-month (1-31)
  |                    YEARDAY   day-of-year (1-366)
  |                    YEARMONTH month-of-year (1-12)
  |    haxis         - macro_variable_name (does not appear anymore?  sxm)
  |                    Set specified macro variable to haxis= spec
  |    cumulate      - Cumulate variables (x2=x1+x2,x3=x1+x2+x3,etc.) or
  |                    one variable by class.  NO|VAR|CLASS|STACK
  |    id            - vars to carry through in an id statement
  |    percent       - NO|YES
  |                    Recode variables as percentages of total of
  |                    all analysis variables
  |    collapse      - NONE|TOD|PERIOD After period and tod recoding, collapse summary statistic by 
  |                    by time (TOD) or datetime (PERIOD)
  |    percents      - NO|specs
  |                    Transform variable into specified percentss
  |                    (cannot be used with more than 1 variable).
  |    invert        - (YES/NO) Invert the data in preparation for
  |                    x*y=z graphs
  |    xvar          - calling plot contains xvar override, instead of datetime.
  |    tzoneoff      - time zone offset (DATAMART, DATA, UTC or a +/- Integer 
  |                    equal to number of tz offset seconds (+/-)
  |    stat          - (valid statistic for proc summary) used in
  |                    collapsing data, or DEFAULT
  |    tvar          - Transform variable
  |    class         - Class variable(s)
  |    by            - Perform all actions by the variables specified
  |    cl_nam        - Specifies classification variables to be kept in
  |                    addition to by and plotting variables
  |    outds         - name of dataset or view to store results
  |    keep          - list of additional variables to keep other than those in
  |                    by, class, id, xvar, etc. lists
  |    keepall       - YES|NO - defaults to NO.  YES means keep all variables
  |    _rc           - return code variable
  |
  | Notes:
  |           Do NOT set names labels note and areas local
  |           These are used by calling programs
  | End
  +----------------------------------------------------------------+*/

%macro evxform(dataset,axis=1,period=ASIS,cumulate=,id=,tzoneoff=,percent=no,
   collapse=PERIOD,percents=,invert=no,xvar=,tvar=,class=,fmt=YES,xvartype=,
   stat=,by=,cl_nam=,outds=tmpdata,keep=,keepall=no,_rc=rc);

   %global cphaxis cprptmd cpfmt1;
   %local i by_stmt cl_stmt mname word;
   
   %let mname=evXform;
   %let period   = %upcase(&period);
   %let cumulate = %upcase(&cumulate);
   %let percent  = %upcase(&percent);
   %let invert   = %upcase(&invert);
   %let keepall  = %upcase(&keepall);
   %let stat     = %upcase(&stat);
   %let collapse = %upcase(&collapse);
   %let tzoneoff = %upcase(&tzoneoff);
   %let xvar = %upcase(&xvar);
   %let by_wodt=;
   %let cl_wotime=;
   %let names=;
   %*let labels=;
   %let areas=0;

   %let keep = %removew(%upcase(&keep),TIME DATETIME);
   %if "&xvar" EQ "DATETIME" or
       "&xvar" EQ "TIME" %then %do;
       %let xvar=;
   %end;
   %if "&xvartype" EQ "" %then %let xvartype=DISCRETE;
   %else %if "&xvartype" NE "MEASURE" and
            "&xvartype" NE "DISCRETE" %then %do;
         %everrmsg(&MNAME,msg=Invalid XVARTYPE [&xvartype.] xvartype [MEASURE|DISCRETE]);
         %goto EXIT;       
   %end;
   
   %*********************************************************;
   %* For collapsing on periods, check period type.          ;
   %*********************************************************;
   
   %if %index(%dqlist(ASIS 15MIN 5MIN HOUR DATE WHOLEDAY WEEK MONTH QTR 24HOUR WEEKDAY 
      MONTHDAY YEARDAY YEARMONTH),"&period.") EQ 0
      %then %do;
         %everrmsg(&MNAME,msg=Type [&period.] not supported.);
         %goto EXIT;
   %end;

   %if "&cl_nam" EQ "" %then %do;
       %let cl_stmt=;
       %if "&by" EQ "" %then %do;
          %let by_stmt=;
          %let by_wodt=;
       %end;
       %else %do;
          %let by_stmt=%str(by &by;);
          %let by_wodt=%str(&by);
       %end;
       %end;
   %else %do;
       %let cl_stmt=%str(class &cl_nam;);
       %let by_stmt=%str(by &by &cl_nam;);
       %let i=0;
       %do %until(%scan(&cl_nam,&i)=);
         %let i = %eval(&i+1);
         %let word = %scan(&cl_nam,&i,' ');
         %if %upcase(&word) NE DATETIME
             %then %let by_wodt = &by_wodt &word;
         %if %upcase(&word) NE TIME
             %then %let cl_wotime = &cl_wotime &word;
       %end;
       %let by_wodt=%str(&by &by_wodt);
   %end;
   
   
   %let id_stmt=;
   %let id_vars=;
   %if "&id" NE "" %then %do;
   	%let id_stmt=%str(id &id);
   	%let idvars=%str(&id);
   %end;
   %if "&axis" EQ "2" %then %do;
       %let var1=&y1_var;
       %let var2=&y2_var;
       %let names=&var1 &var2;
   %end;

   %*********************************************************;
   %* Check parameters;
   %*********************************************************;
   %let &_rc=1;
   %if "&axis" NE "1" and "&axis" NE "2" and "&axis" NE "0" %then %do;
      %put ***********************************;
      %put ERROR: (&MNAME) axis must be 0 or 1 or 2;
      %put ***********************************;
      %goto EXIT;
      %end;
   %if "&cumulate" EQ "CLASS" and "&cl_nam" EQ "" %then %do;
      %put ********************************************************;
      %put ERROR: (&MNAME) cumulate=CLASS requires a class variable;
      %put ********************************************************;
      %goto EXIT;
      %end;
   %if &axis=2 %then %do;
      %if "&cumulate" NE "" or "&percent" EQ "YES" or "&percents" NE "" or
         "&invert" EQ "YES" %then %do;
         %put **************************************************************;
         %put ERROR: (&MNAME) cannot have axis=2 and any of the following:;
         %put (&MNAME)        cumulate,percent,percents,invert.;
         %put **************************************************************;
         %goto EXIT;
         %end;
      %end;
   %if "&cumulate" EQ "CLASS" and ("&percent" EQ "YES" or "&percents" NE "") %then %do;
      %put ********************************************************;
      %put ERROR: (&MNAME) cannot cumulate by class and calculate  ;
      %put                 percentages or percentss;
      %put ********************************************************;
      %goto EXIT;
      %end;
   %if "&percents" NE "" and "&percent" EQ "YES" %then %do;
      %put ********************************************************;
      %put ERROR: (&MNAME) cannot specify both percents & percent;
      %put ********************************************************;
      %goto EXIT;
      %end;

   %let &_rc=0;

   %local slist fmtlist statfmt;
   %if "&statfmt" EQ "" %then %let statfmt=%str(Count);
   %if "&collapse" EQ "PERIOD" or
       "&collapse" EQ "TOD" %then %do;
      %*********************************************************;
      %* For collapsing on periods, check stat type.            ;
      %* it must be one of the first 14 to allow collapsing     ;
      %* > 14 just setting the statfmt (freq, cfreq, percent cpct;
      %*********************************************************;
      %if "&stat" EQ "" %then %do;
         %let stat = %str(MEAN);
         %let statfmt=%str(Means);
      %end;
      %else %do;
      	%let slist = %str(MEAN MAX MIN SUM RANGE STD CV N NMISS SUMWGT CSS
		USS VAR FREQ DEFAULT CFREQ PERCENT CPERCENT);
      	%let fmtlist = %str(Means Maximum Minimums Sums Ranges STD CV N
		NMISS SUMWGT CSS USS VAR Frequency Default CumFreq Percent CumPCT);
      	%let i=0;
      	%do %until(%index(&stat,%scan(&slist,&i))^=0);
      		%let i = %eval(&i+1);
         		%if &i>14 %then %do;
			%put ***********************************************;
			%put ERROR: (&MNAME) Statistics &stat not supported;
			%put ***********************************************;
			%goto EXIT;
			%end;
      	%end;
      	%let statfmt=%scan(&fmtlist,&i);
      %end;
   %end;
  
   %*********************************************************************;
   %* Create formats. Assemble lists of names and labels.                ;
   %* Define areas. Areas used for stack, percents and stackpct gplots   ;
   %*********************************************************************;
   %if "&axis" EQ "1" %then %do;
      %let names=&evrptvars;
      %let areas=0;
      %let cpfmt1 = %str(value typef ); 
      %if &debug GT 1 %then %put numvars [&numvars] evrptvars [&evrptvars];

      %if %quote(&labels) NE %then %do;
         %if &debug GT 1 %then %put DEBUG: labels [&labels];
         %let dsid=%sysfunc(open(&dataset,i));
         %do i = 1 %to &numvars;
            %let areas = &i;
	       %if &dsid %then %do;
	          %let labelval=%sysfunc(varlabel(&dsid,%sysfunc(varnum(&dsid,&&var&i))));
	          %put labelval [&labelval];
            %end;
            %let cpfmt1 = %str(&cpfmt1 &i = %'&labelval%');      
            %if &debug GT 1 %then %put DEBUG: cpfmt1 [&cpfmt1];
         %end;     /* of do loop */
      
         %let rc = %sysfunc(close(&dsid));     
      %end;        /* of label override section */
      %else %do;   /* no override labels, use var names */
         %do i = 1 %to &numvars;
            %let areas=&i;
            %let cpfmt1 = %str( &cpfmt1 &i = "&&var&i");
         %end;     /* do loop */
      %end;        /* end of code for lab1, lab2, lab3 */
   
      %let cpfmt1 = %unquote(&cpfmt1.);
      proc format;
         %quote(&cpfmt1.); 
      run;
      %if &debug GT 1 %then %put DEBUG: areas [&areas.] names [&names.];
   %end;  /* end of if &axis=1 %then %do  */
   
   %if "&period" EQ "ASIS" and 
       "&cumulate" EQ "NO" and 
       "&percent" EQ "NO" and
       "&percents" EQ "" and 
       "&invert" EQ "NO" and 
       "&axis" EQ "1" %then %goto EXIT;

   /* Need to allow Time class to pass thru */
   %**********************************************;
   %* For cumulating by class, ensure the sort   *;
   %* order is correct.                          *;
   %**********************************************;
   %if "&cumulate" EQ "CLASS" %then %do;
       proc sort data=&dataset out=tmpdata;
          by &by datetime &cl_nam;
          &whglobal &whlocal;
          run;
       %let dataset=tmpdata;
       %end;

   %**********************************************;
   %* For class and by group, ensure sort order  *;
   %* order is correct.                          *;
   %**********************************************;
   %if "&cumulate" EQ "NO" and ("&by" NE "" and "&cl_nam" NE "") %then %do;
       proc sort data=&dataset out=tmpdata;
          by &by &cl_nam datetime;
          &whglobal &whlocal;
          run;
       %let dataset=tmpdata;
       %end;

   %********************************************************;
   %* Adjusting for requested time zone offset             *;
   %* LOCAL = use local timezone of host/platform          *;
   %* DATA = use timezone associated with data (per record)*;
   %* UTC or GMT = Use 0 seconds, Data Mart in UTC          *;
   %* or valid integer value = number of offset seconds    *;
   %********************************************************;
   %evsettz;
  
   %**********************************************;
   %* If period collapsing is to be done, create a;
   %* temporary dataset.  Otherwise name the      ;
   %* output dataset according to the outds=      ;
   %* parameter.                                  ;
   %**********************************************;

   %****************************************************************;
   %*  if keepall=YES it generated data set tmpdata,     ;
   %*  but if period is ASIS or collapse is NONE, then no further     ;
   %*  processing is done.  In this case need to create &outds      ;
   %****************************************************************;
   %if "&todbins" NE "" %then %let secspersam=%eval(86400/&todbins);
   %else %let secspersam=%eval(86400/24);
   
   %if "&keepall" EQ "YES" %then %do;
      %if "&period" EQ "ASIS" or "&collapse" EQ "NONE" %then %do;
          data &outds;
      %end;
      %else %do;
          data tmpdata;
      %end;
   %end;
   %else %do;
      %if "&period" EQ "ASIS" or "&collapse" EQ "NONE" %then %do;
          data &outds (keep= shift datetime time _cptype &by &xvar &cl_nam &names &keep );
      %end;
      %else %do;
          data tmpdata (keep= shift datetime time _cptype &by &xvar &cl_nam &names &keep );
      %end;
   %end;
   
   if _n_ =1 then call symput('cpnobs',0);
   set &dataset;

   &whglobal &whlocal;
   call symput('cpnobs',1);

   %if "&names" NE "" %then %str(array _cpname &names;);
   %if "&axis" NE "0" %then %str(format _cptype typef.;);
   retain _cptype 1;

   %**********************************************;
   %* Transform the datetime variable according to;
   %* the period collapsing type.                 ;
   %**********************************************;
      /* make a real time value too */   
      format time hhmm5.;
      time = timepart(int((datetime + &tzoneoff.)/&secspersam)*&secspersam);
      
      %if "&period" EQ "HOUR" %then %do;
          datetime=int((datetime + &tzoneoff.)/3600)*3600;
          format datetime datetime12.;
          %let cpltfmt =datetime7.;
          label datetime = "Hourly &statfmt";
          %end;
      %else %if "&period" EQ "24HOUR" %then %do;
          datetime = hour(timepart(datetime + &tzoneoff.));
          /* change datetime into time value */
          datetime = hms(datetime,0,0);
          format datetime hhmm5.;
          %let cpltfmt=hhmm5.;
          label datetime = "Hourly &statfmt";
          %end;
      %else %if "&period" EQ "15MIN" %then %do;
          datetime=int((datetime + &tzoneoff.)/900)*900;
          format datetime datetime14.;
          %let cpltfmt=datetime7.;
          label datetime = "15Min &statfmt";
          %end;
      %else %if "&period" EQ "5MIN" %then %do;
          datetime=int((datetime + &tzoneoff.)/300)*300;
          format datetime datetime14.;
          %let cpltfmt=datetime7.;
          label datetime = "5Min &statfmt";
          %end;
      %else %if "&period" EQ "DATE" or "&period" EQ "WHOLEDAY" %then %do;
          datetime=int((datetime + &tzoneoff.)/86400)*86400;
          format datetime datetime7.;
          %let cpltfmt=datetime7.;
          label datetime = "Daily &statfmt";           
          %end;
      %else %if "&period" EQ "WEEK" %then %do;
          datetime=intnx('DTWEEK',(datetime + &tzoneoff.),0);
          format datetime datetime7.;
          %let cpltfmt=datetime7.;
          label datetime = "Weekly &statfmt";
          %end;
      %else %if "&period" EQ "MONTH" %then %do;      
          datetime=datepart(intnx('DTMONTH',(datetime + &tzoneoff.),0));
          format datetime monyy7.;
          %let cpltfmt=monyy7.;
          label datetime = "Monthly &statfmt";
          %end;
      %else %if "&period" EQ "QTR" %then %do;
          datetime=datepart(intnx('DTQTR',(datetime + &tzoneoff.),0));
          format datetime monyy7.;
          %let cpltfmt=monyy7.;
          label datetime = "Quarterly &statfmt";
          %end;
      %else %if "&period" EQ "WEEKDAY" %then %do;
          datetime=weekday(datepart(datetime + &tzoneoff.));
          format datetime downame.;
          %let cpltfmt=downame.;
          label datetime = "Daily &statfmt";
          %end;
      %else %if "&period" EQ "MONTHDAY" %then %do;
          datetime=day(datepart(datetime + &tzoneoff.));
          format datetime 2.;
          %let cpltfmt=2.;
          label datetime = "Daily &statfmt";
          %end;
      %else %if "&period" EQ "YEARDAY" %then %do; 
          datetime=put(datepart(datetime + &tzoneoff.),julday3.);
          format datetime 3.;
          %let cpltfmt=3.;
          label datetime = "Daily &statfmt";
          %end;
      %else %if "&period" EQ "YEARMONTH" %then %do;
          datetime=month(datepart(datetime + &tzoneoff.));
          format datetime mon.;
          %let cpltfmt=mon.;
          label datetime = "Monthly &statfmt";
          %end;

   %if "&todbins" EQ "" %then %str(label time = "Time of Day [24]";);
   %else %str(label time = "Time of Day [&todbins]";);
   
   %***********************************************;
   %* Axis = 0  Only convert data;
   %***********************************************;

   %***********************************************;
   %* For the normal 1-axis case, transform the    ;
   %* variables according to the options specified ;
   %* (percent=,cumulate=,invert=).                ;
   %***********************************************;

      %if "&axis" EQ "1" %then %do;

         %if "&cumulate" EQ "CLASS" %then %do;
             retain lastvar1;
             lastvar1=&var1;
             %end;

         %else %if "&cumulate" EQ "NO" %then;
         %else %do;
            %if "&percent" EQ "YES" %then
                %str( total = sum( of &names ); );

            /* special case where all are missing;*/
            allmiss=1;
            do i = 1 to dim(_cpname);
               if _cpname[i] ne . then allmiss=0;
            end;

            do _cptype = 1 to dim( _cpname );

            %if "&percent" EQ "YES" %then
                %str( _cpname[ _cptype ] = _cpname[ _cptype ] * 100 / total; );

            %if "&cumulate" EQ "VAR" %then %do;
                if _cpname[_cptype] = . and not allmiss
                   then _cpname[_cptype] = 0;
                if _cptype > 1
                   then _cpname[_cptype]=_cpname[_cptype] + _cpname[_cptype-1];
            %end;

            %if "&cumulate" EQ "STACK" %then
                %str( if _cptype > 0 then _cpname[ _cptype ] =
                      _cpname[ _cptype ] * dim(_cpname); );

            %if "&invert" EQ "YES" %then %do;
               %let names=&var1;
               if _cptype > 1 then &var1 = _cpname[ _cptype ];
               output;
               %end;
            end;
            %end; /* not class or no */
      %end; /* one axis true */

   %***********************************************;
   %* For the 2-axis case, add the dummy _cptype=2 ;
   %* observation.  This is used later in getting  ;
   %* SAS/Graph to automatically legend the graph  ;
   %* for us.                                      ;
   %***********************************************;

      %if "&axis" EQ "2" %then %do;
         %let names=&y1_var &y2_var;
         output;  /* S0038773  */
         if _n_ = 1 then do;
            _cptype = 2;
            &y1_var = .;
            &y2_var = .;
            datetime=.;
            output;
            _cptype = 1;
         end;
      %end;
      
    run;

    %if "&cumulate" EQ "CLASS" %then %do;
       proc sort data=tmpdata; /* D0073922 */
          by &by &cl_nam  datetime ;
          run;
    %end;

    /***begin D0120990 ***/
    %if ("&period" EQ "ASIS" or "&collapse" EQ "NONE") and "&by" NE "" %then %do;
       %put syserr= &syserr syscc = &syscc sysrc = &sysrc;
       proc sort data=&outds;
            by &by datetime;
       run;
       %put syserr= &syserr syscc = &syscc sysrc = &sysrc;
       %if &syserr gt 4 %then %do;
          %let &_rc=1;
          %put *********************************************************;
          %put ERROR: (&MNAME) Proc Sort Error see Log. Halting.;
          %put *********************************************************;
          %goto EXIT;
       %end;
    %end;

   %let _dsid = %sysfunc(open(&outds.));
   %let cpnobs = %sysfunc(attrn(&_dsid,NOBS));
   %let rc = %sysfunc(close(&_dsid));
   
   %if &debug GT 1 %then %put DEBUG: (&mname) NOBS = &cpnobs; 
   %**********************************************;
   %* No observations, return status code 2       ;
   %**********************************************;
   %if "&cpnobs" EQ "0" %then %do;
       %let &_rc=2;
       %put ***************************************************************;
       %put (&MNAME) Subsetting criteria produced 0 observations. Halting.;
       %put ***************************************************************;
       %goto EXIT;
   %end;

   %**********************************************;
   %* Get list of vars for each statistic         ;
   %**********************************************;
   %if "&stat" NE "DEFAULT"
   %then %do;
      %if "&collapse" EQ "NONE" %then %let stat=MEAN; 
      %if "&keepall" EQ "YES"
         %then %let statlist = %str(&stat = );
         %else %do;
            %let statlist = %str(&stat = &names &xvar &keep);       
            %if "&xvartype" EQ "DISCRETE" %then %let statlist = %str(&stat = &names &keep);
         %end;
   %end;
   %else %if "&stat" EQ "DEFAULT" %then %do;
         %* need to figure out appropriate stat for each var;
         %* first get list of variables into macro var varlist ;
         %if "&keepall" EQ "YES"
         %then %do;
             %* get list of all numeric vars in table that are  ;
             %* not in by, id, class, etc.              ;
             %let libname = %upcase(%scan(&dataset,1,'.'));
             %let memname = %upcase(%scan(&dataset,2,'.'));
             proc sql noprint;
                  select name into :varlist separated by " "
                         from dictionary.columns
                         where libname = "&libname" and
                               memname = "&memname" and
                               type    = "num"      and
                 index("DATETIME &cl_nam &by &id",name) = 0 ;
             quit;
          %end;
          %else %do;
              %let varlist = %upcase(&names &xvar &keep) ;
          %end;
         %* now go through the list and determine stat ;
         %local avglist sumlist maxlist minlist;
         %let i = 1;
         %let nextvar = %quote(%scan(&varlist,&i)       );
         %do %while(&nextvar ne %str() );
             %if %length(&nextvar) ge 8
                 %then %let suffix  = %substr(&nextvar,8,1);
                 %else %let suffix  = %str();
             %if &suffix = %str()
                 %then %let avglist = %str(&avglist  &nextvar);
             %else %if &suffix = C or &suffix = S
                 %then %let sumlist = %str(&sumlist &nextvar);
             %else %if &suffix = X
                 %then %let maxlist = %str(&maxlist &nextvar);
             %else %if &suffix = N
                 %then %let minlist = %str(&minlist &nextvar);

             %let i = %eval(&i+1);
             %let nextvar = %scan(&varlist,&i       );
         %end;

         %if &avglist ne %str()
             %then %let avglist = %str(MEAN(&avglist)= );
         %if &sumlist ne %str()
             %then %let sumlist = %str(SUM(&sumlist)= );
         %if &minlist ne %str()
             %then %let minlist = %str(MIN(&minlist)= );
         %if &maxlist ne %str()
             %then %let maxlist = %str(MAX(&maxlist)= );
         %let statlist = %str(&avglist &sumlist &minlist &maxlist );
   %end ;  %* end of %if &stat = DEFAULT logic ;

   %**********************************************;
   %* Perform the period collapsing.              ;
   %**********************************************;
   %if "&collapse" EQ "PERIOD" or
       "&collapse" EQ "TOD" 
      %then %do;
         %let dt_tod_order=;
         %if "&collapse" EQ "TOD" %then %let dt_tod_order=%str(datetime time);
         %else %do;
            %let dt_tod_order=%str(datetime);
            %if "&id_stmt" EQ "" %then %do;
               %let id_stmt=%str(id time);
               %let idvars=%str(time);
            %end;
            %else %do;
               %let id_stmt=%str(&id_stmt time);
               %let idvars=%str(&idvars time);
            %end;
         %end;
         proc datasets lib=work nolist;
            modify tmpdata;
            format _cptype;
         run;
         quit;
         %if "&keepall" EQ "YES" %then %do;
            %* syserr is set to 1008 by PROC SUMMARY (or PROC MEANS);
            proc means data=tmpdata nway missing noprint;
               &id_stmt.;
               class &by_wodt &dt_tod_order _cptype;
               output out=&outds(drop=_type_ _freq_)
               &statlist ;
               run;
            %if &syserr>4 %then %do;
               %let &_rc=1;
               %put *********************************************************;
               %put ERROR: (&MNAME) Proc Means Error see Log. Halting.;
               %put *********************************************************;
               %goto EXIT;
            %end;
         %end;
         %else %do;
             %if &debug GT 1 %then %put xvartype [&xvartype] xvar [&xvar] names [&names] keep [&keep]; 
%put id_stmt [&id_stmt];
%put idvars [&idvars];
%put by_wodt [&by_wodt];
%put add_dlms [%add_dlm(&idvars &by_wodt &dt_tod_order)];
%put dt_tod_order [&dt_tod_order];
%put xvar [&xvar];
%put names [&names];
%put statlist [&statlist];

   /*          
             proc summary data=tmpdata nway missing;
               &id_stmt.;
               class &by_wodt &dt_tod_order _cptype 
               %if "&XVARTYPE" EQ "DISCRETE" %then %str(&xvar);
               ;
               var &names 
               %if "&XVARTYPE" EQ "MEASURE" %then %str(&xvar);
               &keep;
               output out=&outds(drop=_type_ _freq_)
               &statlist  ;
               run;
               proc summary data=tmpdate nway missing;
               id time;
               class datetime _cptype;
               var freq;
               output out=tmpdata(drop=_type _freq_) mean=freq;
               run;
     */          
     %let selvars=%add_dlm(&idvars &by_wodt &dt_tod_order);
     %put [&selvars.];
            proc sql;
               create table tmpdata as
               select &selvars., _cptype, &stat(&names) as &names
               from tmpdata
               group by &selvars., _cptype;
               quit;
               
               /* data sum; set tmpdata; run; rnb debug */
            %if &syserr>4 %then %do;
               %let &_rc=1;
               %put *********************************************************;
               %put ERROR: (&MNAME) Proc Summary Error see Log. Halting.;
               %put *********************************************************;
               %goto EXIT;
            %end;
         %end;
         proc sql noprint nofeedback; 
            create view classcnt as select distinct _cptype from &outds;
            select count(*) into :areas from work.classcnt;
            drop view classcnt;
            quit;
         %if &debug GT 1 %then %put DEBUG: areas [&areas];
         proc datasets lib=work nolist;
           modify tmpdata;
           format _cptype typef.;
         run;
         quit;
      %end;  /* of collapsing routine */
      
  %if "&period" NE "ASIS" %then %do;
     %************************************************;
     %* Fine-tune the horizontal time axis by setting ;
     %* the macro specified by haxis=.                ;
     %************************************************;
     %let pfold = %str(ASIS HOUR 15MIN 5MIN DATE WHOLEDAY WEEK MONTH QTR);
     %let pclump = %str(24HOUR WEEKDAY MONTHDAY YEARDAY YEARMONTH);
     %if "&period" EQ "24HOUR" or
         "&period" EQ "WEEKDAY" or
         "&period" EQ "MONTHDAY" or
         "&period" EQ "YEARDAY" or
         "&period" EQ "YEARMONTH" %then %do;
       %if &xvar= %str() or &xvar = %str(DATETIME) %then %do;
         proc summary data=&outds nway;
            var datetime;
            output out=work.tmpdata2(drop=_type_ _freq_)
               min=min max=max;
         run;
         %if &syserr>4 %then %do;
            %let &_rc=1;
            %put *********************************************************;
            %put ERROR: (&MNAME) Proc Summary Error see Log. Halting.;
            %put *********************************************************;
            %goto EXIT;
         %end;
         data _null_;
            set work.tmpdata2;
            length haxis $ 200;
            %if &period = 24HOUR %then %do;
               span = max-min;
               if span > '12:00't then unit='04:00't;
               else if span > '05:00't then unit = "02:00"t;
               else unit= "01:00"t;
               min = floor(min / unit) * unit;
               max = ceil(max / unit) * unit;
               haxis = 'haxis = "' || put(min, hhmm5.) || '"t to "' ||
                        put(max, hhmm5.) || '"t by "' || put(unit,hhmm5.)
                        || '"t';
            %end;
            %else %if &period = YEARDAY %then %do;
               span=max-min;
               if span le 30 then unit=1;
                  else if span le 60 then unit=10;
                  else if span le 120 then unit=15;
                  else unit=30;
               min = floor(min / unit) * unit;
               max = ceil(max/ unit) * unit;
               haxis = 'haxis = ' || put( min, 3. ) || ' to ' ||
                    put( max, 3. ) || ' by ' || put(unit, 2.);
            %end;
            %else %do;
               haxis = 'haxis = ' || put( min, 3. ) || ' to ' ||
                       put( max, 3. ) || trim( haxis );
            %end;
            call symput( "cphaxis", haxis );
            stop;
            run;
       %end;
     %end;
   %end;   %* End of Period collapsing;

   %************************************************************;
   %* For percents, calculate the percents specified, merge  *;
   %* the percents with the data, and change the variable to  *;
   %* represent which percents is represented.                 *;
   %************************************************************;

   %if "&percents" NE "" %then %do;
      proc format;
            value quant
         %let nq=0;
         %do i=1 %to 10;
             %if %scan(&percents,&i)^= %then %do;
                %let nq=%eval(&nq+1);
                &i="<=%scan(&percents,&i) Pctl"
             %end;
         %end;
         %eval(&nq+1)-HIGH=">%scan(&percents,&nq) Pctl"
         OTHER="Missing";
      run;
      proc univariate data=&outds noprint;
         &by_stmt
         var &xvar;
         output out=tmpq
            pctlpre=Q
            pctlpts=&percents;
      run;
      data &outds;
         merge &outds tmpq;
         &by_stmt
         array q
         %do i=1 %to &nq;
            q%scan(&percents,&i)
            %end;
         ;
         do i=1 to &nq until(&xvar<=q[i]); end;
         &xvar=i;
         format &xvar quant16.;
         keep &by &cl_nam &names &xvar datetime _cptype;
      run;
   %end;

%EXIT:
   %if "&period" EQ "24HOUR" or
         "&period" EQ "WEEKDAY" or
         "&period" EQ "MONTHDAY" or
         "&period" EQ "YEARDAY" or
         "&period" EQ "YEARMONTH" %then %do;
      %if %quote(&where) NE %quote() %then %do;
         %let whlocal=%str(%quote(&where));
         %let whlocal = %str(where &whlocal;);
      %end;
      %else %let whlocal=;
   %end;

%mend evxform;
