/***************************************************/
/* Copyright (c) 2014 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/



 /*+----------------------------------------------------------------+
  | Name:     findfiles.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  Searches a given directory for files matching a name pattern
  |
  | Inputs    dir - directory path
  |           outds - output dataset
  |           namematchrx - perl reg expression for file name string identifier
  |           i.e. <Audit.Library>
  |
  |           hostnamerx - perl reg expression to extract hostname from filename
  |           i.e.
  |     <hostnamerx=[\.\-\w]*_([\.\w]*)_[\.\w]*(?# Matching last _ bound group as hostname)>
  |
  |           pidrx - perl reg expression to extract process pid from filename
  |           i.e.
  |     <pidrx=[\.\-\w]*_([\w]*).[\w]*(?# Matching last _ . bound group as pid)>
  |
  |           sample logfile name which matches perl reg above would look like
  |           <Audit.Library_server_2013-11-21_hostname20_12549.log>
  |
  |           The above would use a log appender of the format
  |             <param name="FileNamePattern"
  |             value="sasconfig/WorkspaceServer/Logs/Audit.Library_server_%d_%S{hostname}_%S{pid}.log"/>
  |
  |           where hostname is the third substr delimited by underscore and pid is the last
  |
  | History:  Date    action
  |           24Nov13 Initial coding                        snornb
  |
  | Future:
  | End
  +----------------------------------------------------------------+*/

%macro findfiles(dir=, outds=, namematchrx=, hostnamerx=, pidrx=);
 
 %local mname;
 %let mname=findfiles;
 %if "&dir." EQ "" %then %do;
    %put "ERROR: (&MNAME) Requires valid directory path [&dir.]";
    %goto EXIT;
 %end;
 %if "&outds." EQ "" %then %do;
    %let outds=work.filelist;
    %put "NOTE: (&MNAME) Output dataset defaults to [&outds.]";
 %end;
 
 %let libname = %scan(&outds,1,'.');
 %let dsname = %scan(&outds,2,'.');
 proc datasets nolist nowarn dd=&libname;
    delete &dsname. / memtype=data;
 run;
      
 data &outds. (keep=directory logfile hostname pid modfdt);
    length directory $ 200.
           logfile $ 200.
           hostname $ 80.
           pid $ 8.
           modfdt 8. 
           lastmodified 8.
           ;
    format modfdt E8601DT20.;
    
    retain pxtlog pxtmachine pxtpid directory 
           datetime hostname pid modfdt;
    
    if _N_=1 then do;   
       directory="&dir.";
       %if &evdebug > 1 %then %put DEBUG: (&mname) namematchrx [&namematchrx];
       /*
         need to confirm on other os's to see if the mopen modified date
         is formated this way. arggg...  Fri Nov 29 04:30:02 2013 removes word date part
         reorders into time date for standard input datetime call
        */
       pxtdt=prxparse("s/.+ (.+) ([0-9]+) ([0-9]+:[0-9]+:[0-9]+) ([0-9]+)/$2$1$4 $3/");
       %if "%nrquote(&namematchrx)" NE ""  %then %str(pxtlog=prxparse("/%nrbquote(&namematchrx)/");)
       %if "%nrquote(&hostnamerx)" NE "" %then %str(pxtmachine=prxparse("/%nrquote(&hostnamerx)/");)
       %if "%nrquote(&pidrx)"  NE ""     %then %str(pxtpid=prxparse("/%nrquote(&pidrx)/");)
       ;
    end;
    
    rc=filename("ONE","&dir.");
    if rc NE 0 then do;
       msgLine="NOTE: Unable to assign a filename to directory [&dir.]";
       put msgLine;
       stop;
    end;
 
    did=dopen("ONE");
        
    if did GT 0 then do;
       numfiles=dnum(did);
       if &evdebug>1 then put "DEBUG: (&mname.)" '[' numfiles= ']';
       do i=1 to numfiles;
           logfile=dread(did,i);
           fid=mopen(did,logfile,"i");
           
           if fid EQ 0 then do;           
			
				%if &evdebug>1 %then %do;
				 
					put "DEBUG: (&mname.)" '[' 'action:bypassing this path. Reason: cannot be opened' ']';
					put "DEBUG: (&mname.)" '['  directory= logfile= ']' /;
				%end;
				
              continue;
           end;
           /*
              helps with debugging os differences from the finfo all
           */
           %if &evdebug GT 2 %then %do;
              numopt = foptnum(fid);
              do j = 1 to numopt;
                 optname = foptname(fid,j);
                 optvalue = finfo(fid,optname); 
                 put "DEBUG: (&MNAME) " numopt= optname= optvalue=;
              end;
           %end;
           optvalue = finfo(fid,'Last Modified');
           /*   NLDATM should work but it does not interesting  
           modfdt=input(optvalue,NLDATM30.);         
           */
           modfdt=input(prxchange(pxtdt,-1,optvalue),datetime24.);
           if &evdebug>2 then put optvalue= modfdt=;
           fid=fclose(fid);
           
           position=prxmatch(pxtlog,logfile);
           /* 
              namematch and positional matches for both pid and hostname embedded in name 
              as described in the help file above
           */
           if position NE 0 or "&namematchrx" EQ "" 
           then do;
              %if "%nrquote(&hostnamerx)" NE "" %then %do;
                 position=prxmatch(pxtmachine,logfile);
                 hostname = prxposn(pxtmachine, 1, logfile);
              %end;
              %if "%nrquote(&pidrx)"  NE ""     %then %do;
                 position=prxmatch(pxtpid,logfile);
    	            pid = prxposn(pxtpid, 1, logfile);
    	         %end;
              if &evdebug>1 then put "DEBUG: (&MNAME) " hostname= pid= position= logfile=;
              output;
           end;
        
       end; /* of do loop */
    end;
    else do;
       msg = sysmsg();
       put msg $;
    end;
  
    /* close the open filename and data set pointer */
    rc=filename("ONE");
    did=dclose(did);
 run;
   %EXIT:
%mend findfiles;

 