/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

 /*+----------------------------------------------------------------+
   | Name:     get_cas_session.sas                                  |
   | Product:  SAS Environment Manager                              |
   |                                                                |
   | Purpose:  Establishes a CAS session or reconnects to an        |
   |           one.  This macro is tenacious: if asked to connect   |
   |           to an existing CAS Session and it can't, it will try |
   |           to establish a new connection.                       |
   |                                                                |
   | History:  Date    action                                       |
   |           17MAR17 Initial Coding                        sasgzs |
   |           08MAY17 Removed default authinfo location     sasgzs |
   |           12MAY17 Added support for UUID input option   sasgzs |
   |           26MAY17 Specified HOST/PORT/AUTHINFO always   sasgzs |
   |           23JUN17 Added abortonfail parm, checking      sasgzs |
   |                   for unconfigured CAS info                    |
   |           28JUN17 Added calls to sessfound function,    sasgzs |
   |                   improved reconnect logic, messages           |
   |           30JUN17 Data mart lock release if aborting    sasgzs |
   |           14NOV17 Add Support for backup CAS controller sasgzs |
   |                                                                |
   | Global Macro Variables: This macro uses uses the following     |
   |           global macro variables                               |
   |                                                                |
   |           CAS_CONNECT_RC - used to pass a return code to the   |
   |                   calling program.                             |
   |                                                                |
   |   Value | Meaning (related action)                             |
   |  -------+----------------------------------------------------- |
   |       0 | Action completed w/o problems                        |
   |     195 | CAS Configuration information has not been provided  |
   |     196 | CAS connection could not be established              |
   |                                                                |
   +----------------------------------------------------------------+*/

%macro get_cas_session(session_name=evmsvropsSess,/* Name of CAS session */
                       uuid=,                     /* UUID of an existing CAS session */
                       host=&cas_server_host,     /* CAS server (sole/primary) hostname */
                       host2=&cas_server_host2,   /* CAS server (backup) hostname */
                       port=&cas_server_port,     /* CAS server port number */
                       authinfo=&authinfo_file,   /* authinfo file containing CAS credentials */
                       abortonfail=N);            /* Y|N; abort the SAS session if CAS connection cannot be established */
                      
  %global cas_connect_rc;
  %global _cas_session_uuid_;


  %if not %symexist(AUTHINFO_FILE) %then %local authinfo_file;;

  %local auth_string;

  %if "&authinfo_file" ne "" %then %let auth_string=%str(authinfo="&authinfo_file");;
  
  %if "&host2" eq " " %then %let host2=NONE;
  
  %if "&host2" ne "NONE" %then 
  %do; /*backup controller identified */
  
     %put DEBUG: Backup CAS controller specified [&host2];
     
     %let host_string=("&host", "&host2");
     
  %end;/*backup controller identified */
  %else 
  %do; /*no backup controller identified */
 
     %let host_string="&host";
     
  %end;/*no backup controller identified */


  %if %upcase("&host") eq "@@CAS_SERVER_HOST@@" or %upcase("&port")  eq "@@CAS_SERVER_PORT@@" %then
  %do; /* check that CAS connection information has been provided */

         %evcomment(/H,/C,ERROR!,
                    /C, REQUIRED CAS CONFIGURATION INFORMATION NOT PROVIDED!,,
                    /L, It appears that a critical configuration step has NOT been performed.,
                    Configuration changes identifying the CAS Server Host and/or the CAS Server Port have NOT been made.,,                   
                    /D,/C, PROCEED WITH CAUTION,/D,,
                    /L, Establishing a connection to a CAS server is NOT possible with the current configuration information.,
                    In almost all cases you should exit now and complete the necessary configuration steps before attempting this again.,
                    /D,,
                    /C, CAS_SERVER_HOST%str(:) &host,
                    /C, CAS_SERVER_PORT%str(:) &port,
                    /H
                    );

     %let cas_connect_rc=195;
     %goto ERROR;

  %end;/* check that CAS connection information has been provided */
  %else %if "&uuid" ne "" and ("&uuid" ne "&_CAS_SESSION_UUID_") %then
  %do; /* attempt to connect to specified CAS session */
       
     %put DEBUG: Re-connecting to the specified CAS session [&uuid];
     cas &session_name host=&host_string port=&port
                       &auth_string 
                       uuid="&uuid" uuidmac=_CAS_SESSION_UUID_;

     %if &casstmterr eq 0 %then %goto EXIT;
     
     %put WARNING: Unable to re-connect to the specified CAS session [&uuid] %str(;) will attempt to establish a new connection;
     %symdel casstmterr; /* reset error flag before attempting again */
    /* drop through to the CAS statement */
    
  %end;/* attempt to connect to specified CAS session */
  %else %if "&_CAS_SESSION_UUID_" ne "" %then
  %do; /* CAS Session already exists */

     %if %sysfunc(sessfound(&session_name)) %then
     %do; /* session still active */
    
        %put DEBUG: Re-using the existing CAS session;
        %goto EXIT;
     
     %end;/* session still active */
     %else
     %do; /* re-establish connection */
    
        %put DEBUG: Attempting to re-connect to the existing CAS session;
        
        cas &session_name reconnect;
        
        %if &casstmterr eq 0 %then %goto EXIT;
        %else
        %do; /* unable to re-connect */
       
           %put WARNING: Unable to re-connect to the existing CAS session %str(;) will attempt to establish a new connection;
           /* drop through to the CAS statement */
           %symdel casstmterr; /* reset error flag before attempting again */
        %end;/* unable to re-connect */
     %end;/* re-establish connection */
    
  %end;/* CAS Session already exists */
 
  cas &session_name host=&host_string port=&port  &auth_string uuidmac=_CAS_SESSION_UUID_;

  %if &casstmterr eq 0 %then %goto EXIT;

  %let cas_connect_rc=196;
  %goto ERROR;
 %return;


 %EXIT:
  %let cas_connect_rc=0;
 %return;

 
 %ERROR:
  %if %upcase("&abortonfail") eq "Y" %then %goto EXIT_ABORT;     
 %return;   

  
 %EXIT_ABORT:
  %datamartlock(action=RELEASE);  /* release lock on data mart if it exists */

  %put %str(ERROR: Terminating session because CAS connection information missing or incorrect (rc=[&cas_connect_rc]).);
  %abort return &cas_connect_rc;
 %return;
  

%mend get_cas_session;


