/***************************************************/
/* Copyright (c) 2015 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

%macro getnobs(data=,      /* data set name */
               mvar=,      /* output macro var name */
               where=,     /* optional, where clause to apply */
               stat=NOBS); /* NOBS|ANY; must be ANY when passing in WHERE clause */

/* ************************************************ */
/* GETNOBS Macro                                    */
/*                                                  */
/* This macro returns the number of observations in */
/* the specified data set.  If a WHERE clause is    */
/* specified as well, this macro will return a 1 or */
/* 0 to indicate whether there are any matching obs */
/* If a WHERE clause is specified, the STAT para-   */
/* meter is required and must be ANY. The ANY stat  */
/* can also be specified WITHOUT a WHERE clause if  */
/* a simple true/false (1/0) value is desired.      */
/*                                                  */
/* If the specified data set does NOT exist, this   */
/* macro will return a '.'.                         */
/*                                                  */
/* The specified macro variable is used to pass the */
/* return value to the calling program.  The calling*/
/* program is responsible for defining the scope of */
/* that macro variable; e.g. including it in a      */
/* %LOCAL or %GLOBAL statement.  If this is NOT     */
/* done, the specified macro variable will be de-   */
/* fined in the local (to this macro) symbol table, */
/* and will NOT be available to the calling program.*/
/* ************************************************ */
 
 %let MNAME=GETNOBS;

%if (&stat ne ANY and %bquote("&where") ne "") or (%bquote("&where") ne "" and &stat ne ANY) %then
%do; /* invalid parameters */

     %put ERROR: Invalid combinations of parameters passed to the &MNAME macro;
     %put ERROR: A WHERE clause can only be specified if the requested statistic (STAT) is ANY;
     %let &mvar=.;
     %return;

 %end;/* invalid parameters */
 %else %if "&stat" ne "ANY" and "&stat" ne "NOBS" %then
 %do; /* invalid stat parameter */

     %put %str(ERROR: Invalid STAT parameter passed to the &MNAME macro; valid values are NOBS or ANY);
     %let &mvar=.;
     %return;
  
 %end;/* invalid stat parameter */

 %if %sysfunc(exist(&data)) %then
 %do; /* table exists; get nobs */

    /* set to 0 in case there are no obs */
    data _null_;
     call symputx("&mvar",0,'F');
    run;

    data _null_;
     set &data(obs=1) nobs=nobs;
     &where;

     %if &stat eq ANY %then %let nobstr=1;
     %else %let nobstr=nobs;
     
     call symputx("&mvar",&nobstr,'F');
     stop;
    run;

 %end;/* table exists; get nobs */
 %else
 %do; /* table does NOT exist; return missing */

    data _null_;
     call symputx("&mvar",'.','F');
     call symputx("ANYOBS",'.','F');
    run;

 %end;/* table does NOT exist; return missing */

%mend getnobs;
