/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/



 /*+----------------------------------------------------------------+
  | Name:     load_tsv_from_dir.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  Reads whitelist during ETL processing and writes SAS source
  |           code to temp to later be INCLUDEed
  |
  |
  | Inputs    whitelist
  |
  |
  | History:  Date    action
  |           05Apr17 Initial Coding                              snornb
  |           
  | Future:
  | End
  +----------------------------------------------------------------+*/
%macro load_tsv_from_dir(data_dir=, 
                         cutoff_date=0,
                         etl_mname=,
                         resourcetype=,
                         resource_category=, 
                         update_cas=);

 %if %sysfunc(fileexist(&data_dir)) eq 0 %then 
 %do; /* confirm specified data directory exists */

    %put %str(NOTE: The specified data directory [&data_dir] does not exist; skipping this directory); 
    %return; 

 %end;/* confirm specified data directory exists */
 

 filename ltfdcode temp;

 libname rt "&data_dir";

 %findfiles(dir=&data_dir, 
             outds=work._tsvfiles, 
             namematchrx=.*Z-.*\.tsv);

 proc sort data=work._TSVFILES out=work.found(rename=(logfile=tsv_file_name));
  by logfile;
 run;


 %if %sysfunc(exist(RT.INVENTORY)) eq 0 %then
 %do; /* initialize the table if it does not exist */
    data rt.inventory;
      length tsv_file_name $200 file_ext $3 file_date 8 finalized $1 last_moddate 8 last_rec 8;
      format last_moddate datetime. file_date yymmddn8.;
     stop;
    run;
 %end;/* initialize the table if it does not exist */

 lock rt.inventory;

 data inventory;
  length line $2000 caslib $256.;
  merge rt.inventory(in=known) work.found(in=found);
  by tsv_file_name;

  file ltfdcode;
 
  if finalized eq "Y" then return;

  if found and known then
  do; /* file was previously found */
      
     /* ignore files that haven't */
     /* been modified since their */
     /* info in RT.INVENTORY was  */
     /*  last updated.            */
     if last_moddate eq modfdt then return;

     update_action="UPDATE";
     next_rec=max(1,last_rec);
    
  end;/* file was previously found */
  else if found and not known then do; update_action="ADD"; next_rec=1; end;
  else if known and not found then return; /* ignore 'obsolete' files */


  if file_date eq . or file_ext eq "" then
  do; /* extract file_date and file_ext */    

     re=prxparse('/((\d{4})(\d{2})(\d{2}))(\w*)-(\S*)\.(\w*)/');
     match=prxmatch(re,tsv_file_name);

     file_date=input(prxposn(re,1,tsv_file_name),yymmdd8.);
          /*YEAR: 2 */
          /*MONTH: 3 */
          /*DAY: 4 */
          /*misc text: 5 */
     vstring=prxposn(re,6,tsv_file_name);
     file_ext=prxposn(re,7,tsv_file_name);

     drop re match vstring;

  end;/* extract file_date and file_ext */    

  /* ONLY PROCESS FILES MORE RECENT THAN THE CUTOFF */
  if file_date lt  &cutoff_date then return; 


  filename=catt(directory,'/',tsv_file_name);

  put '%let EVDMERR=N;';
  put '%let atobs=0;';

  if "&resource_category" eq "log" then
  do; /* dealing with logs */ 

     caslib="&CAS_SEARCH_CASLIB";
     casmemname="&CAS_SEARCH_IDXNAME";

  end;/* dealing with logs */ 
  else
  do; /* non-log data */

     caslib="&caslib_METRIC_DATA";
     casmemname="&resourcetype";

  end;/* non-log data */

  line=catt('%',"&etl_mname",'(inputtsv=',filename,
            ',firstobs=',next_rec,
            ",resource_category=&resource_category",
            ",resourcetype=&resourcetype",
            ',update_cas=',"&update_cas",
            ',caslib=',caslib,
            ',casmemname=',casmemname,
            ');');   
  put line;

  line=catt('%update_rtinventory(action=',update_action,
            ', inventory=rt.inventory, tsv_file_name=',tsv_file_name,
            ', file_date=', put(file_date,best.),
            ', file_ext=', file_ext,
            ', last_rec=&atobs, last_moddate=',modfdt,
            ', finalized=',finalized,');');
  put line;
  
 run;

 %include ltfdcode /source2 ;

 lock rt.inventory clear;
 
%mend load_tsv_from_dir;
