/***************************************************/
/* Copyright (c) 2013 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

/*+----------------------------------------------------------------+
   | Name:     logparse_sasjobs.SAS
   |
   | Language: SAS
   | Purpose:  Parse log records for the SASJob logfile. This parser is based
   |           on the logparse tool pool entry with updates to work and function
   |           in the SASEV foundation framework.
   |
   | Parameters:
   | Usage:
   |       dir    host directory path of log files to process
   |       outds  output library and dataset of processed log records
   |       append append flag, ( YES/NO ), appends new artifact records
   |              from all logs in dir location into 
   |              an existing dataset assuming the same or compatiable
   |              record structure.
   |       debug  debug level, follows SASEV standards 0,1,2 or 3.
   |
   | History:
   |    date    action
   |    07Nov13 Creation                                           RNB
   | end
   +----------------------------------------------------------------+*/


/******************************************************************
*
* The %LOGPARSE() macro extracts performance statistics from a
* SAS log file.
* 
where:

   saslog  = SAS log file (for MVS, see "pdsloc" below).
   out     = output SAS data set for results (optional):
             - if not specified, WORK.DATAn is created, where n is
               the smallest integer that makes the name unique (same
               behavior as "data; x = 1; run;").
             - if specified, names the data set created by
               this invocation of %logparse().
             - if specified and append=yes is specified, see
               documentation of the append= parameter, below.
   system  = 3-character operating system code. This parameter is
             optional; the default value is the sysem on which
             %logparse() is run. Valid codes are:
              z/OS, OS/390, or MVS    = MVS or OS
              OpenVMS Alpha           = ALP
              OpenVMS VAX             = VMS
              All other OSs           = OTH
             For more information, see "SYSSCP and SYSSCPL Automatic
             Macro Variables" in SAS Macro Language: Reference:
             Macro Language Dictionary in the SAS online documentation.
   pdsloc  = When %logparse() is executed on an MVS system and the SAS
             log file to be analyzed is stored in a partitioned data
             set (PDS), "pdsloc" partially names the PDS and "saslog"
             names the member.  The PDS name is generated with a
             leading period (making your userid the first level of the
             name) and the lowest level LOGS.
             Example:  SAS log is MYACCT.PRJ5.GRP24.LOGS(TEST47)
                       %logparse(test47, , , prj5.grp24);
   append  = create or append to the output data set (see the
             "out" parameter, above).  This parameter is optional;
             the default value is NO.  Valid values are YES and NO:
             NO  = %logparse() creates a new SAS file according to the
                   rules for the out parameter, above.
             YES = %logparse() appends its output to the file named by
                   the out parameter (the file is created if it does
                   not already exist).
             Example:  %logparse(mypgm.log, lpout, VMS, append=yes);
* See the README file for instructions and syntax.
*
*******************************************************************/

%macro logparse_sasjobs(dir=, outds=, namematchrx=,append=YES,
   useJOBHeaderDT=YES, system=, pdsloc=, platform=, context=, instance=);

   /**************************************************************
   * Standard Header infor for SASEV logparsers
   *
   ***************************************************************/
   
   %local mname i outflag dsid;
   %let hostname=%str();
   %let pid=%str();
   %let modfdt=%str();
   
   %let mname=logparse_sasjobs;
   %let append=%upcase(&append);

   %evcomment((&MNAME) ...initializing  RUNID [&runid]);
   
   %evTIMLOG( start );
   %evsetdebug(SET,_debug=&evdebug);
   
   %if "&dir" EQ "" %then %do;
      %put ERROR: (&MNAME) Missing log directory [&dir];
      %goto EXIT;
   %end;
   
   %if "&outds" EQ "" %then %do;
      %let outds=%str(work.recordsperdir);
      %let outflag=%str(DEFAULT);
      %evcomment((&MNAME) No out library.dataset, using default [&outds.]);
   %end;

   %if "&namematchrx" EQ "" %then %do;
      %let namematchrx = %bquote(.[L|l]og);
      %put WARNING: (&MNAME) Namematchrx blank, parse all ending in .log in [&dir];
   %end;
  
   %if %sysfunc(libref(KITS)) %then %do; /* returns nozero if not assigned */
      %put ERROR: (&MNAME) Libref KITS not defined;
      %goto EXIT;
   %end;
   
   /***************************************************************
   * If no system was specified, set it to the ID of the system
   * under which this proram is running, on the assumption that
   * one kind of computer is being used for both generating and
   * analyzing the logs.
   ****************************************************************/
   %if ( &system = )  %then  %let system = &sysscp;

   /* Look for files to process in a directory. */
   
   %let list=work.loglist2;
   %findfiles(dir=&dir,outds=&list.,namematchrx=%nrbquote(&namematchrx.));

   %let dsid = %sysfunc(open(&list));
   %if &dsid %then %do;
      %syscall set(dsid);

      %let nobs=%sysfunc(attrn(&dsid,nlobs));
      %if &evdebug>1 %then %put DEBUG: (&MNAME) Number of directory members from [&list] nobs [&nobs]; 
       
      %if "&nobs." EQ "0" %then %do;
        %let rc = %sysfunc(close(&dsid));
        %put NOTE: (&MNAME) No logs found to process in [&dir];
        %goto EXIT;
      %end;
        
      proc datasets nolist nowarn dd=work;
         delete recordsperlog / memtype=data;
         delete recordsperdir / memtype=data;
      run;
      %if NOT %sysfunc(exist(work.KLOGSPthisrun)) %then %do;
	    data work.KLOGSPthisrun;
	       set kits.LOGSPROCESSED;
	       stop;
	    run;
      %end;
      
      %do i=1 %to &nobs;
         %let rc=%sysfunc(fetch(&dsid));
         %let ldir=%sysfunc(getvarc(&dsid,%sysfunc(varnum(&dsid,DIRECTORY))));
         %let lfile=%sysfunc(getvarc(&dsid,%sysfunc(varnum(&dsid,LOGFILE))));
         %let hostname=%sysfunc(getvarc(&dsid,%sysfunc(varnum(&dsid,HOSTNAME))));
         %let pid=%sysfunc(getvarc(&dsid,%sysfunc(varnum(&dsid,PID))));
         %let modfdt=%sysfunc(getvarn(&dsid,%sysfunc(varnum(&dsid,MODFDT))));
         filename auditlog "&ldir./&lfile."; 
          
         %evcomment((&MNAME) Processing logfile &lfile &i/&nobs,
                    hostname [&hostname] modfdt [&modfdt] pid [&pid]);
         
         data recordsperlog (compress=yes);
 
      length logfile $ 200 stepname runid $ 32 step $ 20
             line $200 upcase_Line $ 200 prevline $ 200
             stepdesc hostname $80
             portdate $ 25 keyword $ 20
             platform scp $ 50 /* UNIX needs this length */
             prevblanks blanks coloncnt reprocess
             realtime usertime systime cputime
             pageflt pagercl pageswp osvconsw osiconsw
             excsemap shrsemap consemap
             blkinput bkoutput buffio dirio
             vatime vutime rsmtime excpcnt
             tskmemd tskmemp totmemd totmemp belowmem abovemem
             obsin obsout varsout
             threads events locks pools poolsx 8;

      format realtime usertime systime cputime time12.2
             datetime datetime. tzoneoff hhmm.
             memused osmem comma20.2
             sumsize sortsize memsize sizekmg.
             pagercl bkoutput blkinput osiconsw osvconsw 
             pageflt osiconsw bkoutput pagercl osvconsw blkinput pageswp comma20.;

      label  bkoutput = 'Block Output Operations'
             blkinput = 'Block Input Operations'
             buffio   = 'Buffered I/O'
             consemap = 'Contended Semaphores'
             cputime  = 'CPU Time'
             datetime = 'Run Date/Time or Last Modified Date/Time (UTC)'
             dirio    = 'Direct I/O'
             events   = 'Events'
             excpcnt  = 'I/O Count'
             excsemap = 'Exclusive Semaphores'
             hostname = 'System Name'
             jobname  = 'User-specified Job Name'
             phase    = 'User-specified Job Phase'
             locks    = 'Locks'
             logfile  = 'Log File Name'
             memused  = 'Memory Used (K)'
             osmem    = 'OS Memory (K)'
             obsin    = 'Observations Read'
             obsout   = 'Observations Written'
             osiconsw = 'Involuntary Context Switches'
             osvconsw = 'Voluntary Context Switches'
             pageflt  = 'Page Faults'
             pagercl  = 'Page Reclaims'
             pageswp  = 'Page Swaps'
             platform = 'Platform'
             pools    = 'Memory Pools Created'
             poolsx   = 'Memory Pools Destroyed'
             portdate = 'SAS Port Date'
             realtime = 'Elapsed Time (sec)'
             rsmtime  = 'RSM Hiperspace'
             scp      = 'Operating System'
             shrsemap = 'Shared Semaphores'
             stepcnt  = 'Step Number'
             stepname = 'Step Name'
             stepdesc = 'Step Description'
             runid    = 'Run ID'
             step     = 'Step'
             systime  = 'System CPU Time (sec)'
             threads  = 'Threads'
             totmemd  = 'Total Memory - data'
             totmemp  = 'Total Memory - program'
             tskmemd  = 'Task Memory - data'
             tskmemp  = 'Task Memory - program'
             belowmem = 'Total Memory - below line'
             abovemem = 'Total Memory - above line'
             usertime = 'User CPU Time (sec)'
             varsout  = 'Variables Written'
             vatime   = 'Vector Affinity'
             vutime   = 'Vector Usage'
             sumsize  = 'SUMSIZE System Option Value'
             timezone = 'Time Zone'
             sortsize = 'SORTSIZE System Option Value'
             memsize  = 'MEMSIZE System Option Value'
             tuneopp  = 'Tunning Suggestion'
             ;

      retain line upcase_Line prevline blanks prevblanks logfile
             obsin obsout varsout;
             
      retain logfile "&lfile"
             tzoneoff
             stepcnt 0
             hdrfnd 0         /* 0=no hdr yet, 1=hdr fnd, 2=hdr finish*/
             portdate
             scp
             pid
             datetime
             platform         /* defined in RELNAME routine           */
             hostname
             jobname
             phase
             parm
             stepname
             stepdesc
             runid
             sumsize
             timezone
             sortsize
             memsize
             tuneopp
             ;

      /*****************************************************************
      * Moved the DROP list down here instead of an option on the DATA
      * statement to avoid generating a syntax error if no output data
      * set name is specified on the macro invocation.
      *****************************************************************/
      
      drop line prevline reprocess coloncnt inbuf pos upcase_Line
           prevblanks blanks indxFld indxLen hdrfnd keyword _I_ len
           excsemap shrsemap  consemap  buffio  dirio  vatime  
           vutime rsmtime  excpcnt  tskmemd  tskmemp  totmemd  totmemp   
           belowmem  abovemem threads  events  locks  pools  poolsx
           ;  

      %let ar_bnd = 4;        /* change value when array is changed   */
      array ar_hdr (&ar_bnd) $20  _temporary_ ('PASS HEADER'
                                               'JOB HEADER'
                                               'APR HEADER'
                                               'WTEST HEADER');
                                               
  /*  infile auditlog recfm=V lrecl=512 pad missover;*/
      infile auditlog lrecl=250 end=lastrec pad truncover;
                                                    

      /************************************************************
      * GETLINE:
      * Read in a line from the LOG file, skipping over blank
      * lines as well as page breaks.  Page breaks can be denoted
      * by "THE SAS SYSTEM" title line, but this string can also
      * appear in a NOTE: statement, so skip the former but keep
      * the latter.  Also skip lines which begin with a number,
      * like a right-justified datetime stamp.  Ordinarily such
      * lines appear on the same line as "THE SAS SYSTEM", but if
      * the user has set the LINESIZE option to a small number,
      * they are sometimes placed on the next line.
      * Stop if we hit the end of the file.
      * Remove any carriage returns (presumably from PC runs).
      * Save off the previous and current amount of indentation
      * to detect whether we're still in a FULLSTIMER block.
      * if lastrec 0, input lines until NOTE: line or lastrec or
      * 
      *************************************************************/
      %macro getline;
         if (lastrec = 0) then do;
            prevline = line;
            do until ( (lastrec = 1) or
                       (upcase_Line =: 'NOTE:') or
                       ((compress(line) ne '') and
                        (substr(line,1,1) ne '0c'x) and
                        (compress(scan(upcase_line,1,' '),
                                  ':0123456789') ne '') and
                        (index(upcase_Line, 'THE SAS SYSTEM') le 0)) );
               input line $char200.;
               upcase_Line = upcase(line);
               if (upcase_Line =: 'MPRINT') then continue;
            end;
            if (upcase_Line =: 'MPRINT') then do;
               prevline=line;
               line='';
               upcase_Line='';
            end;
            line = compress(line, '0d'x);
            prevblanks = blanks;
            blanks = length(line) - length(left(line));
         end;
         else do;
            stop;
         end;
      %mend getline;

      /************************************************************
      * Read in the very first line of the LOG.
      * All subsequent lines are read in at the bottom of the loop.
      * The algorithm is written in this way to get around a
      * problem by which SAS detects a false looping condition if
      * no INPUT statement is executed in a given DATA STEP
      * iteration.
      *************************************************************/
      if (_n_ = 1) then do;
         %getline;
      end;

      /************************************************************
      * Certain steps are preceded with NOTEs containing
      * the number of observations read and/or the number of
      * observations and variables written.  Accumulate these
      * values to be written with the next observation.
      *************************************************************/
      if (index(upcase_Line,'NOTE: THERE WERE ') and
          index(upcase_Line,' OBSERVATIONS READ FROM THE DATA SET'))
      then do;
         obsin + input(scan(line,4,' '), 20.);
      end;
      else if (index(upcase_Line, 'NOTE: THE DATA SET ') and
               index(upcase_Line, ' OBSERVATIONS AND ') and
               index(upcase_Line, ' VARIABLES.')) then do;
         obsout + input(scan(line,7,' '), 20.);
         varsout + input(scan(line,10,' '), 20.);
      end;
      else if (index(upcase_Line, 'NOTE: PROCESSING ON DISK OCCURRED')) then do;
      /* 
       * triggers on string 
       * NOTE: Processing on disk occurred during summarization. Peak disk usage was approximately 
       * 3206 Mbytes. Adjusting MEMSIZE and/or REALMEMSIZE may improve performance.
       */  
         tuneopp=1;
      end;
      
      /************************************************************
      * Process JOB HEADER information
      * Look for JOB HEADER keywords (Also looks for the 9.1 PASS INFO Keyword)
      * Each line can contain only one keyword,
      * so quit looking when a keyword is found
      *************************************************************/
      if (line = ' ' and hdrfnd = 1) then
         hdrfnd = 2;                                /* header finished */
      if (hdrfnd = 0 and (portdate = ' ' or
                          platform = ' ' or
                          scp = ' ')) then do;
         link RELNAME;
      end;

      /*************************************************************
      * Start of JOB HEADER loop
      **************************************************************/
      if (hdrfnd = 0 or hdrfnd = 1 ) then do;       /* check for JOB HEADER info */
         do _I_ = 1 to &ar_bnd;
            keyword = ar_hdr(_I_);
            /* search keyword, trim needed */
            pos = index(upcase_Line, trim(keyword));
            if pos > 0 then do;                    /* keyword found */
               len = length(keyword);
               _I_ = &ar_bnd + 1;                  /* stop looking */
            end;
         end;
         keyword = ' ';                            /* clear last search lit */
         if (pos ge 1 and pos le 5) then do;       /* must be in line begining*/
            hdrfnd = 1;
            upcase_Line = substr(upcase_Line, pos + len);
            keyword = left(scan(upcase_Line,1,'='));/* find var indicator */
            select;                                /* scan out var value */
               when (keyword = 'JOBNAME')
                  jobname = scan(substr(Line,pos+len),2,"=");
               when (keyword = 'RPTDATE')
                  datetime = input(scan(upcase_Line,2,"="),E8601DT20.);
               when (keyword = 'STEPNAME')
                  stepname = scan(upcase_Line,2,"=");
               when (keyword = 'STEPDESC')
                  stepdesc = scan(upcase_Line,2,"=");
               when (keyword = 'RUNID')
                  runid = scan(upcase_Line,2,"=");
               when (keyword = 'OS')
                  scp = scan(upcase_Line,2,"=");
               when (keyword = 'HOST') do;
                  hostname = scan(upcase_Line,2,"=");put hostname=; end;
               when (keyword = 'PARM')
                  parm = scan(upcase_Line,2,"=");
               when (keyword = 'VER')
                  portdate = scan(upcase_Line,2,"=");
               when (keyword = 'SUMSIZE')
                  sumsize = input(scan(upcase_Line,2,"="),20.);   
               when (keyword = 'TIMEZONE')
                  timezone = scan(upcase_Line,2,"=");   
               when (keyword = 'SORTSIZE')
                  sortsize = input(scan(upcase_Line,2,"="),20.);   
               when (keyword = 'MEMSIZE')
                  memsize = input(scan(upcase_Line,2,"="),20.);
               /* add additional variable value scans here */
               otherwise;
            end;  /* select */
         end;  /* if (pos ge 1 and pos le 5) */
      end;  /* if (hdrfnd = 0 or hdrfnd = 1) */


      /************************************************************
      * All other systems share the same basic FULLSTIMER format.
      *************************************************************/

         /*********************************************************
         * If we detect a line beginning "Total Elapsed Time", we
         * must be parsing a Java Test Client Application that is
         * exercising an OMR server.  If so, simply read this line
         * in as the sole statistic to be captured for this test.
         * We are currently ignoring "Elapsed Time" lines in these
         * files since there were too many of them and they were
         * making the qatrack PERFREL data sets too large.
         **********************************************************/
         if (index(line, "Total Elapsed Time:") = 1) then do;
            realtime = input(scan(line, 4, ' '), 20.);
            step = 'Total';
            output;
            stepcnt+1;
         end;

         /*********************************************************
         * As soon as we detect the first statistic in the current
         * block, start reading them all in.
         **********************************************************/
         else if (index(line, "real time ") > 0) then do;
            if (index(prevline, 'DATA ') > 0) then
               step=scan(prevline, 2, ' ');
            else
               step=scan(prevline, 3, ' ');


            /******************************************************
            * TIMESTAT:
            * Parses statistics that could be in time format
            * (i.e., HH:MM:SS.dd).
            * Use it to parse realtime.
            *******************************************************/
            %macro timestat(var, statword);
               inbuf = scan(line, &statword, ' ');
               coloncnt = length(inbuf) -
                          length(compress(inbuf,':'));

               if (coloncnt = 0) then
                  inbuf = '0:0:' || trim(inbuf);
               else if (coloncnt = 1) then
                  inbuf = '0:' || trim(inbuf);
               &var = input(inbuf, time12.3);
            %mend timestat;

            %timestat(realtime, 3);

            /******************************************************
            * Read in the remaining FULLSTIMER lines.  We stop
            * reading either when a line is found that isn't
            * indented (ignoring page breaks) or we reach the end
            * of the LOG.
            *******************************************************/
            %getline; 
            do while (blanks = prevblanks);
               if (index(line,"user cpu time") > 0) then do;
                  %if (("&system" = "ALP") or
                       ("&system" = "VMS_AXP") or
                       ("&system" = "VMS")) %then %do;
                     %timestat(cputime, 4);
                  %end;
                  %else %do;
                     %timestat(usertime, 4);
                  %end;
               end;

               else if (index(line,"system cpu time") > 0) then do;
                  %timestat(systime, 4);
               end;

               else if (index(line,"cpu time") > 0) then do;
                  %timestat(cputime, 3);
               end;

               else if (index(line,"Semaphores") > 0) then do;
                  excsemap=input(scan(line,3," "), 20.);
                  shrsemap=input(scan(line,5," "), 20.);
                  consemap=input(scan(line,7," "), 20.);
               end;

               /***************************************************
               * GETSTAT:
               * Retrieves an integer statistic and places it in
               * the passed variable.
               ****************************************************/
               %macro getstat(text, var);
                  else if (index(line, "&text") gt 0) then do;
                     &var = input(scan(line,-1,': '), 20.);
                  end      /* do NOT put a semi-colon on this line */
               %mend getstat;

               %getstat(Page Faults, pageflt);
               %getstat(Page Reclaims, pagercl);
               %getstat(Page Swaps, pageswp);
               %getstat(Voluntary Context Switches, osvconsw);
               %getstat(Involuntary Context Switches, osiconsw);
               %getstat(Block Input Operations, blkinput);
               %getstat(Block Output Operations, bkoutput);
               %getstat(Buffered IO, buffio);
               %getstat(Direct IO, dirio);
               %getstat(Threads, threads);
               %getstat(Events, events);
               %getstat(Locks, locks);
               %getstat(Memory Pools  Created, pools);
               %getstat(Memory Pools  Destroyed, poolsx);

               else if (index(line,"OS Memory ") > 0) then do;
                  osmem = input(compress(scan(line,3,' '),'k'), 20.);
               end;

               else if (index(line,"memory ") > 0) then do;
                  memused = input(compress(scan(line,2,' '),'k'), 20.);
               end;

               %getline;
            end;  /* do while (blanks = prevblanks) */

            /******************************************************
            * Make sure we reprocess the last line read in since
            * it is not part of the FULLSTIMER block.
            *******************************************************/
            reprocess = 1;

        /************************************************************
	    * JOB HEADER 
	    * Set hostname, pid, file modified datetime from JOB HEADER 
	    * If missing, use file last modified, hostname and pid from file 
	    * properties if missing
	    ************************************************************/
	    if datetime EQ . then datetime = "&modfdt";
	    if hostname EQ "" then hostname = "&hostname";
            if pid EQ "" then pid = "&pid";

            /****************************************************** 
            * Adding required tz and shift variable with value inputs into
            * the to dataset following the SASEV Datamart standard
            *******************************************************/
            %addtzoffvar(datetime=datetime, hostvar=hostname, outvar=tzoneoff, form=DATASTEP);
	       %addshiftvar(datetime=datetime, offsetvar=tzoneoff, outvar=shift, 
	            shiftfmt=shifttxt, prefixvar=hostname, form=DATASTEP);
            
            /******************************************************
            * Output an observation corresponding to the current
            * block of statistics.
            * Clear out the retained variables which contain info
            * from prior NOTE lines.
            *******************************************************/
            if cputime EQ . then do;
               cputime=usertime + systime;
            end;
            output;
            stepcnt+1;
            obsin = .;
            obsout = .;
            tuneopp = .;
            varsout = .;
         end;

      /************************************************************
      * If we haven't already, read in the next line.
      *************************************************************/
      if (reprocess ne 1) then do;
         %getline;
      end;

      return;

      /************************************************************
      * Identify the port date and platform.
      *************************************************************/
      RELNAME:
         length indxFld $255;
         indxFld = "PROPRIETARY SOFTWARE PRE-PRODUCTION VERSION";
         pos = indexw(upcase_Line, indxFld);
         if (pos = 0) then do;
            indxFld = "PROPRIETARY SOFTWARE RELEASE";
            pos = indexw(upcase_Line,indxFld);
         end;
         if (pos = 0) then do;
            indxFld = "PROPRIETARY SOFTWARE VERSION";
            pos = indexw(upcase_Line,indxFld);
         end;
         if (pos > 0) then do;
            /* drop initial part of line */
            indxLen = length(trim(indxFld));
            upcase_Line = substr(upcase_Line,pos + indxLen + 1);
            portdate = compress(scan(upcase_Line,-2,' ') || ' ' ||
                                scan(upcase_Line,-1,' '), '()');
         end;
         indxFld = 'THIS SESSION IS EXECUTING ON THE';
         pos = indexw(upcase_Line, indxFld);
         if (pos > 0) then do;
            /* drop initial part of line */
            indxLen = length(trim(indxFld));
            upcase_Line = substr(upcase_Line,pos + indxLen + 1);
            line = substr(line,pos + indxLen + 1);
            /* find end of platform name */
            pos = index(upcase_Line, "PLATFORM");
            if pos > 1 then do;
               /* save original case of platform name */
               platform = substr(line, 1, pos - 1);
            end;
         end;
         indxFld = 'RUNNING ON';
         pos = indexw(upcase_Line, indxFld);
         if (pos > 0) then do;
            /* drop initial part of line */
            indxLen = length(trim(indxFld));
            upcase_Line = substr(upcase_Line,pos + indxLen + 1);
            line = substr(line,pos + indxLen + 1);
            /* find end of platform name */
            pos = index(upcase_Line, ".   ");
            if (pos le 1) then
               pos = length(trim(upcase_Line)) + 1;
            /* save original case of operating system name */
            scp = substr(line, 1, pos - 1);
         end;
      return;  /* RELNAME: */
   run;
 
        proc sql noprint;
           create table work.klogsp as 
              select "&lfile." as file length=500,
                max(datetime) as maxDT format=datetime23.3,
	           &_loadtm as _loadtm format=datetime23.3 label="Datetime stamp of when log was processed",
	           "&platform" as platform length=64,
	           "&context" as context length=64,
	           "&instance" as instance length=64                   
           from recordsperlog;
        quit;
        proc datasets lib=work nolist nowarn;
           append base=recordsperdir  data=recordsperlog;
           append base=klogspthisrun  data=klogsp;
        quit;
        run;

      %end; /* do loop */

    %let rc = %sysfunc(close(&dsid));
    %end; /* dataset open */

    /***************************************************************
     * If they asked for this invocation to append its output:
     * ==> Append the file produced above to the specified file.
     * ==> Delete the temporary file
     ****************************************************************/
     %let libname = %scan(&outds,1,'.');
     %let dsname = %scan(&outds,2,'.');
     %evcomment(NOTE: (&MNAME) Finished processing [&nobs] logs, Append flag [&append.],
Output libname [&libname] Output dsname [&dsname]);

     %if "&APPEND" EQ "YES" %then %do;
         proc datasets  lib=&libname nolist nowarn;
            append  base=&dsname  data=work.recordsperdir;
         run;
         quit;
	    proc datasets lib=work nolist;
	       modify KLOGSPthisrun;
	       index create file;
	    run;
	    quit;
	    data KITS.LOGSPROCESSED;
	       update KITS.LOGSPROCESSED work.KLOGSPthisrun;
	       by file;
	    run;
     %end;
     %else %if "&outflag" NE "DEFAULT"
     %then %do;
        proc datasets lib=&libname;
           delete &dsname.;
           copy in=work out=&libname;
              select recordsperdir;
              change recordsperdir=&dsname.;
           quit;
     %end;     
     
    /***************************************************************
	* registering table attributes
     ****************************************************************/
     %tableattr(libname=&libname,memname=&dsname,attrname=EVDMRPT,attrvalue=TRUE);
     %tableattr(libname=&libname,memname=&dsname,attrname=KIT2VA,attrvalue=TRUE);
    
    /* "ROLL-OFF" Data older than the data mart retention window */
     %jobheader(jobname=logparse_webserver,stepname=RollOffData);

     %evrolldata(data=&libname..&dsname.,numdays=&retainMart,dstype=EVDMRPT);
    /***************************************************************
    	* registering table with metadata
     ****************************************************************/
	proc metalib;
 		omr (liburi="SASLibrary?@libref='&libname'");        
 		select("&dsname");
	run;
	
     %if &evdebug LT 2 and "&outflag" NE "DEFAULT" %then %do;
        proc datasets nolist nowarn dd=work;
           delete recordsperlog recordsperdir klogspthisrun / memtype=data;
        run;
     %end;
     %else %evcomment((&MNAME) NOTE: Leaving work.recordsperlog and work.recordsperdir for diagnostics);
   
   %EXIT:
      proc datasets nolist nowarn dd=work;
         delete loglist2 klogsp / memtype=data;
      run;
%mend logparse_sasjobs;
