
/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

%macro metric_etl2(inputtsv=,                /* file to read */
                   firstobs=,                /* first record in input file to read */
                   lastrecread=,             /* name of macro var to hold last record read info */
                   delimiter="09"x,          /* field delimiter in file */
                   resource_category=metric, /* Resource category */
                   resourcetype=,         /* Resource type */
                   update_cas=Y,             /* Y|N flag to indicate whether CAS table should be updated */
                   caslib=,                  /* CASLIB of target CAS table */
                   casmemname=);             /* name of target CAS table */;

 /*+----------------------------------------------------------------+
  | Name:     metric_etl.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  parse daily metrics
  |           
  |
  | Inputs    dmpath
  |           date
  |
  | History:  Date    action
  |           12Mar17 Initial Coding                              frf
  |           25Apr17 Load tables to CAS                          frf
  |           04May17 Promote tables in CAS                       frf
  |           30MAY17 Revised; called new output_tsv macro     sasgzs
  | Future:
  | End 
  +----------------------------------------------------------------+*/


	%let records_read=0;
	
        %local version_full file_name file_date;   
        %parse_file_version(file=&inputtsv);

 	
	
	%tsv2sasds(file=&inputtsv,lastrecread=atobs,firstobs=&firstobs,out=&resourcetype,
				dstype=table,resource_category=&resource_category,
				resourcetype=&resourcetype,type=input);
				
	%ev_status(action=readMetricTransform);
	
	/*                                                                 */
	/* Return if there are zero observations in this table.            */
	/*                                                                 */
	
	%symdel nobs/nowarn;
	
	%if %sysfunc(exist(&resourcetype)) = 0 %then %return;
	
	data _null_;
		if 0 then set &resourcetype nobs=nobs;
		call symputx("nobs",nobs);
		stop;
	run;
		
	%if &nobs=0 %then %return;
	
	/*                                                                            */
	/* Extract fields of interest from SASEvent_Headers and drop vars not needed. */
	/*                                                                            */
	
	data &resourcetype;
	 set &resourcetype;
	 %get_map_item(map=sasevent_headers,key=__tenant,var=tenantid, varlength=25);
	 %get_map_item(map=payload_properties,key=hostname,var=machine, varlength=256); 
	 %get_map_item(map=payload_properties,key=consulnodename, varlength=256);
	 %get_map_item(map=sasevent_headers,key=sas-deployment-id,var=deploymentid, varlength=25);

         %add_extra_props(resourceType=&resourcetype);
	 
	 drop sasevent_version sasevent_id sasevent_type sasevent_timestamp
          sasevent_user sasevent_headers payload_version payload_collectorname
          payload_collectorversion payload_properties measurements_properties SASEvent_PayloadType;
          
	 rename measurements_resourcetype=resourcetype
	 	    measurements_resourceid=resourceid;
		
	run;
	
	%ev_status(action=readMetricTransform);
	
	
	/*                                                                 */
	/* Calculate rates. If this resourceType has no counter variables, */
	/* then no rates will be calculated and will return.               */
	/*                                                                 */
	
	 %calc_rates(indata=&resourcetype,file=&inputtsv);

	 
	 %ev_status(action=readMetricTransform);
                           	 
	/*                                                                 */
	/* Write TSV output. Column headings are in the first row and are  */
	/* obtained from sashelp.vcolumn for this resource.                */
	/*                                                                 */ 
	 
		 
	 %output_tsv(indata=&resourcetype,
	             dir=&evdmpath/&resource_category/&resourcetype,
	             file=&file_date.PROCESSED-&version_full..tsv,
	             version=&version_full,
	             appendifexist=Y);
     	 
	 %ev_status(action=readMetricTransform);
                           
    
	/*                                              */
	/* Load this table to CAS with the append mode. */	
	/*                                              */
	
	/*                                              */
	/* Signon to CAS, if UPDATE_CAS=Y               */
	/*                                              */
	
	
	%IF &update_cas ne Y %then %do;
		%put Note: CAS tables not updated due to parm update_cas ne Y ;
		%return;
	%end;
	
  %get_cas_session;

  %ev_status(action=casMetricConnect);
            
     /*                                                       */
     /* Format DATETIME and SHIFT before loading into CAS.    */
     /*                                                       */
    
    %format_vars_for_cas(table=&resourcetype,var=datetime,fmt=datetime22.);
    %format_vars_for_cas(table=&resourcetype,var=SHIFT,fmt=shifttxt.);
   
 	
  %appendSASTable(out_caslib=&caslib,
                  out_table=&resourcetype,
                  data=&resourcetype,
                  mode=append);

				
	%ev_status(action=casMetricLoad); 

	/*                                                                  */
	/* Promote the table. First, see if the    */	
	/* table is already promoted. If so, do not promote again.          */
	/*                                                                  */

	%symdel promoted_rc/nowarn;
	
	proc cas;
     %tableexists(caslib=&caslib,table=&resourcetype);
     call symput('promoted_rc',rc);
     run;
    quit;
	
	%ev_status(action=casMetricLoad);

		
	%if &promoted_rc=1 %then 
	%do;
	
      proc casutil incaslib=&caslib;
	 			   promote casdata="&resourcetype"
				   casout="&resourcetype"
				   outcaslib="&caslib";
	  quit;

			
	%ev_status(action=casMetricLoad);
			
		
   %end;
		
		
	/*                                            */
	/* Delete WORK.resourcetype just processed.   */
	/*                                            */
	
	%if %sysfunc(exist(work.&resourcetype)) =1 %then %do;
		proc datasets lib=work nolist;
			delete &resourcetype ;		
		 quit;	
		run;	
	%end;

	%EXIT:
	
%mend metric_etl2;







