/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/



 /*+----------------------------------------------------------------+
  | Name:     parse_header.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  Builds INPUT statements from headers.tsv
  |
  |
  | Inputs    headers.version.tsv
  |
  |
  | History:  Date    action
  |           05Apr17 Initial Coding                              snornb
  |           
  | Future:
  | End
  +----------------------------------------------------------------+*/
%macro parse_header(file=,out=,append=);

 %let EVDMERR=N;

 %local version_full version_major version_minor version_patch version_level;

 %parse_file_version(file=&file);

 %if "&EVDMERR" eq "Y" %then 
 %do; /* called macro return an error; exit */
    %put %str(ERROR: Call to PARSE_FILE_VERSION failed; exiting PARSE_HEADER macro);
    %goto EXIT_ERR;
 %end; /* called macro return an error; exit */

 filename tsvhdr "&file";		

 data columns(keep=version_major version_minor version_patch version_full resource_category resourcetype var var_type)
      metricproperties(keep=version_major version_minor version_patch version_full resource_category resourcetype  metricname metricvtype detailLevel unit metrictype);

  length line $4096;
  length resource_category $32.resourcetype $32. var var_type $200;
  length metricname metricvtype unit metrictype $200 detailLevel 8;
  length version_full $16;
 
  drop re1 re2 isarray line num_vars tab resourceInfo;
	 
  infile tsvhdr missover length=lv;
  input @1 line $varying4096. lv;

  retain version_major &version_major;
  retain version_minor &version_minor;
  retain version_patch &version_patch;
  retain version_full  "&version_full";
  
  num_vars=count(line,'(');
  tab='09'x;

  resourceInfo=scan(line,1,tab);
 
  if index(resourceInfo,'/') then
  do; /* both resource category & type provided */
     resource_category = scan(resourceInfo,1,'/');
     resourcetype = scan(resourceInfo,2,'/');
  end;/* both resource category & type provided */
  else 
  do; /* only resource category provided */
     resource_category = resourceInfo; 
     resourcetype = resource_category; 
  end;/* only resource category provided */
   
  line = substr(line,index(line,tab));	

  re1=prxparse('/(\w*)\((\w*)\)/');  /* non-metric var */

  /* metric var: 2nd version of re2 is more  */
  /* complex, b/c it supports:               */
  /*        embedded / for "per"             */
  /*        [] for arrays                    */
  /* User first version if that isn't needed */

  *re2=prxparse('/(\w*)\((\w*),(\d),(\w*),(\w*)\)/');  /* metric var */
  re2=prxparse('/(\w*)\((\[*?\]*?)(\w*),(\d),((\w*\/\w)|(\w*)),(\w*)\)/');

  re3=prxparse('/(\w*)\((.*)\)/');  /* all others var */
 
  do i=1 to num_vars;			
   
    raw_var = left(scan(line,i,tab));

    call missing(metricname, metricvtype, detailLevel, unit, metrictype, var, var_type);
    
    if prxmatch(re1,raw_var) then 
    do; /* parse non-metric variable */
   
       var=prxposn(re1,1,raw_var);
       var_type=prxposn(re1,2,raw_var);
       
    end;/* parse non-metric variable */
    else if prxmatch(re2,raw_var) then 
    do; /* parse metric variable */
   
       metricname=prxposn(re2,1,raw_var);
       isarray=prxposn(re2,2,raw_var);
       metricvtype=prxposn(re2,3,raw_var);
       detailLevel=input(prxposn(re2,4,raw_var),4.);
       unit=prxposn(re2,5,raw_var);
       metrictype=prxposn(re2,8,raw_var);   
       
       if isarray ne "" then metricvtype="array of "||metricvtype;
       
       var=metricname;
       var_type = metricvtype;          
       
       output metricproperties;  
       
    end;/* parse metric variable */
    else if prxmatch(re3,raw_var) then 
    do; /* parse non-metric variable */
   
       var=prxposn(re3,1,raw_var);
       var_type=prxposn(re3,2,raw_var);
       
    end;/* parse non-metric variable */
    else var="NO MATCH!"  ; 
    
	output columns;
  end;/* i-loop */
 run;

 data evdmversion;
  set columns (rename=(resourcetype=resource version_full=evdm_version));
  
  length lgth type $8. resource $32. evdm_version $16.;  
  
  type='input';
    
  select(lowcase(var));
   /* common elements */
   when('sasevent_version')   lgth='$8.';
   when('sasevent_id')        lgth='$36.';
   when('sasevent_type')      lgth='$16.';
   when('sasevent_timestamp') lgth='$32.';
   when('sasevent_user')      lgth='$8.';
   when('sasevent_headers')   lgth='$1000.';
   when('payload_version')    lgth='$8.';
   when('payload_properties') lgth='$1000.';
   
   /* metric payload */
   when('payload_collectorname')     lgth='$20.';
   when('payload_collectorversion')  lgth='$5.';
   when('payload_timestamp')         lgth='$32.';
   when('measurements_resourcetype')  lgth='$32.';
   when('measurements_resourceid')    lgth='$500.';
   when('measurements_properties')    lgth='$2000.';
   
   /* log message payload */
   when('payload_level')      lgth='$20.';
   when('payload_name')       lgth='$20.';
   when('payload_source')     lgth='$200.';
   when('payload_messageid')  lgth='$20.';
   when('payload_parameters') lgth='$1000.';
   when('payload_message')    lgth='$2000.';
   
   /* notification payload */
   when('payload_detailstype')    lgth='$30.';
   when('payload_detailsversion') lgth='$8.';
   when('payload_details')        lgth='$2000.';

   /* ledger payload */
   when('payload_starttime')      lgth='$32.';
   when('payload_duration')       lgth='$20.';
   when('payload_customer')       lgth='$200.';
   when('item_name')              lgth='$16.';
   when('item_description')       lgth='$32.';
   when('item_type')              lgth='$16.';
   when('item_source')            lgth='$32.';
   when('item_unit')              lgth='$12.';
   when('item_samples')           lgth='8.';
   when('item_obs')               lgth='8.';
   when('item_max')               lgth='8.';
   when('item_min')               lgth='8.';
   when('item_sum')               lgth='8.';
   when('item_median')            lgth='8.';

   /* application/resource payload */
   when('resource_action')         lgth='$32.'; 
   when('resource_actionstate')    lgth='$32.';  
   when('resource_uri_message')    lgth='$32.';
   when('resource_httpcontext')    lgth='$2000.'; 

   /* application/security payload */
   when('security_action')         lgth='$32.'; 
   when('security_actionstate')    lgth='$32.';  
   when('security_message')        lgth='$32.';
   when('security_properties')     lgth='$2000.'; 
   
   /* misc */
   when('memalarm')      lgth='$8.'; 
   when('diskfreealarm') lgth='$8.';
   when('running')       lgth='$8.';
   when('state')         lgth='$16.';
   when('connections')   lgth='$16.';
   when('channels')      lgth='$16.';
   
   otherwise
   do; /* look at var_type */
      select;
       when(lowcase((var_type) eq  'string'))     lgth="$200.";
       when(lowcase((var_type) eq  'bool'))       lgth="$8.";
       when(lowcase((var_type) eq: 'interface'))  lgth="$2000.";
       when(lowcase((var_type) eq: 'int'))        lgth="8.";
       when(lowcase((var_type) eq: 'float'))      lgth="8.";
       when(lowcase((var_type) eq: 'uint'))       lgth="8.";
       when(lowcase((var_type) eq: 'map['))       lgth="$2000.";
       
       otherwise lgth="$200.";
      end; /* select */
   end;/* look at var_type */
  end; /* select */
 run;

 %if %upcase("&append") eq "Y" %then
 %do; /* append to output data set */

    proc append base=&out data=evdmversion;
    run;

    proc append base=evdm.metricproperties data=metricproperties;
    run;    
 %end;/* append to output data set */ 

 %return;

 %EXIT_ERR:
  %let EVDMERR=Y;
  %return;
%mend parse_header;



