/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/


 /*+----------------------------------------------------------------+
  | Name:     parse_semver.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  Parses the specified string to extract the individual
  |           sub-components of a semantic version string. If the 
  |           specified string doesn't match the semantic version
  |           pattern, an ERROR message is generated.
  |           
  |           Calling programs need to define the macro variables
  |           which will be used to pass back the individual com-
  |           ponents; this can be done via %local or %global 
  |           statements.  This must be done carefully to ensure
  |           the macro variables are defined in the proper scope.
  |
  |           The following macro variables are used:
  |              VERSION_ALL       | The entire version string (should match the input string) 
  |              VERSION_FULL      | The full version string (omits any buildinfo)
  |              VERSION_MAJOR     | Major Version Number
  |              VERSION_MINOR     | Minor Version Number
  |              VERSION_PATCH     | Patch Number
  |              VERSION_LEVEL     | (optional) Release Level
  |              VERSION_BUILDINFO | (optional) Build Information
  |
  |           Several examples of valid input semantic version strings 
  |           and the values returned by this macro for each of these
  |           macro variables appears at the bottom of this file.
  |
  |
  |
  | History:  Date    action
  |           01JUN17 Initial Coding                         sasgzs
  | Future:
  | End
  +----------------------------------------------------------------+*/

%macro parse_semver(vstring=);

 %if "&vstring" ne "" %then
 %do; /* parse version string */

    %local version_pattern;
    %let version_pattern= (^((0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(-((0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(\.(0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?)(\+(([0-9a-zA-Z-]+)(\.[0-9a-zA-Z-]+)*))?);

    %let version_re=%sysfunc(prxparse(/&version_pattern/));
	%let match=%sysfunc(prxmatch(&version_re,&vstring));
   
    %if &match %then
    %do; /* matches version pattern */

       %let version_all      = %sysfunc(prxposn(&version_re,1,&vstring));
       %let version_full     = %sysfunc(prxposn(&version_re,2,&vstring));
       %let version_major    = %sysfunc(prxposn(&version_re,3,&vstring));
       %let version_minor    = %sysfunc(prxposn(&version_re,4,&vstring));
       %let version_patch    = %sysfunc(prxposn(&version_re,5,&vstring));
       %let version_level    = %sysfunc(prxposn(&version_re,7,&vstring));
	   %let version_buildinfo= %sysfunc(prxposn(&version_re,12,&vstring));       

    %end;/* matches version pattern */
    %else
    %do; /* doesn't match pattern */

       %put %str(ERROR: The specified version [&vstring] does NOT match the expected version pattern.);
       %goto EXITERR;

    %end;/* doesn't match pattern */

 %end;/* parse version string */
 %else
 %do; /* nothing to parse */

    %put %str(ERROR: No version string was passed; nothing to do);
    %goto EXITERR;

 %end;/* nothing to parse */

 %if &evdebug ge 2 %then
 %do; /* dump out sub-components extracted from version */
    %put _local_;
 %end;/* dump out sub-components extracted from version */

 %return;  /* all is well */

%EXITERR:
 %let evdmerr=Y;
 %return;

%mend parse_semver;


/*********************************************************************/
/* START        ****** TEST CASES & RESULTS  ******  

     +-------------------------------------------------------+
     | %parse_semver(vstring=1.4.1);                         |
     +-------------------+-----------------------------------+
     | VERSION_ALL       | 1.4.1                             |
     | VERSION_FULL      | 1.4.1                             |
     | VERSION_MAJOR     | 1                                 |
     | VERSION_MINOR     | 4                                 |
     | VERSION_PATCH     | 1                                 |
     | VERSION_LEVEL     |                                   |
     | VERSION_BUILDINFO |                                   |
     +-------------------------------------------------------+
     | %parse_semver(vstring=1.3.10-M.1+01MAR2017.john.doe); |
     +-------------------+-----------------------------------+
     | VERSION_ALL       | 1.3.10-M.1+01MAR2017.john.doe     |
     | VERSION_FULL      | 1.3.10-M.1                        |
     | VERSION_MAJOR     | 1                                 |
     | VERSION_MINOR     | 3                                 |
     | VERSION_PATCH     | 10                                |
     | VERSION_LEVEL     | M.1                               |
     | VERSION_BUILDINFO | 01MAR2017.john.doe                |
     +-------------------------------------------------------+
     | %parse_semver(vstring=1.3.10-M.1+01FEB2017);          |
     +-------------------+-----------------------------------+
     | VERSION_ALL       | 1.3.10-M.1+01FEB2017              |
     | VERSION_FULL      | 1.3.10-M.1                        |
     | VERSION_MAJOR     | 1                                 |
     | VERSION_MINOR     | 3                                 |
     | VERSION_PATCH     | 10                                |
     | VERSION_LEVEL     | M.1                               |
     | VERSION_BUILDINFO | 1-Feb-17                          |
     +-------------------------------------------------------+
     | %parse_semver(vstring=1.3.10-M.1+BOBSMACHINE);        | 
     +-------------------+-----------------------------------+
     | VERSION_ALL       | 1.3.10-M.1+BOBSMACHINE            |
     | VERSION_FULL      | 1.3.10-M.1                        |
     | VERSION_MAJOR     | 1                                 |
     | VERSION_MINOR     | 3                                 |
     | VERSION_PATCH     | 10                                |
     | VERSION_LEVEL     | M.1                               |
     | VERSION_BUILDINFO | BOBSMACHINE                       |
     +-------------------------------------------------------+
     | %parse_semver(vstring=1.3.10-M.1);                    |
     +-------------------+-----------------------------------+
     | VERSION_ALL       | 1.3.10-M.1                        |
     | VERSION_FULL      | 1.3.10-M.1                        |
     | VERSION_MAJOR     | 1                                 |
     | VERSION_MINOR     | 3                                 |
     | VERSION_PATCH     | 10                                |
     | VERSION_LEVEL     | M.1                               |
     | VERSION_BUILDINFO |                                   | 
     +-------------------------------------------------------+
     | %parse_semver(vstring=1.3.10-SNAPSHOT);               |
     +-------------------+-----------------------------------+
     | VERSION_ALL       | 1.3.10-SNAPSHOT                   |
     | VERSION_FULL      | 1.3.10-SNAPSHOT                   |
     | VERSION_MAJOR     | 1                                 |
     | VERSION_MINOR     | 3                                 |
     | VERSION_PATCH     | 10                                |
     | VERSION_LEVEL     | SNAPSHOT                          |
     | VERSION_BUILDINFO |                                   |
     +-------------------------------------------------------+

                ****** TEST CASES & RESULTS  ******              END */
/*********************************************************************/


