/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/



 /*+----------------------------------------------------------------+
  | Name:     rolloff.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  Reloads all data for the defined retention window.
  |           This is "functionally equivalent" to  rolling off "old"
  |           data.
  |
  | Inputs    Raw TSVs (log events) & PROCESSED TSVs (metric events)
  |
  |
  | History:  Date    action
  |           02OCT17 Code pulled from rolloff job to create macro  sasgzs
  |
  | Future:
  | End
  +----------------------------------------------------------------+*/


%macro rolloff;

 %if %isTrue(&emi_cas_load) %then
 %do; /* cas loading is enabled */
 
   /* ATTEMPT TO CONNECT TO CAS;       */
   /* ABORT IF CONNECTION FAILS.       */
   %get_cas_session(abortonfail=Y);

   %ev_status(action=casRolloffConnect);

   /*                                            */
   /* Build user formats into userformats1 in CAS*/
   /*                                            */

   %create_cas_formats;


   /**** ROLL OFF ***/

   %let end=%sysfunc(tzones2u(%sysfunc(datetime())));
   %let start=%sysfunc(intnx(dtDay, &end , %eval(-1*&cas_cutoff)));

   %put %str(DEBUG: Rolloff Processing for period starting [&start] and ending [&end].);

   %get_cas_session();


   /* ************** */
   /* **** LOGS **** */
   /* ************** */


   %if %isTrue(&emi_cas_load_logs) %then
   %do; /* load logs into CAS  */


        /* (re-)initialized the target CAS Search Index.  */         
       proc cas;  
        %initIndex(caslib=&CAS_SEARCH_CASLIB,
                   indexname=&CAS_SEARCH_IDXNAME,
                   schemafile=&schema_file);
        run;
       quit;   


       /* Load documents into target CAS Search Index.    */
       %adhoc_load(start_datetime=&start,
                   end_datetime=&end,
                   load_mname=logs_etl,
                   data_dir=&evdmpath/log/log,
                   resource_category=log,     
                   resourcetype=log,          
                   update_inventory=Y,
                   add_new_tsv_to_inventory=Y,
                   update_cas=Y,
                   promote=N,
                   dropfirst=N,            
                   namematchrx=%str(.*Z-.*\.tsv),             
                   caslib=&CAS_SEARCH_CASLIB,    
                   casmemname=&CAS_SEARCH_IDXNAME); 

    
   %end;/* load logs into CAS  */
   %else
   %do; /* cas logs loading is disabled */

      %put %str(NOTE: Loading of LOG data into CAS is currently disabled by global setting (EMI_CAS_LOAD_LOGS).);

   %end;/* cas logs loading is disabled */



   /* ***************** */
   /* **** METRICS **** */
   /* ***************** */
   
   %if %isTrue(&emi_cas_load_metrics) %then
   %do; /* load metrics into CAS  */

   
       filename roffcode temp;
  
       data _null_;
        length line $4000;
        set evdm.whitelist;
        where resource_category eq "metric";
 
        data_dir=catx('/', "&evdmpath", resource_category, resourcetype);

        file roffcode;
  
        line=catt('%adhoc_load(start_datetime=',&start,
                               ',end_datetime=',&end,
                               ',load_mname=metric_load_output',
                               ',data_dir=', data_dir,
                               ',resource_category=metric',     
                               ',resourcetype=', resourcetype,          
                               ',update_inventory=Y',
                               ',add_new_tsv_to_inventory=Y',
                               ',update_cas=Y',
                               ',promote=Y',
                               ',dropfirst=Y',
                               ',namematchrx=%str(.*PROCESSED.*\.tsv)',             
                               ',caslib=',"&caslib_metric_data",
                               ',casmemname=',metrictablename,
                               ');'); 
        put line;

       run;

       %include roffcode /source2;

       %ev_status(action=casRolloffMetricLoad);
 
   %end;/* load metrics into CAS  */
   %else
   %do; /* cas metrics loading is disabled */

      %put %str(NOTE: Loading of METRIC data into CAS is currently disabled by global setting (EMI_CAS_LOAD_METRICS).);

   %end;/* cas metrics loading is disabled */

   
   %exit_cas_session();

   
 %end;/* cas loading is enabled */
 %else
 %do; /* cas loading is disabled */
 
    %put %str(NOTE: Loading of data into CAS is currently disabled by global setting (EMI_CAS_LOAD); nothing to do, exiting);
    
 %end;/* cas loading is disabled */

%mend rolloff;
