/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

 /*+----------------------------------------------------------------+
   | Name:     tsv2sasds.sas                                        |
   | Product:  SAS Environment Manager                              |
   |                                                                |
   | Purpose:  Reads a TSV file and create a SAS data set from its  |
   |           contents.  Optionally, only read from a particular   |
   |           record in the input file.                            |
   |                                                                |
   | History:  Date    action                                       |
   |           17MAR17 Initial Coding                        sasgzs |
   |           23MAY17 Added support for ZIP files           sasgzs |
   |           20JUN17 Added support for column headings in         |
   |                   raw TSV files.                        frefor |
   |                                                                |
   +----------------------------------------------------------------+*/

%macro tsv2sasds(file=,             /* input TSV file to read */
                 type=,             /* input|output; type of data contained in TSV */
                 resource_category=,/* resource category of data in TSV */
                 resourcetype=,     /* resourcetype of data in TSV */
                 where=,            /* 'WHERE' clause; implemented as subsetting IF */
                 out=,              /* name of target table/view */
                 start_dt=,         /* start of period of data to keep */
                 end_dt=,           /* end of period of data to keep */
                 dstype=view,       /* Type of SAS data set: VIEW|TABLE */
                 firstobs=,         /* first record in input file to read */
                 lastrecread=,      /* name of macro var to hold last record read info */
                 delimiter="09"x);  /* field delimiter in file */

   %let EVDMERR=N;
	/*                                                                   */
	/* Skip the first row of a raw TSV file since it has column headings */
	/*                                                                   */
	
	%let firstobs=%sysfunc(max(&firstobs,2));
	
*   filename zipdtsv clear;      

   data _null_;
    /* clear fileref if it was previously assigned */
    rc=filename(zipdtsv,' ');
   run;

   %local viewstr wherestr;
   
   %if %upcase("&dstype") eq "VIEW" %then
   %do; /* data view requested */
      %let viewstr=%str(/view=&out);
   %end;/* data view requested */


   %if "&where" ne  "" %then
   %do; /* where clause supplied */
      %let wherestr=%str(if &where;);
   %end;/* where clause supplied */


   %let &lastrecread=&firstobs;

   %local version_full file_zip file_name;   
   %parse_file_version(file=&file);
   
   /* cache tsv file info; if   */
   /* header file is read while */
   /* executing %GET_INPUT_STMT */
   /* these values will be lost */ 
   %let tsv_file_zip=&file_zip;
   %let tsv_file_name=&file_name;


   %get_input_stmt(type=&type, version=&version_full, resource_category=&resource_category, resourcetype=&resourcetype);


   %if "&EVDMERR" eq "Y" %then
   %do; /* called macro return an error; exit */
      %put %str(ERROR: No appropriate INPUT STATEMENT could be generated for file [&file]; skipping this file.);
      %goto EXIT_ERR;
   %end; /* called macro return an error; exit */

   %if "&tsv_file_zip" eq "" %then
   %do; /* not a zip */

      %let infile_str="&file"  ;
      
   %end;/* not a zip */
   %else
   %do; /* is a zip */
  
      /*accessing ZIP file requires use of fileref */
      /* this means the view is only usable if the */
      /* fileref is assigned and assigned to the   */
      /* correct file.                             */
      filename zipdtsv zip "&file" member="&tsv_file_name";
      %let infile_str=zipdtsv ;
      
   %end; /* is a zip */

   %local range_start range_end;
   %if "&start_dt" ne "" %then %let range_start=%str(if datetime ge &start_dt;);
   %if "&end_dt" ne "" %then %let range_end=%str(if datetime lt &end_dt;);

   data &out &viewstr;
    length &length_stmt; 
   /* format datetime datetime22.; */
    infile &infile_str firstobs=&firstobs. dsd dlm=&delimiter end=done MISSOVER;
    input &input_stmt;
    
    %if "&tsv_file_zip" ne "" and "&viewstr" ne "" %then
    %do; /* write out name of zip file */
   
       if _n_ eq 1 then put "NOTE: The view [&out] requires fileref ZIP2TSV be assigned to ZIP file [&file].";
    
    %end;/* write out name of zip file */
    
    %if "&type" eq "input" %then
    %do; /* need to create datetime-related vars */

       %if %index(%upcase(&input_stmt),%upcase(payload_timestamp)) EQ 0 %then
       %do;  /* no payload_timestamp field in this TSV */
          datetime=input(SASEvent_TimeStamp,is8601dz35.);
          drop SASEvent_TimeStamp;
       %end;/* no payload_timestamp field in this TSV */
       %else
       %do; /* base datetime on payload timestamp */
          datetime=input(payload_timestamp,is8601dz35.);
          drop payload_timestamp;
       %end;/* base datetime on payload timestamp */ 
       
       if datetime eq . then tzoneoff=.;     
       else tzoneoff =tzoneoff(datetime);

    %end;/* need to create datetime-related vars */

    /* SHIFT is already in the TSV, but there as */
    /* "formatted" value, so it appears to be    */
    /* text and is not read properly; re-calcu-  */
    /* lating here.  When fixed, this macro      */
    /* call should be  move inside the preceding */
    /* %do loop.                                 */
    /*                                           */
    /* ALSO: assigned BEST format to SHIFT var   */
    /*       until we decide on how to handle    */
    /*       our user-defined formats in CAS.    */
    /*                                           */
    %addshiftvar(datetime=datetime, offsetvar=tzoneoff,  outvar=shift, shiftfmt=best, form=DATASTEP);

    rec_count+1;   /* count of records read from TSV */

    &wherestr;
    &range_start;
    &range_end;
    	 
    matching_recs+1;
    
    if done then 
    do; /* last record read */
    
       put "INFO: [" rec_count "] records read from file [&file]";
       call symput("&lastrecread",sum(rec_count,&firstobs));

       call symput("tsv_rec_count",matching_recs);

       %if "&wherestr.&start_dt.&end_dt" ne "" %then
       %do; /* write message showing count of matches */
          put "INFO: [" matching_recs "] records read from file [&file] match the specified WHERE clause or DATETIME range. ";
       %end;/* write message showing count of matches */
       
    end;/* last record read */
   
    drop rec_count matching_recs;

   run;

   %return;

   %EXIT_ERR:
   %return;

%mend tsv2sasds;
