/***************************************************/
/* Copyright (c) 2017 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/



 /*+----------------------------------------------------------------+
  | Name:     ziptsv.sas
  | Product:  SAS Environment Manager
  |
  | Purpose:  1. zip input and PROCESSED TSV file
  |           2. Delete zipped input TSV files
  |
  | Inputs    whitelist
  |           
  |
  | History:  Date    action
  |           05Jul17 Initial Coding                              frf  
  |           08Aug17 Added logic to delete zip files older than
  |                   macro variable DELETE_OLD_ZIPS
  | Future:
  | End 
  +----------------------------------------------------------------+*/
 
%macro ziptsv(cutoff=3);

data _null_;
	set evdm.whitelist;
	inpath=compress("&evdmpath" || '/' || resource_category || '/' || resourcetype) ;
		call symput('dir' || left(trim(_n_)),inpath);
		call symput('dirs_to_zip',left(trim(_n_)));
run;

	%do i=1 %to &dirs_to_zip;

	 %put i=&i dir=%cmpres(&&dir&i);
	 %symdel nobs files_to_zip/nowarn;
		%if %sysfunc(exist(_foundfiles)) %then %do;
	 		proc datasets lib=work nolist;delete _foundfiles;quit;run;
		%end;

		%if %sysfunc(exist(allzips)) %then %do;
	 		proc datasets lib=work nolist;delete allzips;quit;run;
		%end;

		%if %sysfunc(fileexist("&&dir&i")) %then %do;
			%put Searching for eligible files to zip in directory %cmpres(&&dir&i);

	/*                                                      */
	/* Delete any zip file older than DELETE_OLD_ZIPS       */
	/*                                                      */

	%delete_files(dir=%cmpres(&&dir&i),
				  cutoff=&delete_old_zips,
				  extension=zip);
				  
				  
	%ev_status(action=deleteOldZips);			  

	/*                                                     */
	/* Find all TSV files in this existing directory from  */
	/* the whitelist.                                      */
	/*                                                     */

			%findfiles(dir=%cmpres(&&dir&i), 
        	     outds=work._foundfiles, 
          	   namematchrx=.*-.*\.tsv );
			
			   %let nobs=0;

			    data _null_;
                  if 0 then set _foundfiles nobs=nobs;
                  call symputx("nobs",nobs);
                  stop;
                run;

	/*                                                     */
	/* If there are TSV files in this dir, then continue.  */	
	/* Make list of TSVs to zip that are older than the    */
	/* cutoff date (3 days default).                       */
	/*                                                     */

				%if &nobs gt 0 %then %do;
				 %let files_to_zip=0;
					data _null_;set _foundfiles;
					 if index(lowcase(logfile),'.zip') ne 0 then return;
					 d=today();
										 
						if today() - datepart(modfdt) lt &cutoff then return;
						files_to_zip+1;
						
						call symput('inpath' || left(trim(files_to_zip)),compress(directory));
						call symput('tsvfile' || left(trim(files_to_zip)),compress(logfile));
						call symput('files_to_zip',files_to_zip);
				     run;
	/*                                                         */
	/* If there are no eligible files to zip in this directory */
	/* then skip this directory.                               */
	/*                                                         */
						%put files_to_zip=&files_to_zip dir=%cmpres(&&dir&i) ;
					 %if &files_to_zip=0 %then %goto no_zip ;

	/*                                                     */
	/* ZIP each eligible TSV file and delete it if all is  */
	/* OK.                                                 */
	/*                                                     */

					%do k=1 %to &files_to_zip;
						%put Now zipping [%cmpres(&&dir&i)] file=%cmpres(&&tsvfile&k) ;
						%tsv2zip(inpath=%cmpres(&&dir&i),tsvfile=%cmpres(&&tsvfile&k),outpath=%cmpres(&&dir&i));
							%let syscc=0;
							%let sysrc=0;
							
						%ev_status(action=zipTSV);	
							
					%end; /* zipping files in this dir */


						/*                                                      */
						/* Create dataset with files just zipped in this dir.   */
						/* Eliminate PROCESSED zipped files since they do not   */
						/* interact with the inventory.                         */
						/*                                                      */

						data new_zips;
							length tsv_file_name     $200. ;
								%do j=1 %to &files_to_zip;
									tsv_file_name=compress("&&tsvfile&j" || '.zip');
									if index(upcase(tsv_file_name),'PROCESSED') = 0 then output;
								%end;
						run;

						/*                                                      */
                        /* If only PROCESSED files were zipped, then new_zips   */
						/* will be empty and there will be no need to update the*/
						/* inventory.                                           */
						/*                                                      */
	
						%let nobs=0;

			    		data _null_;
                  			if 0 then set new_zips nobs=nobs;
                 			 call symputx("nobs",nobs);
                  			stop;
                		run;
						
						%if &files_to_zip=0 %then %goto no_zip ;	
											
						/*                                                      */
						/* Get modfdt of all zip files in this directory,       */
						/* including those just zipped.                         */
						/*                                                      */

					    %findfiles(dir=%cmpres(&&dir&i), 
        	     					outds=work.allzips, 
          	   						namematchrx=.*Z-.*\.zip );

						/*                                                      */
						/* Need to get modfdt of all files just zipped in this  */
						/* directory.                                           */
						/*                                                      */

						proc sort data=allzips(keep=modfdt logfile rename=(logfile=tsv_file_name));
							by tsv_file_name;
						run;

						proc sort data=new_zips;by tsv_file_name;
						run;
						data new_zips;
							merge allzips(in=in1) new_zips(in=in2);
							by tsv_file_name;
							if in1 & in2 then do;
								tsv_file_name=substr(tsv_file_name,1,length(tsv_file_name)-4);
								output;
							end;
						run;
					
						libname thisinv "&&dir&i";
												
						/*                                                            */
						/* Merge the inventory with the new_zips to get modfdt of the */
						/* newly zipped TSV file.                                     */
						/*                                                            */
					%if %sysfunc(exist(thisinv.inventory)) %then %do;	
						data thisinv.inventory;
							merge thisinv.inventory(in=in1) new_zips(in=in2);
								if in1 & in2 then do;
									finalized='Y';
									file_ext='zip';
									last_moddate=modfdt;
									tsv_file_name=compress(tsv_file_name || '.zip');
								end;
								drop modfdt;
							by tsv_file_name;
						run;
						
						%ev_status(action=updateInventory);
						
					%end; /* thisinv.inventory exists */
						libname thisinv clear;
					%no_zip: /* No eligible files to zip */
				%end; /* nobs > 0 */       
			
		%end; /* dir exists */
	%end; /* dirs_to_zip */
%mend ziptsv;


