#!/bin/bash
SCRIPT_NAME="CAS Batch Load Datasets"
SCRIPT_VERSION="1.3.3"
SCRIPT_BUILD_ID="201703171704"

MYPATH=$(dirname $0)
MYNAME=$(basename $0)

# set up some defaults

rootdir=/opt/sas
deployid=viya
instance=default
input_data=$rootdir/$deployid/config/var/lib/evmsvrops/ETL/persistent
logdir=$rootdir/$deployid/config/var/log/evmsvrops

# count number of whitespace-separated elements in a string/var
# warning, pattern/glob characters will yield unexpected results

howmany() { echo $#; }

usage()
{
  echo 
  echo "Upload all SAS bdat datasets from the given input dir into a specified CAS host."
  echo "Must be run as the SAS installer UID, requires a predefined .sasiom_authinfo file"
  echo "in the SAS installer's home directory."
  echo
  echo "$MYNAME [OPTION]... <cas_hostname>"
  echo
  echo "  -r rootdir   (default '/opt/sas')"
  echo "  -d deployid  (default 'viya')"
  echo "  -i instance  (default 'default')"
  echo "  -l logdir    (default '$rootdir/$deployid/config/var/log/evmsvrops')"
  echo "  --input dir  (default '$rootdir/$deployid/config/var/lib/evmsvrops/ETL/persistent)"
  echo
}

MYPATH=$(dirname $0)
prodname=$(basename $MYPATH | cut -d\. -f1)

excessargs=""

while [ -n "$1" ]
do
  case "$1" in

    -r)
      shift
      rootdir="$1"
      ;;

    -l)
      shift
      logdir="$1"
      ;;

    -d)
      shift
      deployid="$1"
      ;;

    -i)
      shift
      instance="$1"
      ;;

    --input) 
      shift
      input_data="$1"
      ;;

    -h | -help | --help)
      usage;
      exit 1
      ;;

    -v | -version | --version)
      echo "$SCRIPT_NAME"
      echo "$SCRIPT_VERSION"
      echo "$SCRIPT_BUILD_ID"
      exit 0
      ;;

    -*)
      echo
      echo "unexpected option: $1"
      echo
      usage;
      exit 1
      ;;
     
     *)
# if we don't recognize argument as expected option, accumulate the excess
# if only one exists after completing options processing, assume that it's the cashost
      excessargs="${excessargs} $1"
  esac
  shift  # next value
done

# Sanity check for required parameters

if [ $(howmany "$excessargs") -ne 1 ]; then
  echo "Expected a CAS hostname. Got this instead \"$excessargs\""
  exit 1
fi

cashost="$excessargs"

SASINSTALLUSER=$(find $rootdir/$deployid -maxdepth 0 -printf '%u')

AUTHINFO=~/.sasiom_authinfo   # careful, the tilde will not expand within quotes

if [ "$USER" != "$SASINSTALLUSER" ]; then
  echo "This script must be run as the SAS install user to be able to authenticate to"
  echo "the CAS server. The code assumes that an appropriate ~/.sasiom_authinfo file"
  echo "exists with credentials to connect to the specified CAS host."
  echo
  echo "Derived SAS install user is $SASINSTALLUSER"
  exit 1
fi

if [ ! -r "$AUTHINFO" ]; then
  echo "This script must be run as the SAS install user to be able to authenticate to"
  echo "the CAS server. The code assumes that an appropriate ~/.sasiom_authinfo file"
  echo "exists with credentials to connect to the specified CAS host."
  echo
  echo "No readable ~/.sasiom_authinfo file was found."
  exit 1
fi


# the sas executable
SAS_BIN_EXE=${rootdir}/home/SASFoundation/sas

echo
echo "Datasets will be read from $input_data"
echo "Logs are written to $logdir"
echo

export SAS_BIN_EXE=/opt/sas/viya/home/SASFoundation/sas

DATASETS="${input_data}/*.sas7bdat"
echo DATASETS is $DATASETS

for i in $DATASETS
do
  NAME=$(basename ${i%.*})
echo i is $i
echo NAME is $NAME
  echo "Invoking $SAS_BIN_EXE"
INVOCATION="$SAS_BIN_EXE -set inputdatapath $input_data \
    -set libpath $MYPATH \
    -set logpath . \
    -set mydata $NAME \
    -set cashost "$cashost" \
    $MYPATH/SASCode/casload.sas"

echo $INVOCATION
  $INVOCATION
  rc=$?
  if [ $rc -gt 1 ]; then
    echo "SAS upload attempt against $input_data/$NAME failed, rc=$rc"
    echo "exiting..."
    exit 1
  else
    echo "Uploaded $input_data/$NAME, rc=$rc"
  fi
done

echo
echo "Done."

